/*
 * catalog/awt/AnimatedCatalogDialog.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.awt;

import catalog.customer.Customer;
import catalog.net.*;

import java.awt.*;
import java.awt.event.*;
import java.net.ConnectException;
import java.io.IOException;

import javax.swing.*;

import smi.io.*;
import smi.mechanisms.ConflictException;
import smi.mechanisms.NoCommonMechanismException;
import smi.mechanisms.ParameterNotCompatibleException;

import ssonet.*;

/**
* A window with an animated catalog during the order of a catalog
*/
public class AnimatedCatalogDialog extends Thread implements ActionListener
{
    private Customer customer;
    private JDialog dialog;
    private JButton button;
    String merchantHost;
    int merchantHostPort;
    JFrame parentFrame;

    /**
    * Creates a window with an animated icon and assigns values to variables.
    */
    public AnimatedCatalogDialog(JFrame parentFrame,
	Customer customer,
	String merchantHost,
	int merchantHostPort)
    {
        dialog = new JDialog(parentFrame,"Katalog anfordern", false);
        this.customer = customer;
        this.parentFrame = parentFrame;
        this.merchantHost = merchantHost;
        this.merchantHostPort = merchantHostPort;
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        dialog.getContentPane().setLayout(gridBagLayout);
        gridBagConstraints.insets.top = 10;
        gridBagConstraints.insets.right = 10;
        gridBagConstraints.insets.left = 10;
        gridBagConstraints.insets.bottom = 10;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.ipadx = 0;
        gridBagConstraints.weightx = 0;
        Icon icon = new ImageIcon(getClass().getResource("images/catalog.gif"));
        FixedSizeJLabel fixedSizeJLabel = new FixedSizeJLabel(50,50);
        gridBagLayout.setConstraints(fixedSizeJLabel, gridBagConstraints);
        fixedSizeJLabel.setIcon(icon);
        dialog.getContentPane().add(fixedSizeJLabel);
        JLabel label = new JLabel("Lade Katalog.");
        label.setFont(new Font("Dialog", Font.BOLD,12));
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagLayout.setConstraints(label, gridBagConstraints);
        dialog.getContentPane().add(label);
        button = new JButton("Abbrechen");
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.weightx = 1;
        gridBagLayout.setConstraints(button, gridBagConstraints);
        dialog.getContentPane().add(button);
        button.addActionListener(this);
        dialog.pack();
        dialog.setResizable(false);
        Point parentLocation = customer.getLocationOnScreen();
        Dimension parentDimension = customer.getSize();
        Dimension ownDimension = dialog.getSize();
        int x = 0;
        int y = 0;
        x = parentLocation.x + (parentDimension.width - ownDimension.width) / 2;
        y = parentLocation.y + (parentDimension.height - ownDimension.height) / 2;
        dialog.setLocation(x, y);
        start();
    }

    /**
    * creates a new SSONETConnection to the merchant and gets the catalog
    */
    public void run() {
        SSONETClientConnection ssonetClientConnection = null;
        try {
            dialog.show();
            boolean returnValue = false;
            ssonetClientConnection = new SSONETClientConnection(customer.name,
		OrderPanel.CATALOG,
		parentFrame,
		merchantHost,
		merchantHostPort);
            IOStream ioStream = ssonetClientConnection.connect();
            Object object = ioStream.readObject(); /* read answer*/
            if (object instanceof CatalogFile)
                returnValue = customer.getCatalog((CatalogFile)object);
            else {
                System.out.println("falsche Antwort vom Hndler (kein Katalog erhalten)");
                throw new NullPointerException();
            }
            ioStream.close();
            ssonetClientConnection.close();
            dialog.dispose();
	    JOptionPane.showMessageDialog(null,"Katalog erfolgreich erhalten.","Information", JOptionPane.INFORMATION_MESSAGE);
        } catch (ParameterNotCompatibleException pe) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
	    JOptionPane.showMessageDialog(null,"Folgender Konflikt trat auf:\n"+pe.getMessage(),"Information", JOptionPane.ERROR_MESSAGE);
        } catch (NoCommonMechanismException me) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
	    JOptionPane.showMessageDialog(null,"Folgender Konflikt trat auf:\n"+me.getMessage(),"Information", JOptionPane.ERROR_MESSAGE);
        } catch(ConnectException ce) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
            JOptionPane.showMessageDialog(null,"Konnte Hndler nicht erreichen","Fehler", JOptionPane.ERROR_MESSAGE);
            System.out.println("AnimatedCatalogDialog.run() - Hndler nicht erreichbar.");
        } catch (ConflictException ce) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
	    JOptionPane.showMessageDialog(null,"Folgender Konflikt trat auf:\n"+ce.getMessage(),"Information", JOptionPane.ERROR_MESSAGE);
        } catch(SSONETCloseConnectionException se) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
            JOptionPane.showMessageDialog(null,"Abbruch auf Benutzerwunsch","Information", JOptionPane.ERROR_MESSAGE);
        } catch (IOException ie) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
            JOptionPane.showMessageDialog(null,"Fehler: Konnte Auftrag nicht bertragen.","Fehler", JOptionPane.ERROR_MESSAGE);
        } catch (NullPointerException ne) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
            JOptionPane.showMessageDialog(null,"Fehler: Konnte Konfiguration nicht lesen.","Fehler", JOptionPane.ERROR_MESSAGE);
        } catch (SSONETCertificateNotValidException cnve) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
            JOptionPane.showMessageDialog(null,"Das Zertifikat wurde abgelehnt.","Information", JOptionPane.ERROR_MESSAGE);
        } catch(Exception e) {
            if (ssonetClientConnection != null)
                ssonetClientConnection.close();
            dialog.dispose();
            JOptionPane.showMessageDialog(null,
		"Unbekannter Fehler beim Versenden\ndes Kataloges",
		"Fehler",
		JOptionPane.ERROR_MESSAGE);
            e.printStackTrace();
        }
    }

    public void actionPerformed(ActionEvent event) {
        Object object = event.getSource();
        if (object == button) dialog.dispose();
    }
}
