/*
 * catalog/awt/OrderSumPanel.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.awt;

import javax.swing.JPanel;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import java.awt.*;
import java.text.NumberFormat;

/**
* A panel which displays the sum of an order
*/
public class OrderSumPanel extends JPanel {
    private Label sumLabel, withTaxes, taxLabel;
    private String currencyString;
    private int valueAddedTax;
    private NumberFormat numberFormat;

    /**
    * Erzeugt das Panel, welches die Summe einer Bestellung anzeigt
    */
    public OrderSumPanel() {
	numberFormat = NumberFormat.getInstance();
	numberFormat.setMaximumFractionDigits(2);
	numberFormat.setMinimumFractionDigits(2);
	currencyString = "";
	valueAddedTax = 0;
	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	this.setLayout(gridBagLayout);
	this.setBorder(new TitledBorder(new EtchedBorder()," Bestellsumme "));
	this.setDoubleBuffered(true);
	Label l = new Label("Bestellwert ohne Mehrwertsteuer:");

	gridBagConstraints.insets.top = 15;
	gridBagConstraints.insets.left = 10;
	gridBagConstraints.insets.right = 10;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.weightx = 1;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.fill = GridBagConstraints.HORIZONTAL;
	gridBagLayout.setConstraints(l, gridBagConstraints);
	this.add(l, gridBagConstraints);

	Font f = new Font("DIALOG", Font.BOLD,18);
	sumLabel = new Label();
	sumLabel.setAlignment(Label.RIGHT);
	sumLabel.setFont(f);
	gridBagConstraints.insets.top = 10;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 1;
	gridBagConstraints.anchor = GridBagConstraints.NORTHEAST;
	gridBagLayout.setConstraints(sumLabel, gridBagConstraints);
	this.add(sumLabel);

	taxLabel = new Label();
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 2;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagLayout.setConstraints(taxLabel, gridBagConstraints);
	this.add(taxLabel);

	withTaxes = new java.awt.Label();
	withTaxes.setAlignment(Label.RIGHT);
	withTaxes.setFont(f);
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 3;
	gridBagConstraints.insets.bottom = 15;
	gridBagConstraints.weighty = 1;
	gridBagConstraints.anchor = GridBagConstraints.NORTHEAST;
	gridBagLayout.setConstraints(withTaxes, gridBagConstraints);
	this.add(withTaxes);
    }

    /**
    * sets the value added tax to taxRate
    */
    public void setTaxRate(int taxRate) {
	valueAddedTax = taxRate;
	taxLabel.setText("Bestellwert mit " + valueAddedTax + "% Mehrwertsteuer:");
    }

    /**
    * sets the currencyString
    */
    public void setCurrency(String currency) {
	currencyString = currency;
    }

    /**
    * calculates the order sum with and without value added tax
    */
    public void setSum(float ml)
    {
	float summe = ml;
	float steuer = summe/(float)(1.0+(float)valueAddedTax/100);
	sumLabel.setText(numberFormat.format(steuer)+" "+currencyString);
	withTaxes.setText(numberFormat.format(summe)+" "+currencyString);
    }

    /**
    * returns order sum with value added tax
    */
    public String getSum() {
	return withTaxes.getText();
    }
}
