/*
 * catalog/merchant/MerchantServer.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.merchant;

import catalog.merchant.Merchant;
import catalog.net.Order;
import catalog.net.CatalogFile;
import catalog.net.Connection;

import java.io.*;

import java.net.*;

import ssonet.SSONETServerConnection;
import ssonet.SSONETStream;

/**
* A MerchantServer
*/
public class MerchantServer extends Thread {
    private CatalogFile catalogFile;
    private SSONETServerConnection ssonetServerConnection;
    private String merchantName;
    private Merchant merchant;
    boolean serverRunning;
    ThreadGroup connections;

    /**
    * displays merchantservers state at the state
    */
    private void writeStatus(String stringToWrite) {
	try {
	    String stringToDisplay = merchantName + ": " + stringToWrite + "\n";
	    byte[] bytesToDisplay = new byte[stringToDisplay.length() + 1];
	    bytesToDisplay[0] = (byte)state.Pad.MERCHANT;
	    byte[] stringBytes = stringToDisplay.getBytes();
	    for (int i = 0; i < stringBytes.length; i++) bytesToDisplay[i + 1] = stringBytes[i];
	    DatagramPacket packet = new DatagramPacket(bytesToDisplay, bytesToDisplay.length, InetAddress.getLocalHost(),2005);
	    DatagramSocket socket = new DatagramSocket(2006);
	    socket.send(packet);
	    socket.close();
	} catch(Exception e) {
	    System.out.println("Fehler in writeStatus");
	}
    }

    /**
    * creates a MerchantServer
    */
    public MerchantServer(Merchant merchant, String merchantName, String catalogFileName, int portNumber) {
	try {
	    this.merchant = merchant;
	    this.merchantName = merchantName;
	    System.out.println("Initialisiere Server.");
	    System.out.print("\n  Name: " + merchantName);
	    System.out.print(" - Katalog file: " + catalogFileName);
	    System.out.println(" - Portnummer: " + portNumber);
	    System.out.println("\n...ok!");
	    catalogFile = new CatalogFile(new File(catalogFileName));
	    writeStatus("Katalog \"" + catalogFileName + "\" gefunden.");
	    writeStatus("MerchantServer startet mit Port " + portNumber);
	    ssonetServerConnection = new SSONETServerConnection(merchantName, merchant, portNumber);
	    serverRunning = true;
	    setDaemon(true);
	    connections = new ThreadGroup("Connections");
	    start();
        } catch (Exception e) {
            System.out.println("Merchant Fehler:" + e);
        }
    }

    /**
    * accept Connections while server running
    */
    public void run() {
	while(serverRunning) {
	    try {
		writeStatus("Warte auf Verbindung.");
		new Connection(connections,(SSONETStream)ssonetServerConnection.accept(), this);
	    } catch (Exception e) {
		writeStatus("Fehler in MerchantServer.run(): " + e);
	    }
	}
    }

    /**
    * stops the MerchantServer, all connections will be closed after 10 seconds
    */
    public void stopServer() {
	serverRunning = false;
	ssonetServerConnection.close();
	for(int i = 0; i < 10; i++) {
	    if(connections.activeCount() > 0)
		try {
		sleep(1000);
} catch(InterruptedException e) {}
else
break;
	}
	connections.stop();
	connections.destroy();
	stop();
    }

    /**
    * returns a message that an order was received and displays messages regarding
    * an order at the state
    */
    public String getOrder(Order order) {
	writeStatus("Bestellung erhalten: " + order.ordererName);
	writeStatus("Bestellung von " + order.ordererName + " ber " + order.orderedArticles.length + " Produkte:");
	for (int i = 0; i < order.orderedArticles.length; i++)
	    writeStatus("\t" + order.orderedArticles[i]);
	writeStatus("Die Summe betrgt:" + order.sum);

	return (new String("Wir haben ihren Auftrag erhalten,\nIhre endgltige Besttigung erhalten Sie\nin Krze per E-Mail."));
    }

    /**
    * returns the actual catalog
    */
    public CatalogFile requestCatalog() {
	writeStatus("Katalog wird verschickt.");
	return catalogFile;
    }
}
