/*
 * catalog/zip/Zip.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.zip;

import catalog.customer.Catalog;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;


/**
* contains methods for working with zip files
*/
public class Zip {

    private ZipInputStream zipInputStream;

    /**
    * opens a catalog file
    */
    public void open() {
	try {
            zipInputStream = new ZipInputStream(new FileInputStream(System.getProperty("ssonet_cfg")+"catalog/katalog.zip"));
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    * closes the catalog file
    */
    public void close() {
        try {
            zipInputStream.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    * returns the catalog from a zip file
    */
    public Catalog getCatalog() {
	Catalog catalog = new Catalog();

	String zipEntryAsString = getZipEntryAsString("katalog.txt");
	StringTokenizer stringTokenizer = new StringTokenizer(zipEntryAsString,"\n\r");
	int numberOfTokens = stringTokenizer.countTokens();
	catalog.setTitle(stringTokenizer.nextToken());
	catalog.setInfo(stringTokenizer.nextToken());
	catalog.setHost(stringTokenizer.nextToken());
	catalog.setPort(stringTokenizer.nextToken());
	catalog.setCurrency(stringTokenizer.nextToken());
	catalog.setTax(stringTokenizer.nextToken());
	//minus 6 because we already used 6 lines
	catalog.initTemp(numberOfTokens - 6);
	for (int i = 0; i < catalog.numberOfRows; i++)
	    catalog.setTemp(i, stringTokenizer.nextToken());
	return catalog;
    }

    /**
    * returns information for a order number
    */
    public String getInfoText(String orderNumber) {
	return (getZipEntryAsString(orderNumber + ".txt"));
    }

    /**
    * returns the picture of an order number as an array of bytes
    */
    public byte[] getPicture(String orderNumber) {
        ZipEntry zipEntry;
        byte allBytes[];
        try {
            zipInputStream = new ZipInputStream(new FileInputStream(System.getProperty("ssonet_cfg")+"catalog/katalog.zip"));
        } catch (Exception e) {
	    e.printStackTrace();
        }
        try {
            do {
                zipEntry = zipInputStream.getNextEntry();
            } while((zipEntry != null) &&
                (!zipEntry.getName().equals(orderNumber + ".jpg")) &&
                (!zipEntry.getName().equals(orderNumber + ".gif")));
            if(zipEntry == null) {
                System.out.println("Eintrag \"" + orderNumber + "\" existiert nicht.");
                return null;
            }
            InputStream inputStream = zipInputStream;
            int zipEntrySize = (new Long(zipEntry.getSize())).intValue();
            allBytes = new byte[zipEntrySize];
            for (int i = 0; i < zipEntrySize; i++)
		allBytes[i] = (byte)inputStream.read();
            return allBytes;
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /**
    * @return zipEntry as String
    */
    private String getZipEntryAsString(String zipEntryName)
    {
        ZipEntry zipEntry;
        InputStream inputStream;
        byte[] allBytes;
        int zipEntrySize, numberOfBytesRead, position = 0;
        try {
            do {
                zipEntry = zipInputStream.getNextEntry();
            } while((zipEntry != null) && (!zipEntry.getName().equals(zipEntryName)));
            if (zipEntry == null) {
                System.out.println("Eintrag \"" + zipEntryName + "\" existiert nicht.");
                return null;
            }
            inputStream = zipInputStream;
            zipEntrySize = (new Long(zipEntry.getSize())).intValue();
            allBytes = new byte[zipEntrySize];
            while (zipEntrySize > 0) {
                numberOfBytesRead = inputStream.read(allBytes, position, zipEntrySize);
                zipEntrySize -= numberOfBytesRead;
                position += numberOfBytesRead;
            }
            return (new String(allBytes));
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
