/*
 * gateway/GAccServer.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 * experimental source code
 */
package gateway;

import cryptix.provider.rsa.RawRSAPublicKey;
import cryptix.provider.rsa.RawRSAPrivateKey;

import javax.swing.JFrame;
import java.net.*;
import java.io.*;
import java.util.Hashtable;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.Signature;
import java.security.PublicKey;
import java.security.PrivateKey;


import smi.appConf.ApplicationConfiguration;
import smi.io.*;

import ssonet.SSONETConnection;
import ssonet.SSONETServerConnection;
import ssonet.SSONETConstants;
import ssonet.Preload;
import ssonet.SSONETConstraints;
import ssonet.KeyExchange;
import smi.baseConf.BaseConfiguration;
import sun.security.x509.X509Cert;
import smi.mechanisms.confidentiality.RSA;
import java.security.Key;
import smi.io.IOStream;
import smi.mechanisms.*;

public class GAccServer extends JFrame implements Serializable {
    final static int PORT = 5001;
    Preload p = new Preload();
    IOStream rsaStream, ioStream;
    ServerSocket serversocket = null;
    Socket socket = null;
    // Action-ID
    //  Cipher CipherInH, CipherOutH, CipherInB, CipherOutB;
    static String KEY_MODE_PREFERENCELIST = "Mode.Preferencelist";
    private static String KEY_VARIANT = "KeyVariante";

    //  String mechanismName;

    protected X509Cert ownTestCertificate, ownCipherCertificate;
    private X509Cert mechanismCertificate;
    protected PublicKey ownPublicKey, trustCenterTestKey;

    private String packageName;
    private static PrivateKey ownPrivateKey;

    Signature hinSignatureIn;
    Signature hinSignatureOut;
    Signature backSignatureIn;
    Signature backSignatureOut;
    PublicKey testKeyHin;
    PublicKey testKeyBack;


    boolean isServer=true;
    KeyExchange GWKeyExchange;

    static public void main(String[] argv) {
	new GAccServer();
    }

    private GAccServer() {
	try {
	    serversocket = new ServerSocket(PORT);
	} catch(Exception e) {
	    System.out.println("Error creating Socket!");
	    e.printStackTrace();
	    System.exit(1);
	}

	while (true) {
	    try {

		//RSA rsa = new RSA();
		//rsa.init(rsaStream, null);//, null);
		System.out.println("*");

		try {
		    //              serversocket = new ServerSocket(PORT);
		    socket = serversocket.accept();
                }
		catch (UnknownHostException e) {
		    System.err.println(e);
		}
		catch (IOException e) {
		    System.err.println(e);
		}
		System.out.println("Server Socket created");
		InputStream in = socket.getInputStream();
		OutputStream out = socket.getOutputStream();
		IOStream ioStream = new ObjectIOStream(in, out);
		try {
		    initConnection();

		    // Get PublicKey of Server
		    PublicKey testKeyHin=(PublicKey)ioStream.readObject();

		    String variant=(String)ioStream.readObject();

		    MechanismConfiguration mechs = (MechanismConfiguration)ioStream.readObject();
		    String mechanismNameBack = (String)ioStream.readObject();
		    System.out.println("GAccServer - mechanismNameBack: "+mechanismNameBack);

		    packageName = (String)ioStream.readObject();
		    String mechanismNameHin = (String)ioStream.readObject();
		    System.out.println("GAccServer - mechanismNameHin: "+mechanismNameHin);

		    // DSA-Testkey hardcodiert!
		    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPublicKey.object");
		    ObjectInputStream objectInputStream1 = new ObjectInputStream(fileInputStream);
		    PublicKey testKeyBack = (PublicKey)objectInputStream1.readObject();
		    fileInputStream.close();
		    objectInputStream1.close();

		    MechanismConfiguration backMech = (MechanismConfiguration)mechs.get("BACK");
		    MechanismConfiguration hinMech  = (MechanismConfiguration)mechs.get("HIN");
		    initCipherBack(mechanismNameBack, testKeyBack, ioStream);
		    initCipherHin(variant, mechanismNameHin, testKeyHin, ioStream);

		    while(ioStream.available()==0) {
			//immer vom Client
			System.out.println("GAccGW: lese Objekt vom Client");
			byte[] inp  = (byte[])ioStream.readObject();
			Hashtable ht  = (Hashtable)(new ObjectInputStream(new ByteArrayInputStream(inp))).readObject();
			System.out.println("Object empfangen.");
			Object object = ht.get("OBJECT");


			if(((String)ht.get("TODO")).equals("SIGN")) {
			    System.out.println("Object mit Hinmech. signieren.");
			    //Nachricht direkt vom Client im Klartext
			    //muss vom Gateway signiert (HinMech).
			    System.out.println(packageName);
			    if(packageName.equals("accountability.")) {
				// signieren!!
				SignatureObject signatureObject = new SignatureObject(object);
				hinSignatureOut.update(signatureObject.getObjectAsByteArray());
				signatureObject.setSignature(hinSignatureOut.sign());
				ioStream.writeObject(signatureObject);
				System.out.println("Object mit Hinmech. signiert.");
			    }
			}
			else if(((String)ht.get("TODO")).equals("VERIFY&SIGN")) {
			    System.out.println("Signatur testen und neu kodieren.");
			    if(packageName.equals("accountability.")) {
				//Object umcodieren  (testen (HinMech), signieren (RckMech))
				//byte[] buffer = (byte[])object;
				SignatureObject hinSignatureObject = (SignatureObject)object;

				// Signaturtest
				hinSignatureIn.update(hinSignatureObject.getObjectAsByteArray());
				if (hinSignatureIn.verify(hinSignatureObject.getSignature())==false) {
				    System.out.println("GAccServer: Ungltige Serversignatur!");
				    System.out.println("GAccServer: Arbeite dennoch weiter!");
				}

				//neu signieren
				System.out.println("Object umsignieren.");

				// Objekt zum Signieren extrahieren
				SignatureObject backSignatureObject = new SignatureObject(hinSignatureObject.getObject());

				backSignatureOut.update(backSignatureObject.getObjectAsByteArray());
				backSignatureObject.setSignature(backSignatureOut.sign());
				ioStream.writeObject(backSignatureObject);

				//System.out.println("GAccServer - Classname:" + (String)(returnObject.getClass().getName()));
				System.out.println("Object umsigniert.");
			    }
			}
		    }
		    out.close();
		    in.close();
		    ioStream.close();
		    socket.close();
		    System.out.println("Socket closed");
                }
                catch(IOException e) {
                    System.out.println("inner catch");
                    out.close();
                    in.close();
                    ioStream.close();
                    socket.close();
                    System.out.println("Socket closed");
		    //                  System.err.println(e);
		    //                  System.exit(1);
                }
	    } catch(Exception e) {
		System.out.println("outer catch");
		e.printStackTrace();
		System.exit(1);
	    }
	}
    }
    //Hinmechanismus initialisieren
    private void initCipherHin(String variant, String mechanismname, PublicKey testKey, IOStream ioStream) {
	try {
	    this.ioStream = ioStream;
	    System.out.println("GAccServer, initCipherHin, mechanismname (zunchst RSA):" + mechanismname);
	    System.out.println("GAccServer, initCipherHin, variante:" + variant);
	    byte[] rawBuffer;
	    if (testKey instanceof RawRSAPublicKey) {
		rawBuffer = testKey.getEncoded();
		System.out.println("RawRSAKEy");
	    } else {
		rawBuffer = KeyExchange.DERtoRAW(testKey.getEncoded());
		System.out.println("DerRSAKEy");
	    }
	    ByteArrayInputStream byteArrayInputStream = new ByteArrayInputStream(rawBuffer);
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(byteArrayInputStream);

	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/RSAPrivateKey.der");
	    RawRSAPrivateKey signKey = new RawRSAPrivateKey(fileInputStream);
	    fileInputStream.close();

	    System.out.println("RSA-Instanzen erzeugen!");
	    hinSignatureIn = Signature.getInstance(variant + "/" + mechanismname);
	    hinSignatureOut = Signature.getInstance(variant + "/" + mechanismname);

	    System.out.println("RSA-Instanzen initialisieren");
	    hinSignatureIn.initVerify(rawRSAPublicKey);
	    hinSignatureOut.initSign(signKey);
	    System.out.println(mechanismname + " instance initialized");
	}
	catch(Exception e) {
	    System.out.println(e);
	}
    }
    //Rckmechanismus initialisieren
    private void initCipherBack(String mechanismName, PublicKey testKey, IOStream ioStream) {
	// zunchst nur DSA!!
	try {
	    this.ioStream = ioStream;

	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey signKey = (PrivateKey)objectInputStream.readObject();
	    fileInputStream.close();
	    objectInputStream.close();

	    backSignatureIn = Signature.getInstance(mechanismName);
	    backSignatureIn.initVerify(testKey);
	    backSignatureOut = Signature.getInstance(mechanismName);
	    backSignatureOut.initSign(signKey);
	    System.out.println(mechanismName + " instance as BackMech initialized");
	}
	catch(Exception e) {
	    System.out.println(e);
	}
    }
    private void initConnection() {
	System.out.println("reading trustcenter certificate:");
	X509Cert trustCenterCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/TrustCenterCertificate.der");
	trustCenterTestKey = trustCenterCertificate.getPublicKey();

	System.out.println("testing trustcenter certificate.");
	if (!KeyExchange.testCertificate(trustCenterCertificate, trustCenterTestKey))
	    throw new SecurityException();

	System.out.println("reading ownTestCertificate:");
	ownTestCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeDSACertificate.der");
	ownPublicKey = ownTestCertificate.getPublicKey();

	System.out.println("testing ownTestCertificate.");
	if (!KeyExchange.testCertificate(ownTestCertificate, trustCenterTestKey))
	    throw new SecurityException();

	System.out.println("reading ownCipherCertificate:");
	ownCipherCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeRSACertificate.der");

	System.out.println("testing ownCipherCertificate.");
	if (!KeyExchange.testCertificate(ownCipherCertificate, ownPublicKey))
	    throw new SecurityException();
    }

    /** decrypt ByteArray with private key
    * @param input data to decrypt
    * @param privateKey private key
    * @return byte[] decrypted data
    */
    private byte[] decrypt(byte[] input, PrivateKey privateKey) {
	return null;
    }

    public byte[] getObjectAsByteArray(Object object) {
	try {
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(object);
	    objectOutputStream.flush();
	    byte[] b = byteArrayOutputStream.toByteArray();
	    System.out.println("getObjectAsByteArray lgt.: "+b.length);
	    return b;
	} catch (Exception e) {
	    System.out.println("Error while transforming into ByteArray.");
	    e.printStackTrace();
	    return null;
	}
    }
}
