/*
 * goals/awt/ButtonGroupComponent.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package goals.awt;

import java.awt.Color;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

public class ButtonGroupComponent extends JComponent implements ActionListener {
    private JRadioButton unconditionalButton,
	ifPossibleButton,
	doesntMatterButton,
	ifNeccessaryButton,
	neverButton;
    private JCheckBox unconditionalNegotiableCheckBox, neverNegotiableCheckBox;
    private boolean negotiable;
    private JLabel iconLabel;
    private String iconOnFileName, iconEqualFileName, iconOffFileName;
    private ActionListener actionListener;

    public ButtonGroupComponent(String iconOnFileName,
	String iconEqualFileName,
	String iconOffFileName,
	boolean negotiable,
	String unconditionalToolTipText,
	String neverToolTipText) {
	this.iconOnFileName = iconOnFileName;
	this.iconEqualFileName = iconEqualFileName;
	this.iconOffFileName = iconOffFileName;
	this.negotiable = negotiable;

	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
        setLayout(gridBagLayout);

	iconLabel = new JLabel(new ImageIcon(getClass().getResource(iconOnFileName)));
	gridBagConstraints.anchor = GridBagConstraints.NORTH;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.weighty = 0;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.insets = new Insets(5,20,0,0);
	gridBagConstraints.fill = GridBagConstraints.NONE;
	gridBagLayout.setConstraints(iconLabel, gridBagConstraints);
	add(iconLabel);

	JPanel buttonPanel = new JPanel();
	gridBagConstraints.anchor = GridBagConstraints.CENTER;
	gridBagConstraints.weightx = 1;
	gridBagConstraints.weighty = 1;
	gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	gridBagConstraints.fill = GridBagConstraints.BOTH;
	gridBagConstraints.insets = new Insets(0,20,0,0);
	gridBagLayout.setConstraints(buttonPanel, gridBagConstraints);
	add(buttonPanel);
	buttonPanel.setLayout(gridBagLayout);

	ButtonGroup buttonGroup = new ButtonGroup();

	Color myColor = new Color(102,102,153);
	Font myFont = new Font("Dialog", Font.PLAIN,12);

	unconditionalButton = new JRadioButton("unbedingt");
	unconditionalButton.setToolTipText(unconditionalToolTipText);
	unconditionalButton.setFont(myFont);
	unconditionalButton.addActionListener(this);
	gridBagConstraints.anchor = GridBagConstraints.WEST;
	gridBagConstraints.fill = GridBagConstraints.NONE;
	if (negotiable) {
	    gridBagConstraints.gridwidth = 1;
	} else {
	    gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	}
	gridBagConstraints.insets = new Insets(0,0,0,0);
	gridBagLayout.setConstraints(unconditionalButton, gridBagConstraints);
	buttonGroup.add(unconditionalButton);
	buttonPanel.add(unconditionalButton);

	if (negotiable) {
	    unconditionalNegotiableCheckBox = new JCheckBox("verhandelbar");
	    unconditionalNegotiableCheckBox.setFont(myFont);
	    unconditionalNegotiableCheckBox.addActionListener(this);
	    gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	    gridBagConstraints.insets = new Insets(0,10,0,0);
	    gridBagLayout.setConstraints(unconditionalNegotiableCheckBox, gridBagConstraints);
	    buttonPanel.add(unconditionalNegotiableCheckBox);
	}

	ifPossibleButton = new JRadioButton("mglichst");
	ifPossibleButton.setToolTipText(unconditionalToolTipText);
	ifPossibleButton.setFont(myFont);
	ifPossibleButton.addActionListener(this);
	gridBagConstraints.insets = new Insets(0,0,0,0);
	gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	gridBagLayout.setConstraints(ifPossibleButton, gridBagConstraints);
	buttonGroup.add(ifPossibleButton);
	buttonPanel.add(ifPossibleButton);

	doesntMatterButton = new JRadioButton("egal");
	doesntMatterButton.setFont(myFont);
	doesntMatterButton.addActionListener(this);
	gridBagLayout.setConstraints(doesntMatterButton, gridBagConstraints);
	buttonGroup.add(doesntMatterButton);
	buttonPanel.add(doesntMatterButton);

	ifNeccessaryButton = new JRadioButton("notfalls");
	ifNeccessaryButton.setToolTipText(neverToolTipText);
	ifNeccessaryButton.setFont(myFont);
	ifNeccessaryButton.addActionListener(this);
	gridBagLayout.setConstraints(ifNeccessaryButton, gridBagConstraints);
	buttonGroup.add(ifNeccessaryButton);
	buttonPanel.add(ifNeccessaryButton);

	neverButton = new JRadioButton("keinesfalls");
	neverButton.setToolTipText(neverToolTipText);
	neverButton.setFont(myFont);
	neverButton.addActionListener(this);
	gridBagConstraints.gridwidth = 1;
	gridBagLayout.setConstraints(neverButton, gridBagConstraints);
	buttonGroup.add(neverButton);
	buttonPanel.add(neverButton);

	if (negotiable) {
	    neverNegotiableCheckBox = new JCheckBox("verhandelbar");
	    neverNegotiableCheckBox.setFont(myFont);
	    neverNegotiableCheckBox.addActionListener(this);
	    gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
	    gridBagConstraints.insets = new Insets(0,10,0,0);
	    gridBagLayout.setConstraints(neverNegotiableCheckBox, gridBagConstraints);
	    buttonPanel.add(neverNegotiableCheckBox);
	}
    }

    public void setLevel(int level) {
	switch (level) {
	case 0:
	    unconditionalButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOnFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(true);
		neverNegotiableCheckBox.setEnabled(false);
		unconditionalNegotiableCheckBox.setSelected(false);
	    }
	    break;
	case 1:
	    unconditionalButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOnFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(true);
		neverNegotiableCheckBox.setEnabled(false);
		unconditionalNegotiableCheckBox.setSelected(true);
	    }
	    break;
	case 2:
	    ifPossibleButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOnFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(false);
	    }
	    break;
	case 3:
	    doesntMatterButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconEqualFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(false);
	    }
	    break;
	case 4:
	    ifNeccessaryButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOffFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(false);
	    }
	    break;
	case 5:
	    neverButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOffFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(true);
		neverNegotiableCheckBox.setSelected(true);
	    }
	    break;
	case 6:
	    neverButton.setSelected(true);
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOffFileName)));
	    if (negotiable) {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(true);
		neverNegotiableCheckBox.setSelected(false);
	    }
	}
    }

    public void setMaxLevel(int maxLevel) {
	// change level if neccessary
	if(getLevel() > maxLevel) setLevel(maxLevel);
	int currentLevel = getLevel();

	// enable or disable Buttons
	switch (maxLevel) {
	case 0:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(false);
	    doesntMatterButton.setEnabled(false);
	    ifNeccessaryButton.setEnabled(false);
	    neverButton.setEnabled(false);
	    if (negotiable) unconditionalNegotiableCheckBox.setEnabled(false);
	    break;
	case 1:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(false);
	    doesntMatterButton.setEnabled(false);
	    ifNeccessaryButton.setEnabled(false);
	    neverButton.setEnabled(false);
	    if (negotiable) unconditionalNegotiableCheckBox.setEnabled(true);
	    break;
	case 2:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(true);
	    doesntMatterButton.setEnabled(false);
	    ifNeccessaryButton.setEnabled(false);
	    neverButton.setEnabled(false);
	    if (negotiable && (currentLevel == 0 || currentLevel == 1)) {
		unconditionalNegotiableCheckBox.setEnabled(true);
	    }
	    break;
	case 3:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(true);
	    doesntMatterButton.setEnabled(true);
	    ifNeccessaryButton.setEnabled(false);
	    neverButton.setEnabled(false);
	    if (negotiable && (currentLevel == 0 || currentLevel == 1)) {
		unconditionalNegotiableCheckBox.setEnabled(true);
	    }
	    break;
	case 4:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(true);
	    doesntMatterButton.setEnabled(true);
	    ifNeccessaryButton.setEnabled(true);
	    neverButton.setEnabled(false);
	    if (negotiable && (currentLevel == 0 || currentLevel == 1)) {
		unconditionalNegotiableCheckBox.setEnabled(true);
	    }
	    break;
	case 5:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(true);
	    doesntMatterButton.setEnabled(true);
	    ifNeccessaryButton.setEnabled(true);
	    neverButton.setEnabled(true);
	    if (negotiable) {
		if (currentLevel == 0 || currentLevel == 1) unconditionalNegotiableCheckBox.setEnabled(true);
		if (currentLevel == 5) neverNegotiableCheckBox.setEnabled(true);
	    }
	    break;
	case 6:
	    unconditionalButton.setEnabled(true);
	    ifPossibleButton.setEnabled(true);
	    doesntMatterButton.setEnabled(true);
	    ifNeccessaryButton.setEnabled(true);
	    neverButton.setEnabled(true);
	    if (negotiable) {
		if (currentLevel == 0 || currentLevel == 1) unconditionalNegotiableCheckBox.setEnabled(true);
		if (currentLevel == 5 || currentLevel == 6) neverNegotiableCheckBox.setEnabled(true);
	    }
	}
    }

    public int getLevel() {
	if (unconditionalButton.isSelected()) {
	    if (negotiable) {
		if (unconditionalNegotiableCheckBox.isSelected()) return 1;
	    }
	    return 0;
	}
	else if (ifPossibleButton.isSelected()) return 2;
	else if (doesntMatterButton.isSelected()) return 3;
	else if (ifNeccessaryButton.isSelected()) return 4;
	else if (neverButton.isSelected()) {
	    if (negotiable) {
		if (neverNegotiableCheckBox.isSelected()) return 5;
	    }
	    return 6;
	}
	// in case of an error
	return -1;
    }

    public void actionPerformed(ActionEvent event) {
	// turn buttonStates on or off
	if (negotiable) {
	    if (event.getSource() == unconditionalButton) {
		unconditionalNegotiableCheckBox.setEnabled(true);
		neverNegotiableCheckBox.setEnabled(false);
	    } else if (event.getSource() == neverButton) {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(true);
	    } else if (event.getSource() == unconditionalNegotiableCheckBox ||
		event.getSource() == neverNegotiableCheckBox) {
		// do nothing
	    } else {
		unconditionalNegotiableCheckBox.setEnabled(false);
		neverNegotiableCheckBox.setEnabled(false);
	    }
	}

	// set icon images regarding to choosen security value
	if (event.getSource() == unconditionalButton ||
	    event.getSource() == ifPossibleButton) {
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOnFileName)));
	} else if (event.getSource() == doesntMatterButton) {
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconEqualFileName)));
	} else if (event.getSource() == ifNeccessaryButton ||
	    event.getSource() == neverButton) {
	    iconLabel.setIcon(new ImageIcon(getClass().getResource(iconOffFileName)));
	}
	actionListener.actionPerformed(new ActionEvent(this,12345,"ButtonGroup"));
    }

    public void addActionListener(ActionListener actionListener) {
	this.actionListener = actionListener;
    }
}
