/*
 * goals/awt/CommunicationSecurityDialog.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package goals.awt;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.RandomAccessFile;
import java.net.URL;
import java.util.Enumeration;

import help.Help;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.border.LineBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

import goals.SecuritySettings;

public class CommunicationSecurityDialog extends JDialog
implements ActionListener,
TreeSelectionListener,
TreeExpansionListener {
    private Container container;
    private JScrollPane treeScrollPane;
    private JButton okButton, cancelButton, helpButton, settingsButton;
    private JTree tree;
    private JRadioButton highSecurityButton,
	middleSecurityButton,
	lowSecurityButton,
	userDefinedSecurityButton;
    private Help help;
    private UserSecurityDefinitionDialog userDialog;
    private WindowAdapter myWindowAdapter = new WindowAdapter() {
        public void windowClosing(WindowEvent e) {
            dispose();
        }
    };

    public CommunicationSecurityDialog(Frame parentFrame, Help help) {
        super(parentFrame,"Sicherheitsforderungen fr Kommunikation", true);

        this.help = help;

        setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
        addWindowListener(myWindowAdapter);
        setModal (false);
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        container = this.getContentPane();
        container.setLayout(gridBagLayout);

        // read configuration
        try {
            FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg")+"goals/Security.settings");
            ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);

            DefaultMutableTreeNode teleshopping = (DefaultMutableTreeNode)objectInputStream.readObject();
            DefaultMutableTreeNode browse = (DefaultMutableTreeNode)objectInputStream.readObject();
            teleshopping.add(browse);
            DefaultMutableTreeNode business = (DefaultMutableTreeNode)objectInputStream.readObject();
            teleshopping.add(business);

            DefaultMutableTreeNode goals = (DefaultMutableTreeNode)objectInputStream.readObject();
            browse.add(goals);
            DefaultMutableTreeNode price = (DefaultMutableTreeNode)objectInputStream.readObject();
            browse.add(price);

            DefaultMutableTreeNode treeLeaf = (DefaultMutableTreeNode)objectInputStream.readObject();
            business.add(treeLeaf);
            treeLeaf = (DefaultMutableTreeNode)objectInputStream.readObject();
            business.add(treeLeaf);
            treeLeaf = (DefaultMutableTreeNode)objectInputStream.readObject();
            business.add(treeLeaf);
            treeLeaf = (DefaultMutableTreeNode)objectInputStream.readObject();
            business.add(treeLeaf);
            treeLeaf = (DefaultMutableTreeNode)objectInputStream.readObject();
            business.add(treeLeaf);
            treeLeaf = (DefaultMutableTreeNode)objectInputStream.readObject();
            business.add(treeLeaf);

            tree = new JTree(teleshopping);

            objectInputStream.close();
            fileInputStream.close();
        } catch (Exception e) {

            // use default values
            DefaultMutableTreeNode treeTop = new DefaultMutableTreeNode(new SecuritySettings("Teleshopping"));
            DefaultMutableTreeNode treeBranch = new DefaultMutableTreeNode(new SecuritySettings("Informationen einholen"));
            treeTop.add(treeBranch);

            DefaultMutableTreeNode treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Kataloganforderung"));
            treeBranch.add(treeLeaf);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Preisauskunft"));
            treeBranch.add(treeLeaf);

            treeBranch = new DefaultMutableTreeNode(new SecuritySettings("Geschftsabwicklung"));
            treeTop.add(treeBranch);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Bestellung"));
            treeBranch.add(treeLeaf);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Lieferbesttigung"));
            treeBranch.add(treeLeaf);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Warensendung"));
            treeBranch.add(treeLeaf);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Rechnung"));
            treeBranch.add(treeLeaf);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Zahlung"));
            treeBranch.add(treeLeaf);
            treeLeaf = new DefaultMutableTreeNode(new SecuritySettings("Zahlungsquittung"));
            treeBranch.add(treeLeaf);

            tree = new JTree(treeTop);
        }

        tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
        tree.addTreeExpansionListener(this);
        tree.addTreeSelectionListener(this);

        LineBorder lineBorder = new LineBorder(Color.black);
        TitledBorder titledBorder = new TitledBorder(lineBorder);
        titledBorder.setTitle("Aktionen der Anwendung");

        treeScrollPane = new JScrollPane(tree);
        treeScrollPane.setBorder(titledBorder);
        gridBagConstraints.gridheight = 2;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagLayout.setConstraints(treeScrollPane, gridBagConstraints);
        container.add(treeScrollPane);

        JPanel securityPanel = new JPanel();
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 1;
        gridBagConstraints.insets = new Insets(0,10,0,10);
        gridBagLayout.setConstraints(securityPanel, gridBagConstraints);
        container.add(securityPanel);

        titledBorder = new TitledBorder(lineBorder);
        titledBorder.setTitle("Sicherheit");
        securityPanel.setBorder(titledBorder);
        securityPanel.setLayout(gridBagLayout);

        ButtonGroup buttonGroup = new ButtonGroup();
        highSecurityButton = new JRadioButton("hoch");
        highSecurityButton.addActionListener(this);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.insets = new Insets(5,10,0,0);
        gridBagLayout.setConstraints(highSecurityButton, gridBagConstraints);
        buttonGroup.add(highSecurityButton);
        securityPanel.add(highSecurityButton);

        JLabel label = new JLabel("Schutz gegen professionelle Angreifer");
        gridBagConstraints.insets = new Insets(0,30,0,0);
        gridBagLayout.setConstraints(label, gridBagConstraints);
        securityPanel.add(label);

        middleSecurityButton = new JRadioButton("mittel");
        middleSecurityButton.addActionListener(this);
        gridBagConstraints.insets = new Insets(15,10,0,0);
        gridBagLayout.setConstraints(middleSecurityButton, gridBagConstraints);
        buttonGroup.add(middleSecurityButton);
        securityPanel.add(middleSecurityButton);

        label = new JLabel("Schutz gegen erfahrene Angreifer");
        gridBagConstraints.insets = new Insets(0,30,0,0);
        gridBagLayout.setConstraints(label, gridBagConstraints);
        securityPanel.add(label);

        lowSecurityButton = new JRadioButton("niedrig");
        lowSecurityButton.addActionListener(this);
        gridBagConstraints.insets = new Insets(15,10,0,0);
        gridBagLayout.setConstraints(lowSecurityButton, gridBagConstraints);
        buttonGroup.add(lowSecurityButton);
        securityPanel.add(lowSecurityButton);

        label = new JLabel("Schutz gegen unerfahrene Angreifer");
        gridBagConstraints.insets = new Insets(0,30,0,0);
        gridBagLayout.setConstraints(label, gridBagConstraints);
        securityPanel.add(label);

        userDefinedSecurityButton = new JRadioButton("benutzerdefiniert");
        userDefinedSecurityButton.addActionListener(this);
        gridBagConstraints.insets = new Insets(20,10,0,0);
        gridBagConstraints.gridwidth = 1;
        gridBagLayout.setConstraints(userDefinedSecurityButton, gridBagConstraints);
        buttonGroup.add(userDefinedSecurityButton);
        securityPanel.add(userDefinedSecurityButton);

        settingsButton = new JButton("Einstellungen...");
        settingsButton.addActionListener(this);
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagConstraints.insets = new Insets(20,10,0,10);
        gridBagLayout.setConstraints(settingsButton, gridBagConstraints);
        securityPanel.add(settingsButton);

        label = new JLabel("Individuell definierter Schutz");
        gridBagConstraints.insets = new Insets(0,30,10,0);
        gridBagLayout.setConstraints(label, gridBagConstraints);
        securityPanel.add(label);

        JPanel buttonPanel = new JPanel();
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.insets = new Insets(10,10,10,10);
        gridBagLayout.setConstraints(buttonPanel, gridBagConstraints);
        container.add(buttonPanel);

        buttonPanel.setLayout(gridBagLayout);
        okButton = new JButton("OK");
        okButton.addActionListener(this);
        gridBagConstraints.insets = new Insets(0,0,0,0);
        gridBagLayout.setConstraints(okButton, gridBagConstraints);
        buttonPanel.add(okButton);

        cancelButton = new JButton("Abbrechen");
        cancelButton.addActionListener(this);
        gridBagConstraints.insets = new Insets(0,5,0,0);
        gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
        buttonPanel.add(cancelButton);

        helpButton = new JButton("Hilfe");
        helpButton.addActionListener(this);
        gridBagLayout.setConstraints(helpButton, gridBagConstraints);
        buttonPanel.add(helpButton);

        tree.setSelectionRow(1);

        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension ownDimension = getSize();
        setLocation((screenSize.width - ownDimension.width) / 2,
	    (screenSize.height - ownDimension.height) / 2);
        requestFocus();
        setResizable(false);
    }

    public void setSubTreeSecurity(int securityLevel) {
	DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)(tree.getSelectionPath()).getLastPathComponent();
	Enumeration subNodes = selectedNode.breadthFirstEnumeration();
	int[] userValues = ((SecuritySettings)selectedNode.getUserObject()).getUserDefinedValues();
	while(subNodes.hasMoreElements()) {
	    DefaultMutableTreeNode subNode = (DefaultMutableTreeNode)subNodes.nextElement();
	    SecuritySettings securitySettings = (SecuritySettings)subNode.getUserObject();
	    securitySettings.setSecurityLevel(securityLevel);
	    if (securityLevel == SecuritySettings.USER_DEFINED) {
		// make a copy of userValues
		// is necessary, because we can't use the same settings for all nodes
		// Arrays are Objects, so this would lead to changes in all nodes when changing only one node
		int[] newValues = new int[userValues.length];
		System.arraycopy(userValues,0, newValues,0, userValues.length);
		securitySettings.setUserDefinedValues(newValues);
	    }
	}
    }

    public void actionPerformed(ActionEvent event) {
	// self made modality
	if(userDialog != null && userDialog.isShowing()) return;

	if (event.getSource() == settingsButton) {
	    DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)(tree.getSelectionPath()).getLastPathComponent();
	    boolean doIt = true;
	    if (!selectedNode.isLeaf()) {
		Object[] options = {"OK","Abbrechen"};
		int returnValue = JOptionPane.showOptionDialog(this,
		    "Sie berschreiben die zuvor\n" +
		    "getroffenen Einstellungen fr\n" +
		    "alle Aktionen dieser Transaktion.",
		    "Warnung",
		    JOptionPane.OK_CANCEL_OPTION,
		    JOptionPane.WARNING_MESSAGE,
		    null,
		    options,
		    options[0]);
		if(returnValue == JOptionPane.OK_OPTION) {
		    // ok, lets do it
		} else {
		    // ups, do nothing
		    doIt = false;
		}
	    }
	    if (doIt) {
		userDialog = new UserSecurityDefinitionDialog(this, selectedNode, help);
	    }
	} else if (event.getSource() == okButton) {
	    try {
		FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg")+"goals/Security.settings");
		ObjectOutputStream objectOutputStream = new ObjectOutputStream(fileOutputStream);
		RandomAccessFile logFile = new RandomAccessFile(System.getProperty("ssonet_cfg")+"goals/Logfile.txt","rw");
		// goto end of logfile
		logFile.seek(logFile.length());

		DefaultMutableTreeNode topNode = (DefaultMutableTreeNode)tree.getModel().getRoot();
		Enumeration subNodes = topNode.breadthFirstEnumeration();
		while(subNodes.hasMoreElements()) {
		    DefaultMutableTreeNode subNode = (DefaultMutableTreeNode)subNodes.nextElement();
		    objectOutputStream.writeObject(subNode);
		    SecuritySettings securitySettings = (SecuritySettings)subNode.getUserObject();
		    int securityLevel = securitySettings.getSecurityLevel();
		    if (securityLevel != SecuritySettings.USER_DEFINED) {
			// just write security level
			switch (securityLevel) {
			case SecuritySettings.HIGH:
			    logFile.writeBytes(subNode.toString() + ": HIGH\n");
			    break;
			case SecuritySettings.MIDDLE:
			    logFile.writeBytes(subNode.toString() + ": MEDIUM\n");
			    break;
			case SecuritySettings.LOW:
			    logFile.writeBytes(subNode.toString() + ": LOW\n");
			}
		    } else {
			// write all details
			logFile.writeBytes(subNode.toString() + ": ");
			int[] userValues = securitySettings.getUserDefinedValues();
			for (int i = 0; i < userValues.length; i++) {
			    Integer integer = new Integer(userValues[i]);
			    logFile.writeBytes(integer.toString());
			    if (i < userValues.length - 1) logFile.writeBytes(",");
			}
			logFile.writeBytes("\n");
		    }
		}
		logFile.writeBytes("<----------------------------->\n");

		objectOutputStream.flush();
		fileOutputStream.close();
		logFile.close();
	    } catch (Exception ex) {
		ex.printStackTrace();
	    }
	    dispose();
	} else if (event.getSource() == cancelButton) {
	    dispose();
	} else if (event.getSource() == helpButton) {
	    help.vanishWindow();
	    help = new Help("file:///"+System.getProperty("ssonet_help")+"goals/HelpSet.hs");
	    help.viewPage("beginner");
	} else if (event.getSource() == highSecurityButton) {
	    setSubTreeSecurity(SecuritySettings.HIGH);
	    settingsButton.setEnabled(false);
	} else if (event.getSource() == middleSecurityButton) {
	    setSubTreeSecurity(SecuritySettings.MIDDLE);
	    settingsButton.setEnabled(false);
	} else if (event.getSource() == lowSecurityButton) {
	    setSubTreeSecurity(SecuritySettings.LOW);
	    settingsButton.setEnabled(false);
	} else if (event.getSource() == userDefinedSecurityButton) {
	    setSubTreeSecurity(SecuritySettings.USER_DEFINED);
	    settingsButton.setEnabled(true);
	}
    }

    private void checkButtons() {
	DefaultMutableTreeNode selectedNode = (DefaultMutableTreeNode)tree.getLastSelectedPathComponent();
	if (selectedNode != null) {
	    SecuritySettings securitySettings = (SecuritySettings)selectedNode.getUserObject();
	    switch (securitySettings.getSecurityLevel()) {
	    case SecuritySettings.HIGH:
		highSecurityButton.setSelected(true);
		settingsButton.setEnabled(false);
		break;
	    case SecuritySettings.MIDDLE:
		middleSecurityButton.setSelected(true);
		settingsButton.setEnabled(false);
		break;
	    case SecuritySettings.LOW:
		lowSecurityButton.setSelected(true);
		settingsButton.setEnabled(false);
		break;
	    case SecuritySettings.USER_DEFINED:
		userDefinedSecurityButton.setSelected(true);
		settingsButton.setEnabled(true);
	    }

	    if (selectedNode.isLeaf()) {
		highSecurityButton.setEnabled(true);
		middleSecurityButton.setEnabled(true);
		lowSecurityButton.setEnabled(true);
		userDefinedSecurityButton.setEnabled(true);
	    } else {
		if(tree.isExpanded(tree.getSelectionPath())) {
		    highSecurityButton.setEnabled(false);
		    middleSecurityButton.setEnabled(false);
		    lowSecurityButton.setEnabled(false);
		    userDefinedSecurityButton.setEnabled(false);
		    settingsButton.setEnabled(false);
		} else {
		    highSecurityButton.setEnabled(true);
		    middleSecurityButton.setEnabled(true);
		    lowSecurityButton.setEnabled(true);
		    userDefinedSecurityButton.setEnabled(true);
		}
	    }
	}
    }

    public void valueChanged(TreeSelectionEvent event) {
	checkButtons();
    }

    public void treeCollapsed(TreeExpansionEvent event) {
	checkButtons();
    }

    public void treeExpanded(TreeExpansionEvent event) {
	checkButtons();
    }
}
