/*
 * help/Help.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package help;

import javax.swing.*;
import java.awt.Dimension;
import java.awt.Point;
import java.net.URL;
import javax.help.HelpSet;
import javax.help.JHelp;
import javax.help.BadIDException;

/**
* An alternativ help-system using JavaHelp
*/
public class Help {
    private JHelp help;
    private JFrame helpFrame;
    private static Dimension helpSize = null;
    private static Point helpPos = null;

    public Help (String helpFile) {
        try {
            URL helpSetURL = new URL (helpFile);
            HelpSet helpSet = new HelpSet(null, helpSetURL);
            help = new JHelp(helpSet);
            helpFrame = new JFrame (helpSet.getTitle());
            helpFrame.getContentPane().add(help);
            if (helpSize == null) {
                helpSize = new Dimension(800,500);
                helpFrame.setSize(helpSize);
            }
            if (helpPos == null) {
                helpPos = new Point(0,0);
                helpFrame.setLocation(helpPos);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    * Sets the Theme of the help window and open the window
    * @param theme      main Theme (mostly name of htmlfile)
    * @param subtheme   an more specific theme (e.g. an specific algorithm)
    */
    public void viewPage (String theme, String subtheme) {
        try {
	    System.out.println (theme+"_"+subtheme);
            viewPage(theme+"_"+subtheme);
        } catch (BadIDException bie) {
            viewPage(theme);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
    * Sets the Theme of the help window and open the window
    * @param theme      main Theme (mostly name of htmlfile)
    */
    public void viewPage (String theme) {
        try {
	    System.out.println (theme);
            help.setCurrentID(theme);
        } catch (BadIDException bie) {
            // Helptheme not available, show Standard-helpscreen
	    System.out.println ("index");
            help.setCurrentID("index");  // should always be present!
        }
        reopen ();
    }

    /**
    * sets the size and location and display the Frame
    */
    protected void reopen () {
        helpFrame.setSize(helpSize);
        helpFrame.setLocation(helpPos);
        helpFrame.show();
    }

    public void vanishWindow() {
        helpSize = helpFrame.getSize();
        helpPos = helpFrame.getLocation();
        helpFrame.dispose();
    }
}
