/*
 * rating/Rating.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package rating;

import java.awt.GridBagLayout;
import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.GridBagConstraints;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.Color;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Enumeration;
import java.util.Vector;
import java.util.Locale;
import java.util.ResourceBundle;
import java.io.BufferedReader;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.StreamTokenizer;
import java.io.FileNotFoundException;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTabbedPane;
import javax.swing.event.ChangeListener;
import javax.swing.event.ChangeEvent;

public class Rating extends JPanel implements ActionListener
{

    public static JFrame frame;

    public static int slider1 = 0;
    public static int slider2 = 0;
    public static int slider3 = 0;
    public static int slider4 = 0;
    public static int slider5 = 0;
    public static int slider6 = 0;
    public static int slider7 = 0;

    public static JLabel sliderText1;
    public static JLabel sliderText2;
    public static JLabel sliderText3;
    public static JLabel sliderText4;
    public static JLabel sliderText5;
    public static JLabel sliderText6;
    public static JLabel sliderText7;

    private static JPanel panel1;
    private static JPanel panel2;
    private static JPanel panel3;
    private static JPanel panel4;
    private static JPanel panel5;
    private static JPanel panel6;
    private static JPanel panel7;
    private static JTabbedPane rightPanel;

    private static Vector algConfVector;
    private static Vector algIntVector;
    private static Vector algAccVector;
    private static Vector algAnonVector;

    private static RatingCanvas confidentialityCanvas;
    private static RatingCanvas integrityCanvas;
    private static RatingCanvas accountabilityCanvas;
    private static RatingCanvas anonymityCanvas;

    private static int costMinimum;
    private static int costMaximum;
    private static int throughputMinimum;
    private static int throughputMaximum;
    private static boolean firstThroughputReading = true;
    private static boolean firstCostReading = true;

    static Rating rating;
    private ResourceBundle resource = null;
    JButton okButton =null;
    JButton helpButton = null;
    JButton perfButton = null;

    // Read algorithms and their values from file "algo.txt"
    public static Vector getAlgValues(String fileName) {
	Vector vector = new Vector();
	AlgorithmDescriptor descriptor;

	System.out.println("getAlgValues: " + fileName);
	try {
	    String algorithmName = new String("");
	    Reader reader = new BufferedReader(new InputStreamReader(new FileInputStream(fileName)));
	    StreamTokenizer streamTokenizer = new StreamTokenizer(reader);
	    streamTokenizer.slashSlashComments(true);
	    streamTokenizer.eolIsSignificant(false);
	    int[] factors = new int[7];
	    while (true) {
		if (streamTokenizer.nextToken() == -1) break;
		algorithmName = streamTokenizer.sval;
		for (int j = 0; j < 7; j++) {
		    if (streamTokenizer.nextToken() == -1) {
			break;
		    }
		    factors[j] = (int)streamTokenizer.nval;
		    if (j == 3) {
			if(firstThroughputReading) {
			    throughputMinimum = factors[j];
			    throughputMaximum = factors[j];
			    firstThroughputReading = false;
			} else {
			    if(throughputMinimum > factors[j]) throughputMinimum = factors[j];
			    if(throughputMaximum < factors[j]) throughputMaximum = factors[j];
			}
		    }
		    if (j == 5) {
			if(firstCostReading) {
			    costMinimum = factors[j];
			    costMaximum = factors[j];
			    firstCostReading = false;
			} else {
			    if(costMinimum > factors[j]) costMinimum = factors[j];
			    if(costMaximum < factors[j]) costMaximum = factors[j];
			}
		    }
		}
		descriptor = new AlgorithmDescriptor(algorithmName,
		    factors[0],
		    factors[1],
		    factors[2],
		    factors[3],
		    factors[4],
		    factors[5],
		    factors[6]);
		vector.addElement(descriptor);
	    }
	} catch (FileNotFoundException fe) {
	    return null;
	} catch (Exception e) {
	    e.printStackTrace();
	    return null;
	}
	return vector;
    }

    public Rating() {
	try {
	    rating = this;
	    if(smi.baseConf.BaseConfiguration.getLangResources()!=null)
		resource = smi.baseConf.BaseConfiguration.getLangResources();
	    else resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.ENGLISH);
	    algConfVector = new Vector();
	    algIntVector = new Vector();
	    algAccVector = new Vector();
	    algAnonVector = new Vector();
	    algConfVector = getAlgValues(System.getProperty("ssonet_cfg")+"rating/confidentiality.dat");
	    algIntVector = getAlgValues(System.getProperty("ssonet_cfg")+"rating/integrity.dat");
	    algAccVector = getAlgValues(System.getProperty("ssonet_cfg")+"rating/accountability.dat");
	    algAnonVector = getAlgValues(System.getProperty("ssonet_cfg")+"rating/anonymity.dat");
	    if (algConfVector == null ||
		algIntVector == null ||
		algAccVector == null ||
		algAnonVector == null)
		throw new FileNotFoundException();

	    GridLayout gridLayout;
	    JPanel mainPanel;
	    JPanel leftPanel;

	    frame = new JFrame(resource.getString("RATING"));
	    frame.setResizable(false);

	    // adjust look & feel
	    final Dimension hpad5 = new Dimension(5,1);
	    final Dimension hpad10 = new Dimension(10,1);
	    final Dimension hpad20 = new Dimension(20,1);
	    final Dimension hpad25 = new Dimension(25,1);
	    final Dimension hpad30 = new Dimension(30,1);
	    final Dimension hpad40 = new Dimension(40,1);
	    final Dimension hpad80 = new Dimension(80,1);

	    final Dimension vpad5 = new Dimension(1,5);
	    final Dimension vpad10 = new Dimension(1,10);
	    final Dimension vpad20 = new Dimension(1,20);
	    final Dimension vpad25 = new Dimension(1,25);
	    final Dimension vpad30 = new Dimension(1,30);
	    final Dimension vpad40 = new Dimension(1,40);
	    final Dimension vpad80 = new Dimension(1,80);

	    // main panel
	    mainPanel = new JPanel();
	    mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.X_AXIS));
	    mainPanel.setAlignmentY(TOP_ALIGNMENT);
	    mainPanel.setAlignmentX(LEFT_ALIGNMENT);
	    mainPanel.add(Box.createRigidArea(hpad20));

	    // left panel
	    WeightSlider weightSlider1;
	    WeightSlider weightSlider2;
	    WeightSlider weightSlider3;
	    WeightSlider weightSlider4;
	    WeightSlider weightSlider5;
	    WeightSlider weightSlider6;
	    WeightSlider weightSlider7;

	    ChangeListener listener;

	    leftPanel = new JPanel();
	    leftPanel.setLayout(new BoxLayout(leftPanel, BoxLayout.Y_AXIS));
	    leftPanel.setAlignmentY(TOP_ALIGNMENT);
	    leftPanel.setAlignmentX(LEFT_ALIGNMENT);
	    leftPanel.setPreferredSize(new Dimension(320,390));
	    mainPanel.add(leftPanel);
	    leftPanel.add(Box.createRigidArea(vpad10));

	    // Slider 1
	    listener = new SliderListener();
	    panel1 = new JPanel();
	    panel1.setLayout(new BoxLayout(panel1, BoxLayout.Y_AXIS));
	    sliderText1 = new JLabel(resource.getString("SECURITY OF THE ALGORITHM")+slider1+"    ");
	    panel1.add(sliderText1);
	    weightSlider1 = new WeightSlider(0,100, slider1, "SliderOne");
	    weightSlider1.addChangeListener(listener);
	    weightSlider1.setToolTipText(resource.getString("RATINGTOOLTIP1"));
	    panel1.add(weightSlider1);
	    leftPanel.add(panel1);
	    leftPanel.add(Box.createRigidArea(vpad5));

	    // Slider 2
	    listener = new SliderListener();
	    panel2 = new JPanel();
	    panel2.setLayout(new BoxLayout(panel2, BoxLayout.Y_AXIS));
	    sliderText2 = new JLabel(resource.getString("SECURITY OF THE IMPLEMENTATION")+slider2+"    ");
	    panel2.add(sliderText2);
	    weightSlider2 = new WeightSlider(0,100, slider2, "SliderTwo");
	    weightSlider2.addChangeListener(listener);
	    weightSlider2.setToolTipText(resource.getString("RATINGTOOLTIP2"));
	    panel2.add(weightSlider2);
	    leftPanel.add(panel2);
	    leftPanel.add(Box.createRigidArea(vpad5));

	    // Slider 3
	    listener = new SliderListener();
	    panel3 = new JPanel();
	    panel3.setLayout(new BoxLayout(panel3, BoxLayout.Y_AXIS));
	    sliderText3 = new JLabel(resource.getString("PERFORMANCE")+slider3+"    ");
	    panel3.add(sliderText3);
	    weightSlider3 = new WeightSlider(0,100, slider3, "SliderThree");
	    weightSlider3.addChangeListener(listener);
	    weightSlider3.setToolTipText(resource.getString("RATINGTOOLTIP3"));
	    panel3.add(weightSlider3);
	    leftPanel.add(panel3);
	    leftPanel.add(Box.createRigidArea(vpad5));

	    // Slider 4
	    listener = new SliderListener();
	    panel4 = new JPanel();
	    panel4.setLayout(new BoxLayout(panel4, BoxLayout.Y_AXIS));
	    slider4 = throughputMinimum;
	    sliderText4 = new JLabel(resource.getString("MINIMUM ALGORITHM THROUGHPUT")+slider4+" kBytes/s    ");
	    panel4.add(sliderText4);
	    weightSlider4 = new WeightSlider(throughputMinimum, throughputMaximum, slider4, "SliderFour");
	    weightSlider4.addChangeListener(listener);
	    weightSlider4.setToolTipText(resource.getString("RATINGTOOLTIP4"));
	    panel4.add(weightSlider4);
	    leftPanel.add(panel4);
	    leftPanel.add(Box.createRigidArea(vpad5));

	    // Slider 5
	    listener = new SliderListener();
	    panel5 = new JPanel();
	    panel5.setLayout(new BoxLayout(panel5, BoxLayout.Y_AXIS));
	    sliderText5 = new JLabel(resource.getString("LOW COSTS")+slider5+"    ");
	    panel5.add(sliderText5);
	    weightSlider5 = new WeightSlider(0,100, slider5, "SliderFive");
	    weightSlider5.addChangeListener(listener);
	    weightSlider5.setToolTipText(resource.getString("RATINGTOOLTIP5"));
	    panel5.add(weightSlider5);
	    leftPanel.add(panel5);
	    leftPanel.add(Box.createRigidArea(vpad5));

	    // Slider 6
	    listener = new SliderListener();
	    panel6 = new JPanel();
	    panel6.setLayout(new BoxLayout(panel6, BoxLayout.Y_AXIS));
	    slider6 = costMaximum;
	    sliderText6 = new JLabel(resource.getString("MAXIMUM COSTS")+slider6+"    ");
	    panel6.add(sliderText6);
	    weightSlider6 = new WeightSlider(costMinimum, costMaximum, slider6, "SliderSix");
	    weightSlider6.addChangeListener(listener);
	    weightSlider6.setToolTipText(resource.getString("RATINGTOOLTIP6"));
	    panel6.add(weightSlider6);
	    leftPanel.add(panel6);
	    leftPanel.add(Box.createRigidArea(vpad5));

	    // Slider 7
	    listener = new SliderListener();
	    panel7 = new JPanel();
	    panel7.setLayout(new BoxLayout(panel7, BoxLayout.Y_AXIS));
	    sliderText7 = new JLabel(resource.getString("CHALLANGES")+slider7+"    ");
	    panel7.add(sliderText7);
	    weightSlider7 = new WeightSlider(0,100, slider7, "SliderSeven");
	    weightSlider7.addChangeListener(listener);
	    weightSlider7.setToolTipText(resource.getString("RATINGTOOLTIP7"));
	    panel7.add(weightSlider7);
	    leftPanel.add(panel7);
	    leftPanel.add(Box.createRigidArea(vpad20));

	    // Perfomancetestbutton
	    okButton = new JButton(resource.getString("OK"));
	    helpButton = new JButton(resource.getString("HELP"));
	    perfButton = new JButton(resource.getString("PERFORMANCE_TEST"));
	    JPanel buttonPanel = new JPanel();
	    buttonPanel.setLayout(new BoxLayout(buttonPanel, BoxLayout.X_AXIS));
	    perfButton.setToolTipText(resource.getString("RATINGTOOLTIP8"));
	    perfButton.setMnemonic('p');

	    perfButton.addActionListener(this);
	    perfButton.getAccessibleContext().setAccessibleName("perfButton");
	    perfButton.setHorizontalAlignment(JButton.CENTER);
	    leftPanel.add(buttonPanel);
	    buttonPanel.setSize(new Dimension(320,30));
	    buttonPanel.add(perfButton);
	    buttonPanel.add(Box.createRigidArea(hpad10));
	    buttonPanel.add(okButton);
	    okButton.addActionListener(this);
	    buttonPanel.add(Box.createRigidArea(hpad10));
	    helpButton.addActionListener(this);
	    helpButton.setEnabled(false);
	    buttonPanel.add(helpButton);
	    buttonPanel.add(Box.createRigidArea(hpad80));
	    mainPanel.add(Box.createRigidArea(hpad20));

	    // right panel
	    confidentialityCanvas = new RatingCanvas(algConfVector);
	    integrityCanvas = new RatingCanvas(algIntVector);
	    accountabilityCanvas = new RatingCanvas(algAccVector);
	    anonymityCanvas = new RatingCanvas(algAnonVector);

	    rightPanel = new JTabbedPane();
	    rightPanel.setDoubleBuffered(true);
	    rightPanel.addTab(resource.getString("CONFIDENTIALITY"), null, confidentialityCanvas, resource.getString("RATINGTOOLTIP9"));
	    rightPanel.addTab(resource.getString("INTEGRITY"), null, integrityCanvas, resource.getString("RATINGTOOLTIP10"));
	    rightPanel.addTab(resource.getString("ACCOUNTABILITY"), null, accountabilityCanvas, resource.getString("RATINGTOOLTIP11"));
	    rightPanel.addTab(resource.getString("ANONYMITY"), null, anonymityCanvas, resource.getString("RATINGTOOLTIP12"));
	    rightPanel.setPreferredSize(new Dimension(350,300));
	    mainPanel.add(rightPanel);

	    WindowListener frameListener = new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		}
	    };
	    frame.addWindowListener(frameListener);
	    frame.setContentPane(mainPanel);
	    frame.pack();
	    Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
	    Dimension ownDimension = frame.getSize();
	    frame.setLocation((screenSize.width - ownDimension.width) / 2,
		(screenSize.height - ownDimension.height) / 2);
	    frame.show();
	} catch (FileNotFoundException fe) {
	    JOptionPane.showMessageDialog(frame, resource.getString("FILES MISSING"));
	}
    }

    class SliderListener implements ChangeListener {
	public SliderListener() {
	}

	public void stateChanged(ChangeEvent e) {
	    WeightSlider s1 = (WeightSlider)e.getSource();
	    String temp = new String(s1.getAccessibleContext().getAccessibleName());
	    if (temp.equals("SliderOne")) {
		slider1 = s1.getValue();
		sliderText1.setText(resource.getString("SECURITY OF THE ALGORITHM")+slider1);
	    }
	    else if (temp.equals("SliderTwo")) {
		slider2 = s1.getValue();
		sliderText2.setText(resource.getString("SECURITY OF THE IMPLEMENTATION") + slider2);
	    }
	    else if (temp.equals("SliderThree")) {
		slider3 = s1.getValue();
		sliderText3.setText(resource.getString("PERFORMANCE") + slider3);
	    }
	    else if (temp.equals("SliderFour")) {
		slider4 = s1.getValue();
		sliderText4.setText(resource.getString("MINIMUM ALGORITHM THROUGHPUT") + slider4+" kBytes/s");
	    }
	    else if (temp.equals("SliderFive")) {
		slider5 = s1.getValue();
		sliderText5.setText(resource.getString("LOW COSTS") + slider5);
	    }
	    else if (temp.equals("SliderSix")) {
		slider6 = s1.getValue();
		sliderText6.setText(resource.getString("MAXIMUM COSTS") + slider6);
	    }
	    else if (temp.equals("SliderSeven")) {
		slider7 = s1.getValue();
		sliderText7.setText(resource.getString("CHALLANGES") + slider7);
	    }
	    int x = rightPanel.getSelectedIndex();
	    switch (x) {
	    case 0: confidentialityCanvas.update(confidentialityCanvas.getGraphics());
		break;
	    case 1: integrityCanvas.update(integrityCanvas.getGraphics());
		break;
	    case 2: accountabilityCanvas.update(accountabilityCanvas.getGraphics());
		break;
	    case 3: anonymityCanvas.update(anonymityCanvas.getGraphics());
	    }
	}
    }

    public void actionPerformed(ActionEvent e) {
	if(e.getSource() == perfButton)
	    (new smi.baseConf.PerformanceTest()).start();
	else if(e.getSource() == okButton)
	    frame.dispose();
	else if(e.getSource() == helpButton)
	    System.out.println("help not impl.");
    }

    class WeightSlider extends JSlider {
	public WeightSlider(int min, int max, int value, String name) {
	    setPaintLabels(true);
	    setMinimum(min);
	    setMaximum(max);
	    setValue(value);
	    setPaintTicks(true);

	    int intervall = (max-min);
	    int majorTick = 0;
	    int minorTick = 0;
	    int i = 1;
	    int j;
	    do {
		j = (int)java.lang.Math.pow(10, i);
		if (j >= intervall) {
		    majorTick = j / 5;
		    minorTick = majorTick / 2;
		    if ((j / intervall) >= 2) {
			majorTick = j / 10;
			minorTick = majorTick / 2;
		    }
		    if (((j / intervall) >= 5)&&(j > 10)) {
			majorTick = j / 20;
			minorTick = majorTick / 5;
		    }
		}
		i++;
	    } while (j < intervall);
	    setMajorTickSpacing(majorTick);
	    setMinorTickSpacing(minorTick);
	    setSnapToTicks( true );
	    putClientProperty( "JSlider.isFilled", Boolean.TRUE );
	    this.getAccessibleContext().setAccessibleName(name);
	    this.getAccessibleContext().setAccessibleDescription("A plain slider");
	}
    }

    class RatingCanvas extends JComponent {
	public Vector vector;

	Image offImage;
	Graphics offGraphics;

	public RatingCanvas(Vector v) {
	    vector = v;
	    setDoubleBuffered(true);
	}

	public void update(Graphics g) {
	    paint(g);
	}

	public void paint(Graphics graphics) {
	    Dimension dimension = getSize();
	    AlgorithmDescriptor descriptor = new AlgorithmDescriptor("",0,0,0,0,0,0,0);

	    //Create the offscreen graphics context, if no good one exists.
	    if (offGraphics == null) {
		offImage = createImage(dimension.width, dimension.height);
		offGraphics = offImage.getGraphics();
	    }

	    offGraphics.clearRect(0, 0, dimension.width, dimension.height);
	    offGraphics.setFont(new java.awt.Font("Dialog", java.awt.Font.BOLD,12));

	    int i = 0;
	    int ratingValue = 0;

	    offGraphics.setColor(Color.gray);
	    int middleWidth = dimension.width / 2;
	    offGraphics.drawLine(dimension.width / 2,0, dimension.width / 2, dimension.height);
	    offGraphics.setColor(Color.blue);

	    for (Enumeration e = vector.elements(); e.hasMoreElements();) {
		descriptor = (AlgorithmDescriptor)e.nextElement();
		ratingValue = ((descriptor.factor1 - 50) * slider1 +
		    (descriptor.factor2 - 50) * slider2 +
		    (descriptor.factor3 - 50) * slider3 +
		    (descriptor.factor5 - 50) * slider5 +
		    (descriptor.factor7 - 50) * slider7) / 250;
		if((descriptor.factor4 < slider4) && (descriptor.factor6 <= slider6)) {
		    offGraphics.setColor(Color.gray);
		    offGraphics.drawString(descriptor.name + resource.getString("TOO SLOW"), middleWidth - 10,20 + i * 38);
		} else if((descriptor.factor4 >= slider4) && (descriptor.factor6 > slider6)) {
		    offGraphics.setColor(Color.gray);
		    offGraphics.drawString(descriptor.name + resource.getString("TOO EXPENSIVE"), middleWidth - 10,20 + i * 38);
		} else if((descriptor.factor4 < slider4) && (descriptor.factor6 > slider6)) {
		    offGraphics.setColor(Color.gray);
		    offGraphics.drawString(descriptor.name + resource.getString("TOO SLOW&EXPENSIVE"), middleWidth - 10,20 + i * 38);
		} else {
		    offGraphics.drawString(descriptor.name, middleWidth - 10,20 + i * 38);
		} if (ratingValue >= 0) {
		    offGraphics.fill3DRect(middleWidth,25 + i * 38, dimension.width * ratingValue / 240,10, true);
		    offGraphics.drawString("" + ratingValue,
			middleWidth + 10 + dimension.width * ratingValue / 240,
			35 + i * 38);
		} else {
		    if (offGraphics.getColor() != Color.gray)
			offGraphics.setColor(Color.red);
		    offGraphics.fill3DRect(middleWidth + (dimension.width * ratingValue / 240),
			25 + i * 38,
			-(dimension.width * ratingValue / 240),
			10,
			true);
		    offGraphics.drawString("" + ratingValue, middleWidth + 10,35 + i * 38);
		}
		offGraphics.setColor(Color.blue);
		i++;
	    }
	    graphics.drawImage(offImage,0,0, this);
	}
    }

    public static void main(String[] args) {
	rating = new Rating();
	rating.requestDefaultFocus();
    }
}
