/*
 * smi/awt/JPreferenceList.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.awt;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.lang.String;

import javax.swing.DefaultListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.SwingConstants;
import javax.swing.UIManager;
import javax.swing.border.Border;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class JPreferenceList extends JPanel implements ActionListener, ListSelectionListener {
    private JList preferenceList;
    private JSameSizeList list;
    private JLabel label1, label2;
    private ArrowButton upButton, downButton, addButton, removeButton;

    class ColorListCellRenderer implements ListCellRenderer {

	protected  Border noFocusBorder;

	public ColorListCellRenderer() {}

	public Component getListCellRendererComponent(JList list,
	    Object value,
	    int index,
	    boolean isSelected,
	    boolean cellHasFocus) {
	    JLabel label = new JLabel();
	    label.setOpaque(true);

	    if (isSelected) {
		label.setBackground(list.getSelectionBackground());
		label.setForeground(list.getSelectionForeground());
	    }
	    else {
		label.setBackground(list.getBackground());
		label.setForeground(list.getForeground());
	    }
	    if (value == null)
		label.setText("");
	    else
		label.setText(value.toString());

	    if (cellHasFocus)
		label.setBorder(UIManager.getBorder("JList.focusCellHighlightBorder"));
	    else
		label.setBorder(new EmptyBorder(1, 1, 1, 1));

	    if(((DefaultListModel)preferenceList.getModel()).contains(value))
		label.setEnabled(false);

	    return label;
	}
    }

    public JPreferenceList() {}

    public JPreferenceList(int lines, String[] itemStrings1, String[] itemStrings2) {
        this(lines, null, null, itemStrings1, itemStrings2);
    }

    public JPreferenceList(int lines, String l1, String l2, String[] itemStrings1, String[] itemStrings2)
    {
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        setLayout(gridBagLayout);
        label1 = new JLabel(l1);
        label2 = new JLabel(l2);

        // Label ... Head Left
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(5,5,0,0);
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.gridx = 1;
        gridBagLayout.setConstraints(label1, gridBagConstraints);
        add(label1);

        //Label Head Right
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridwidth = gridBagConstraints.REMAINDER;
        gridBagLayout.setConstraints(label2, gridBagConstraints);
        add(label2);

        JPanel leftButtonsPanel = new JPanel();
        leftButtonsPanel.setLayout(gridBagLayout);

        // Up...
        upButton = new ArrowButton(SwingConstants.NORTH);
        gridBagConstraints.insets = new Insets(0,0,0,0);
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.gridx = gridBagConstraints.RELATIVE;
        gridBagLayout.setConstraints(upButton, gridBagConstraints);
        upButton.addActionListener(this);
        leftButtonsPanel.add(upButton);

        // Down...
        downButton = new ArrowButton(SwingConstants.SOUTH);
        downButton.setMinimumSize(new Dimension(16,16));
        gridBagConstraints.gridwidth = 1;
        gridBagLayout.setConstraints(downButton, gridBagConstraints);
        downButton.addActionListener(this);
        leftButtonsPanel.add(downButton);

        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(leftButtonsPanel, gridBagConstraints);
        add(leftButtonsPanel);

        DefaultListModel defaultListModel = new DefaultListModel();
        if(itemStrings1!=null)
            for(int i = 0; i<itemStrings1.length; i++)
	    defaultListModel.addElement(itemStrings1[i]);
        preferenceList = new JList(defaultListModel);
        preferenceList.setVisibleRowCount(lines);
        preferenceList.addListSelectionListener(this);

        JScrollPane scrollPane = new JScrollPane(preferenceList);
        gridBagConstraints.insets = new Insets(5,5,0,0);
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(scrollPane, gridBagConstraints);
        add(scrollPane);

        JPanel rightButtonsPanel = new JPanel();
        rightButtonsPanel.setLayout(gridBagLayout);

        // --- Add.....
        addButton = new ArrowButton(SwingConstants.WEST);
        addButton.addActionListener(this);
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        gridBagConstraints.insets = new Insets(0,0,0,0);
        gridBagConstraints.gridwidth = gridBagConstraints.REMAINDER;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(addButton, gridBagConstraints);
        rightButtonsPanel.add(addButton);

        // ---- Remove.....
        removeButton = new ArrowButton(SwingConstants.EAST);
        removeButton.addActionListener(this);
        gridBagConstraints.gridwidth = 1;
        gridBagLayout.setConstraints(removeButton, gridBagConstraints);
        rightButtonsPanel.add(removeButton);

        gridBagConstraints.insets = new Insets(0,5,0,0);
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.fill = GridBagConstraints.VERTICAL;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(rightButtonsPanel, gridBagConstraints);
        add(rightButtonsPanel);

        defaultListModel = new DefaultListModel();
        if(itemStrings2 != null)
            for(int i = 0; i<itemStrings2.length; i++)
	    defaultListModel.addElement(itemStrings2[i]);

        list = new JSameSizeList(defaultListModel);
        list.setMaster(preferenceList);
        list.setVisibleRowCount(lines);
        list.setCellRenderer(new ColorListCellRenderer());
        list.addListSelectionListener(this);

        scrollPane = new JScrollPane(list);
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.insets = new Insets(5,5,0,0);
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 0.5;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(scrollPane, gridBagConstraints);
        add(scrollPane);
    }

    public String[] getPreferenceListItems() {
	ListModel listModel = preferenceList.getModel();
	String [] preferenceListItems = new String[listModel.getSize()];
	for(int i = 0; i<listModel.getSize(); i++)
	    preferenceListItems[i] = (String)listModel.getElementAt(i);
	return preferenceListItems;
    }

    public JList getPreferenceList() {
	return preferenceList;
    }


    public JList getList() {
	return list;
    }

    public void setHeader(String left, String right) {
	label1.setText(left);
	label2.setText(right);
    }

    private void moveItemUp() {
	DefaultListModel defaultListModel = (DefaultListModel)preferenceList.getModel();
	int index = preferenceList.getSelectedIndex();
	if(index<1)
	    return;
	Object obj = preferenceList.getSelectedValue();
	defaultListModel.setElementAt(defaultListModel.elementAt(index-1), index);
	defaultListModel.setElementAt(obj, index-1);
	preferenceList.setSelectedIndex(index-1);
    }

    private void moveItemDown() {
	DefaultListModel defaultListModel = (DefaultListModel)preferenceList.getModel();
	int index = preferenceList.getSelectedIndex();
	if((index == - 1) || (index >= (defaultListModel.getSize() - 1)))
	    return;
	Object object = defaultListModel.getElementAt(index);
	defaultListModel.setElementAt(defaultListModel.getElementAt(index+1), index);
	defaultListModel.setElementAt(object, index+1);
	preferenceList.setSelectedIndex(index+1);
    }


    final private void moveItem(JList list1, JList list2, boolean delete) {
	DefaultListModel defaultListModel1 = (DefaultListModel)list1.getModel();
	DefaultListModel defaultListModel2 = (DefaultListModel)list2.getModel();
	int index = list2.getSelectedIndex();
	if(index == -1)
	    return;
	Object newItem = defaultListModel2.elementAt(index);
	if(delete)
	    defaultListModel2.remove(index);
	if(index == defaultListModel2.getSize())
	    index--;
	list2.setSelectedIndex(index);
	if(delete) {
	    list1.repaint();
	    return;
	}
	for(int i = 0; i < defaultListModel1.getSize(); i++)
	    if(defaultListModel1.elementAt(i).equals(newItem)) {
	    java.awt.Toolkit.getDefaultToolkit().beep();
	    return;
	}
	defaultListModel1.addElement(newItem);
    }

    public void actionPerformed(ActionEvent event) {
	Object object = event.getSource();
	if(object == upButton)
	    moveItemUp();
	else if(object == downButton)
	    moveItemDown();
	else if(object == addButton)
	    moveItem(preferenceList, list, false);
	else if(object == removeButton)
	    moveItem(list, preferenceList, true);

    }

    final public void valueChanged(ListSelectionEvent event) {
	JList listWhereEventOccured = (JList)event.getSource();
	if(listWhereEventOccured.getSelectedIndex() != -1)
	    if(listWhereEventOccured == preferenceList)
	    list.clearSelection();
	else
	    preferenceList.clearSelection();
    }
}
