/*
 * smi/baseConf/BaseConfiguration.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.baseConf;

import cryptix.provider.rsa.RawRSAPublicKey;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.Dimension;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.IOException;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.SecureRandom;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.Locale;
import java.util.ResourceBundle;

import javax.swing.*;
import javax.swing.event.*;

import smi.awt.AppletFrame;
import smi.awt.JPreferenceList;
import smi.awt.Ping;
import smi.baseConf.RatedString;
import smi.baseConf.CancelException;
import smi.mechanisms.Mechanism;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.ParameterNotCompatibleException;
import smi.io.IOStream;

import ssonet.SSONETConstants;
import ssonet.KeyExchange;
import ssonet.IdentityCheck;
import help.Help;
import gateway.GatewayWindow;

import sun.security.util.BigInt;
import sun.security.util.DerValue;
import sun.security.util.DerOutputStream;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

final public class BaseConfiguration extends JApplet implements ActionListener {

    private JButton cancelButton;
    private JTabbedPane tabbedPane, accoutabilityTabbedPane;
    private String[] tabHeadKey = {
        "CONFIDENTIALITY",
	"ANONYMITY",
	"INTEGRITY",
	"ACCOUNTABILITY",
    };

    private JMenu menu[] = new JMenu[5];
    private String menuTextKey[] = {
        "SETTINGS",
	"KEYEXCHANGE",
	"HELP",
	"LANGUAGE",
	"KEYS"
    };

    private JMenuItem menuitem[] = new JMenuItem[8];
    private String menuitemTextKey[] =
    {
        "PERFORMANCE_TEST",
	"RATING",
	"TEST_CERTIFICATES",
	"NEW_CERTIFICATE",
	"NEW_CIPHERKEY",
	"HELP",
	"INFO",
	"PROVIDER"
    };

    private JCheckBoxMenuItem checkBoxMenuItem[] = new JCheckBoxMenuItem[2];
    private String checkBoxMenuItemTextKey[] = {"GERMAN","ENGLISH"};

    private static String PERFORMANCE_TEST = "PERFORMANCE_TEST";
    private static String INFO = "INFO";
    private static String HELP = "HELP";
    private static String RATING = "RATING";
    private static String PROVIDER = "PROVIDER";
    private static String TEST_CERTIFICATES = "TEST_CERTIFICATES";
    private static String NEW_CERTIFICATE = "NEW_CERTIFICATE";
    private static String NEW_CIPHERKEY = "NEW_CIPHERKEY";
    private static String HELPANONYMITY = "help/hilfetexte/hilfeAnonymitt.html";
    private static String HELPCONFIDENTIALITY = "help/hilfetexte/hilfeVertraulichkeit.html";
    private static String HELPINTEGRITY = "help/hilfetexte/hilfeVertraulichkeit.html";
    private static String HELPACCOUNTABILITY = "help/hilfetexte/hilfeZurechenbarkeit.html";

    private JLabel confidetialityPanelLabel, integrityPanelLabel, anonymityPanelLabel;

    private JCheckBox askBeforeSignCheckBox;
    private JRadioButton accountabilitySignRadioButton = new JRadioButton();
    private JRadioButton accountabilityAcceptRadioButton = new JRadioButton();
    private static Help hp = null;
    private static ResourceBundle resource = null;

    private JPreferenceList
	confidentialityPreferenceList,
	accountabilitySignPreferenceList,
	accountabilityAcceptSignPreferenceList,
	anonymityPreferenceList,
	integrityPreferenceList;

    private JButton changeConfigurationButton[] = new JButton[4];
    private JButton helpButton[] = new JButton[5];
    private String changeConfigurationButtonTextKey[] =
    {
        "DETAIL",
	"DETAIL",
	"DETAIL",
	"DETAIL"
    };

    private AppletFrame appletFrame = null;
    private Hashtable baseConfiguration;


    private class ButtonPanel extends JPanel implements ActionListener {
        private JButton okButton;
        public ButtonPanel() {
            super();
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            GridBagLayout gridBagLayout = new GridBagLayout();
            this.setLayout(gridBagLayout);

            gridBagConstraints.insets = new Insets(10,10,10,10);
            gridBagConstraints.anchor = GridBagConstraints.CENTER;
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 0;
            gridBagConstraints.gridwidth = 1;
            gridBagConstraints.gridheight = 1;
            gridBagConstraints.fill = GridBagConstraints.NONE;
            gridBagConstraints.ipadx = 0;
            gridBagConstraints.ipady = 0;
            gridBagConstraints.weightx = 0.333;
            gridBagConstraints.weighty = 1;

            this.okButton = new JButton("     Ok     ");
            gridBagLayout.setConstraints(okButton, gridBagConstraints);
            this.add(okButton);
            okButton.addActionListener(this);

            gridBagConstraints.gridx = 1;
            cancelButton = new JButton();
            gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
            this.add(cancelButton);
            cancelButton.addActionListener(this);
        }

        public void actionPerformed(ActionEvent event) {
            if(event.getSource() == okButton) {
                baseConfiguration.put("preflist.confidentiality",
		    confidentialityPreferenceList.getPreferenceListItems());
                baseConfiguration.put("preflist.accountability.sign",
		    accountabilitySignPreferenceList.getPreferenceListItems());
                baseConfiguration.put("preflist.accountability.accept",
		    accountabilityAcceptSignPreferenceList.getPreferenceListItems());
                baseConfiguration.put("preflist.anonymity",
		    anonymityPreferenceList.getPreferenceListItems());
                baseConfiguration.put("preflist.integrity",
		    integrityPreferenceList.getPreferenceListItems());
                StartUp.writeBaseConfiguration(baseConfiguration);
            }
            appletFrame.dispose();
            System.exit(0);
        }
    }

    private void changeResource(ResourceBundle resource) {
        for(int i = 0; i < changeConfigurationButton.length; i++)
            changeConfigurationButton[i].setText(resource.getString(changeConfigurationButtonTextKey[i]));
	for(int i = 0; i < helpButton.length; i++)
            helpButton[i].setText(resource.getString("HELP"));
	cancelButton.setText(resource.getString("CANCEL"));
        for(int i = 0; i < tabHeadKey.length; i++)
            tabbedPane.setTitleAt(i, resource.getString(tabHeadKey[i]));

        confidetialityPanelLabel.setText(resource.getString("CHIPHER_MESSAGES"));
        anonymityPanelLabel.setText(resource.getString("USEAGE_OF_MIXES"));
        integrityPanelLabel.setText(resource.getString("USE_AUTHENTICATION"));

        accoutabilityTabbedPane.setTitleAt(0, resource.getString("CREATE_SIGNATURES"));
        accoutabilityTabbedPane.setTitleAt(1, resource.getString("ACCEPT_SIGNATURES"));
        appletFrame.setTitle(resource.getString("BASE CONFIGURATION"));

        for(int i = 0; i < menu.length; i++)
            menu[i].setText(resource.getString(menuTextKey[i]));
        for(int i = 0; i < menuitem.length; i++)
            menuitem[i].setText(resource.getString(menuitemTextKey[i]));
        for(int i = 0; i < checkBoxMenuItem.length; i++)
            checkBoxMenuItem[i].setText(resource.getString(checkBoxMenuItemTextKey[i]));

        confidentialityPreferenceList.setHeader(resource.getString("PREFERENCELIST"),
	    resource.getString("AVAILABLE_MECHANISMS"));

        accountabilitySignPreferenceList.setHeader(resource.getString("PREFERENCELIST"),
	    resource.getString("AVAILABLE_MECHANISMS"));

        accountabilityAcceptSignPreferenceList.setHeader(resource.getString("PREFERENCELIST"),
	    resource.getString("AVAILABLE_MECHANISMS"));

        anonymityPreferenceList.setHeader(resource.getString("CHOSEN_ROUTE"),
	    resource.getString("AVAILABLE_MIXES"));

        integrityPreferenceList.setHeader(resource.getString("PREFERENCELIST"),
	    resource.getString("AVAILABLE_MECHANISMS"));

        askBeforeSignCheckBox.setText(resource.getString("ASK_BEFORE_SIGN"));

        accountabilitySignRadioButton.setText(resource.getString("CREATE_SIGNATURES"));
        accountabilityAcceptRadioButton.setText(resource.getString("ACCEPT_SIGNATURES"));

        validate();
    }

    /** creates a mechanism without key exchange, e.g. for performance tests or similar
    */
    public static IOStream getMechanismInstance(int mechanismType,
	String mechanismName,
	IOStream ioStream) throws ParameterNotCompatibleException
    {
        try {
            String packageName = null;
            if(mechanismType == SSONETConstants.CONFIDENTIALITY)
                packageName = "confidentiality.";
            else if(mechanismType == SSONETConstants.ACCOUNTABILITY_SIGN||
                mechanismType == SSONETConstants.ACCOUNTABILITY_ACCEPT)
                packageName = "accountability.";
            else if(mechanismType == SSONETConstants.INTEGRITY)
                packageName = "integrity.";
            else {
                throw new ParameterNotCompatibleException();
            }
            Mechanism mechanism = (Mechanism)Class.forName("smi.mechanisms." + packageName + mechanismName).newInstance();
            mechanism.init(ioStream, getMechanismConfiguration(mechanismType, mechanismName));
            return mechanism;
        } catch(Exception e) {
	    e.printStackTrace();
            throw new ParameterNotCompatibleException();
        }
    }

    /** creates a mechanism with key exchange
    */
    public static IOStream getMechanismInstance(int mechanismType,
	String mechanismName,
	IOStream ioStream,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws ParameterNotCompatibleException
    {
        try {
            String packageName = null;
            if(mechanismType == SSONETConstants.CONFIDENTIALITY)
                packageName = "confidentiality.";
            else if((mechanismType == SSONETConstants.ACCOUNTABILITY_SIGN) ||
                (mechanismType == SSONETConstants.ACCOUNTABILITY_ACCEPT))
                packageName = "accountability.";
            else if(mechanismType == SSONETConstants.INTEGRITY)
                packageName = "integrity.";
            else throw new ParameterNotCompatibleException();
            System.out.println("*"+"smi.mechanisms." + packageName + mechanismName);
            MechanismConfiguration mechanismconfiguration=null;
            mechanismconfiguration = (MechanismConfiguration)getMechanismConfiguration(mechanismType, mechanismName);
            String gatewayName = ((String[])mechanismconfiguration.get("Providerlist"))[0];
            System.out.println("BaseConfig: mechanismconfiguration erhalten.");
            System.out.println("Provider: "+gatewayName);
            if(gatewayName.equals("Cryptix"))
            {
                Mechanism mechanism = (Mechanism)Class.forName("smi.mechanisms." + packageName + mechanismName).newInstance();
                System.out.println("**");
                mechanism.init(ioStream,
		    getMechanismConfiguration(mechanismType, mechanismName),
		    partnerMechanismConfiguration,
		    keyExchange);
                System.out.println("***");
                return mechanism;
            } else {
		//? hier eine Art Gatewaymechanismus instanziieren.
		//dieser hat die Aufgabe, 1. die Aushandlung zu vollziehen und dann
		//den iostream vom Partner zum Gateway umzuleiten und diesen dann
		//im brauchbaren Format (Rckmechanismus) zu empfangen und auszugeben.
		System.out.println("init Gateway.");
		Hashtable configuration = GatewayWindow.readGatewayConfiguration();
		MechanismConfiguration gwHashConfig = new MechanismConfiguration();
		MechanismConfiguration mechs = new MechanismConfiguration();
		String[] gwConfig = (String[])configuration.get(gatewayName);
		//System.out.println("gwconfig: 1. put: gatewayName=" + gatewayName);
		mechs.put("BACK", getMechanismConfiguration(mechanismType, gwConfig[GatewayWindow.GW_BACK_MECH]));
		//System.out.println("gwconfig: 2. put: BackMech="+ gwConfig[GatewayWindow.GW_BACK_MECH]);
		mechs.put("HIN", getMechanismConfiguration(mechanismType, gwConfig[GatewayWindow.GW_HIN_MECH]));
		//System.out.println("gwconfig: 3. put: HinMech="+ gwConfig[GatewayWindow.GW_HIN_MECH]);
		//System.out.println("Hinmech-Variante = " + ((String[])getMechanismConfiguration(mechanismType, gwConfig[GatewayWindow.GW_HIN_MECH]).get("KeyVariante"))[0] );
		gwHashConfig.put("GATEWAY", gwConfig);
		//System.out.println("gwconfig: 4. put");
		gwHashConfig.put("PACKAGENAME", packageName);
		//Mechanism gw = (Mechanism)Class.forName("smi.mechanisms." + packageName + "Gateway").newInstance();
		Mechanism gw;
		if (packageName.equals("confidentiality.")) {
		    gw = (Mechanism)Class.forName("smi.mechanisms.confidentiality.Gateway").newInstance();
		}
		else if (packageName.equals("accountability.")) {
                    System.out.println("accountability Gateway erzeugen");
                    gw = (Mechanism)Class.forName("smi.mechanisms.accountability.Gateway").newInstance();
                    System.out.println("accountability Gateway generiert");
                } else {
                    gw = (Mechanism)Class.forName("smi.mechanisms.Gateway").newInstance();
                }
                System.out.println("GW.init aufrufen");
                gw.init(ioStream,
		    mechs,
		    gwHashConfig,
		    keyExchange);
                System.out.println("Bis hierhin erfolgreich!");
                return gw;
            }
        } catch (ParameterNotCompatibleException e) {
            throw e;
        } catch (Exception e) {
            System.out.println("Could not instantiate mechanisms");
            e.printStackTrace();
            return null;
        }
    }

    public static MechanismConfiguration getMechanismConfiguration (int mechanismType, String mechanismName) {
        Hashtable baseConfiguration = StartUp.getBaseConfiguration();
        if(baseConfiguration == null) {
            System.out.println("Configuration not found.");
            return null;
        }
        MechanismConfiguration mechanismConfiguration = null;
        if(mechanismType == SSONETConstants.CONFIDENTIALITY)
            mechanismConfiguration = (MechanismConfiguration)baseConfiguration.get(mechanismName + ".configuration");
        else if(mechanismType == SSONETConstants.INTEGRITY)
            mechanismConfiguration = (MechanismConfiguration)baseConfiguration.get
	    ("integrity." + mechanismName + ".configuration");
        else if(mechanismType == SSONETConstants.ACCOUNTABILITY_SIGN)
            mechanismConfiguration = (MechanismConfiguration)baseConfiguration.get
	    ("signatures.sign." + mechanismName + ".configuration");
        else if(mechanismType == SSONETConstants.ACCOUNTABILITY_ACCEPT)
            mechanismConfiguration = (MechanismConfiguration)baseConfiguration.get
	    ("signatures.accept." + mechanismName + ".configuration");
        else return null;
        if(mechanismConfiguration != null)
            return mechanismConfiguration;
        String packageName = null;
        if(mechanismType == SSONETConstants.CONFIDENTIALITY)
            packageName = "confidentiality.";
        else if((mechanismType == SSONETConstants.ACCOUNTABILITY_SIGN) ||
            (mechanismType == SSONETConstants.ACCOUNTABILITY_ACCEPT))
            packageName = "accountability.";
        else if(mechanismType == SSONETConstants.INTEGRITY)
            packageName = "integrity.";
        else return null;
        try {
            Mechanism mechanism = (Mechanism)Class.forName("smi.mechanisms." + packageName + mechanismName).newInstance();
            return mechanism.getDefaultConfiguration();
        } catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    /** get a preference list ....
    */
    public static RatedString[] getPreferenceList(int securityGoal)
    {
        try {
            Hashtable baseConfiguration = StartUp.getBaseConfiguration();
            if(baseConfiguration == null)
            {
                System.out.println("Configuration not found...");
                return null;
            }
            String[] preferenceList = null;
            if(securityGoal == SSONETConstants.CONFIDENTIALITY)
                preferenceList = (String[])baseConfiguration.get("preflist.confidentiality");
            else if(securityGoal == SSONETConstants.INTEGRITY)
                preferenceList = (String[])baseConfiguration.get("preflist.integrity");
            else if(securityGoal == SSONETConstants.ACCOUNTABILITY_SIGN)
                preferenceList = (String[])baseConfiguration.get("preflist.accountability.sign");
            else
                return null;
            return RatedString.toRatedString(preferenceList);
        } catch(Exception e) {
            e.printStackTrace();
            return null;
        }
    }

    private void addMenu() {
        JMenuBar menuBar = new JMenuBar();

        for (int i = 0; i < menu.length; i++)
            menu[i] = new JMenu();
        for (int i = 0; i < menuitem.length; i++)
            menuitem[i] = new JMenuItem();
        ButtonGroup languageButtonGroup = new ButtonGroup();
        for (int i = 0; i < checkBoxMenuItem.length; i++) {
            checkBoxMenuItem[i] = new JCheckBoxMenuItem();
            languageButtonGroup.add(checkBoxMenuItem[i]);
        }

        checkBoxMenuItem[0].setState(true);

        menu[0].add(menu[3]);                   // Settings ... Language
        menu[3].add(checkBoxMenuItem[0]);           // Language... German
        checkBoxMenuItem[0].addActionListener(this);
        menu[3].add(checkBoxMenuItem[1]);           // Language... English
        checkBoxMenuItem[1].addActionListener(this);

	menu[0].add(menuitem[7]);               //Settings ... Provider
	menuitem[7].addActionListener(this);
	menuitem[7].setActionCommand(PROVIDER);

        menu[0].add(menuitem[0]);               //Settings ... Performancetest
        menuitem[0].addActionListener(this);
        menuitem[0].setActionCommand(PERFORMANCE_TEST);

        menu[0].add(menuitem[1]);               //Settings ... Rating
        menuitem[1].addActionListener(this);
        menuitem[1].setActionCommand(RATING);
        menuBar.add(menu[0]);

        menu[1].add(menuitem[2]);               //Keyexchange ... Test Certificates
        menuitem[2].addActionListener(this);
        menuitem[2].setActionCommand(TEST_CERTIFICATES);

        menu[1].add(menu[4]);                   //Schlssel -->

        menu[4].add(menuitem[4]);               //Keyexchange ... New Cipherkey
        menuitem[4].addActionListener(this);
        menuitem[4].setActionCommand(NEW_CIPHERKEY);
        menuBar.add(menu[1]);

	menu[4].add(menuitem[3]);               //Keyexchange ... New Certificates
        menuitem[3].addActionListener(this);
        menuitem[3].setActionCommand(NEW_CERTIFICATE);

        menu[2].add(menuitem[5]);               //Help ... Help
	menuitem[5].addActionListener(this);
	menuitem[5].setActionCommand(HELP);
	menuBar.add(menu[2]);

	menu[2].add(menuitem[6]);               //Help ... Info
        menuitem[6].addActionListener(this);
        menuitem[6].setActionCommand(INFO);
        menuBar.add(menu[2]);

        setJMenuBar(menuBar);
    }


    class PreferenceListSelectionListener implements ListSelectionListener {
        private JList leftList, rightList;
        private JButton button;

        public PreferenceListSelectionListener(JList leftList, JList rightList, JButton button) {
            this.leftList = leftList;
            this.leftList.addListSelectionListener(this);
            this.rightList = rightList;
            this.rightList.addListSelectionListener(this);
            this.button = button;
        }

        public void valueChanged(ListSelectionEvent event) {
            if(((JList)event.getSource()).getSelectedIndex() != -1) {
                if(event.getSource() == this.leftList)
                    this.button.setEnabled(true);
                else
                    this.button.setEnabled(false);
            }
        }
    }


    private String getMechanismsDir() {
        return "./smi/mechanisms";
    }

    private void initialize() {
        setBackground(Color.lightGray);
        StartUp startup = null;
        try {
            startup = new StartUp(appletFrame);
            startup.start();
            startup.join();
            baseConfiguration = startup.getBaseConfiguration();
        } catch (Exception e) {
            System.out.println("Error in Startup!");
            e.printStackTrace();
            System.exit(0);
        }
        if(baseConfiguration == null)
            baseConfiguration = new Hashtable();

        addMenu();

        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        tabbedPane = new JTabbedPane(JTabbedPane.TOP);

        /*------------------------- Panel Confidentiality ----------------------- */
        //--------------------------------------------------------------------------

        JPanel confidentialityPanel = new JPanel();
        confidentialityPanel.setLayout(gridBagLayout);

        // Label ...
        confidetialityPanelLabel = new JLabel();
        gridBagConstraints.insets = new Insets(5,10,0,0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(confidetialityPanelLabel, gridBagConstraints);
        confidentialityPanel.add(confidetialityPanelLabel);


        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.insets = new Insets(5,10,0,10);
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        confidentialityPreferenceList = new JPreferenceList(12,
	    (String[])baseConfiguration.get("preflist.confidentiality"),
	    startup.getMechanisms());
        gridBagLayout.setConstraints(confidentialityPreferenceList, gridBagConstraints);
        confidentialityPanel.add(confidentialityPreferenceList);

	JPanel buttonConfPanel = new JPanel();
	buttonConfPanel.setLayout(new BoxLayout(buttonConfPanel, BoxLayout.X_AXIS));
        changeConfigurationButton[0] = new JButton();
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(15,30,15,0);
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(buttonConfPanel, gridBagConstraints);
        buttonConfPanel.add(changeConfigurationButton[0]);
        changeConfigurationButton[0].addActionListener(this);
        changeConfigurationButton[0].setEnabled(false);
        changeConfigurationButton[0].setToolTipText("Click to configure the selected mechanism.");
        new PreferenceListSelectionListener(confidentialityPreferenceList.getPreferenceList(),
	    confidentialityPreferenceList.getList(),
	    changeConfigurationButton[0]);
	buttonConfPanel.add(Box.createRigidArea(new Dimension(100,1)));
	helpButton[0] = new JButton();
	helpButton[0].addActionListener(this);
	buttonConfPanel.add(helpButton[0]);
        confidentialityPanel.add(buttonConfPanel);
        /*------------------------- Panel Anonymity ------------------------- */
        //----------------------------------------------------------------------

	JPanel anonymityPanel = new JPanel();
        anonymityPanel.setLayout(gridBagLayout);
        anonymityPanelLabel = new JLabel();
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(5,10,0,0);
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(anonymityPanelLabel, gridBagConstraints);
        anonymityPanel.add(anonymityPanelLabel);


        anonymityPreferenceList = new JPreferenceList(12,
	    (String[])baseConfiguration.get("preflist.anonymity"),
	    startup.getMixe());
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.insets = new Insets(5,10,0,10);
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(anonymityPreferenceList, gridBagConstraints);
        anonymityPanel.add(anonymityPreferenceList);


        JPanel buttonAnonPanel = new JPanel();
        buttonAnonPanel.setLayout(new BoxLayout(buttonAnonPanel, BoxLayout.X_AXIS));
	gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(15,30,15,0);
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(buttonAnonPanel, gridBagConstraints);
        buttonAnonPanel.add(Box.createRigidArea(new Dimension(260,1)));
	helpButton[4] = new JButton();
	helpButton[4].addActionListener(this);
	buttonAnonPanel.add(helpButton[4]);
        anonymityPanel.add(buttonAnonPanel);

        /*------------------------- Panel Integrity ------------------------- */
        //----------------------------------------------------------------------

        JPanel integrityPanel = new JPanel();
        integrityPanel.setLayout(gridBagLayout);

        integrityPanelLabel = new JLabel();
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(5,10,0,0);
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(integrityPanelLabel, gridBagConstraints);
        integrityPanel.add(integrityPanelLabel);


        integrityPreferenceList = new JPreferenceList(12,
	    (String[])baseConfiguration.get("preflist.integrity"),
	    startup.getMechanismsIntegrity());
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new Insets(5,10,0,10);
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(integrityPreferenceList, gridBagConstraints);
        integrityPanel.add(integrityPreferenceList);

	JPanel buttonIntePanel = new JPanel();
        buttonIntePanel.setLayout(new BoxLayout(buttonIntePanel, BoxLayout.X_AXIS));
        changeConfigurationButton[3] = new JButton();
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new Insets(15,30,15,0);
        gridBagConstraints.fill = GridBagConstraints. NONE;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(buttonIntePanel, gridBagConstraints);
        buttonIntePanel.add(changeConfigurationButton[3]);
        changeConfigurationButton[3].addActionListener(this);
        changeConfigurationButton[3].setEnabled(false);
        changeConfigurationButton[3].setToolTipText("Click to configure the selected mechanism.");
        new PreferenceListSelectionListener(integrityPreferenceList.getPreferenceList(),
	    integrityPreferenceList.getList(),
	    changeConfigurationButton[3]);
	buttonIntePanel.add(Box.createRigidArea(new Dimension(100,1)));
	helpButton[3] = new JButton();
	helpButton[3].addActionListener(this);
	buttonIntePanel.add(helpButton[3]);
        integrityPanel.add(buttonIntePanel);


        /*------------------------- Panel Accountability ------------------------- */
        //---------------------------------------------------------------------------

        // Panel create Signatures

        JPanel createSignaturesPanel = new JPanel();

        createSignaturesPanel.setLayout(gridBagLayout);
        createSignaturesPanel.setDoubleBuffered(false);

        accountabilitySignPreferenceList =
	    new JPreferenceList(7,
	    (String[])baseConfiguration.get("preflist.accountability.sign"),
	    startup.getSignatures());
        gridBagConstraints.insets = new Insets(5,5,0,10);
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(accountabilitySignPreferenceList, gridBagConstraints);
        createSignaturesPanel.add(accountabilitySignPreferenceList);

	JPanel buttonAccoSignPanel = new JPanel();
        buttonAccoSignPanel.setLayout(new BoxLayout(buttonAccoSignPanel, BoxLayout.X_AXIS));
	changeConfigurationButton[1] = new JButton();
        changeConfigurationButton[1].setEnabled(false);
        gridBagConstraints.insets = new Insets(10,25,0,0);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(buttonAccoSignPanel, gridBagConstraints);
        buttonAccoSignPanel.add(changeConfigurationButton[1]);
        changeConfigurationButton[1].addActionListener(this);
        new PreferenceListSelectionListener(accountabilitySignPreferenceList.getPreferenceList(),
	    accountabilitySignPreferenceList.getList(),
	    changeConfigurationButton[1]);
	buttonAccoSignPanel.add(Box.createRigidArea(new Dimension(100,1)));
	helpButton[1] = new JButton();
	helpButton[1].addActionListener(this);
	buttonAccoSignPanel.add(helpButton[1]);
        createSignaturesPanel.add(buttonAccoSignPanel);

        askBeforeSignCheckBox = new JCheckBox("", true);
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new Insets(5,5,0,0);
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagLayout.setConstraints(askBeforeSignCheckBox, gridBagConstraints);
        createSignaturesPanel.add(askBeforeSignCheckBox);


        // Panel accept Signatures
        JPanel acceptSignaturesPanel = new JPanel();

        acceptSignaturesPanel.setLayout(gridBagLayout);
        acceptSignaturesPanel.setDoubleBuffered(false);

        accountabilityAcceptSignPreferenceList =
	    new JPreferenceList(7,
	    (String[])baseConfiguration.get("preflist.accountability.accept"),
	    startup.getSignatures());
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.insets = new Insets(5,5,0,10);
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(accountabilityAcceptSignPreferenceList, gridBagConstraints);
        acceptSignaturesPanel.add(accountabilityAcceptSignPreferenceList);

        JPanel buttonAccoAcceptPanel = new JPanel();
        buttonAccoAcceptPanel.setLayout(new BoxLayout(buttonAccoAcceptPanel, BoxLayout.X_AXIS));
	changeConfigurationButton[2] = new JButton();
        changeConfigurationButton[2].setEnabled(false);
        changeConfigurationButton[2].addActionListener(this);
        gridBagConstraints.anchor = GridBagConstraints.WEST;
        gridBagConstraints.insets = new Insets(10,25,10,0);
        gridBagConstraints.insets.left = 20;
        gridBagConstraints.insets.top = 10;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 0;
        gridBagLayout.setConstraints(buttonAccoAcceptPanel, gridBagConstraints);
        buttonAccoAcceptPanel.add(changeConfigurationButton[2]);
        new PreferenceListSelectionListener(accountabilityAcceptSignPreferenceList.getPreferenceList(),
	    accountabilityAcceptSignPreferenceList.getList(),
	    changeConfigurationButton[2]);
        buttonAccoAcceptPanel.add(Box.createRigidArea(new Dimension(100,1)));
	helpButton[2] = new JButton();
	helpButton[2].addActionListener(this);
	buttonAccoAcceptPanel.add(helpButton[2]);
        acceptSignaturesPanel.add(buttonAccoAcceptPanel);

        // Mainpanel for Accountability
        accoutabilityTabbedPane = new JTabbedPane();
        accoutabilityTabbedPane.add(createSignaturesPanel);
        accoutabilityTabbedPane.add(acceptSignaturesPanel);

        tabbedPane.add(confidentialityPanel);
        tabbedPane.add(anonymityPanel, getClass().getResource("images/red.gif"));
        tabbedPane.add(integrityPanel, getClass().getResource("images/green.gif"));
        tabbedPane.add(accoutabilityTabbedPane, getClass().getResource("images/red.gif"));
        tabbedPane.setIconAt(0, new ImageIcon(getClass().getResource("images/red.gif")));
        tabbedPane.setIconAt(1, new ImageIcon(getClass().getResource("images/red.gif")));
        tabbedPane.setIconAt(2, new ImageIcon(getClass().getResource("images/green.gif")));
        tabbedPane.setIconAt(3, new ImageIcon(getClass().getResource("images/green.gif")));


        getContentPane().setLayout(new BorderLayout());
        getContentPane().add("Center", tabbedPane);
        getContentPane().add("South", new ButtonPanel());
        getContentPane().add("North", new JPanel());
        getContentPane().add("West", new JPanel());
        getContentPane().add("East", new JPanel());

	resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.GERMAN);
        changeResource(resource);
    }

    public static void main(String  argv[]) {
	BaseConfiguration baseConfiguration = new BaseConfiguration();
	baseConfiguration.init();
    }

    public void init() {
	appletFrame = new AppletFrame("BASE CONFIGURATION");
	initialize();
	start();
	appletFrame.getContentPane().add(this);
	appletFrame.pack();
	appletFrame.setSize(appletFrame.getSize().width+70, appletFrame.getSize().height + 50);
	appletFrame.center();
	appletFrame.show();
    }

    public void actionPerformed(ActionEvent event) {
	Object object = event.getSource();

	if(object == changeConfigurationButton[0]) {
	    try {
		String mechanismName = (String)confidentialityPreferenceList.getPreferenceList().getSelectedValue();
		Mechanism mechanism = (Mechanism)Class.forName
		    ("smi.mechanisms.confidentiality." + mechanismName).newInstance();
		MechanismConfiguration mechanismConfiguration = mechanism.configDetail
		    (appletFrame,
		    (MechanismConfiguration)baseConfiguration.get(mechanismName + ".configuration"));
		System.out.println(mechanismConfiguration);
		baseConfiguration.put(mechanismName + ".configuration", mechanismConfiguration);
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	} else if(object == changeConfigurationButton[1]) {
	    try {
		String mechanismName = (String)accountabilitySignPreferenceList.getPreferenceList().getSelectedValue();
		Mechanism mechanism = (Mechanism)Class.forName
		    ("smi.mechanisms.accountability." + mechanismName).newInstance();
		MechanismConfiguration mechanismConfiguration = mechanism.configDetail
		    (appletFrame,
		    (MechanismConfiguration)baseConfiguration.get("signatures.sign." + mechanismName + ".configuration"));
		System.out.println(mechanismConfiguration);
		baseConfiguration.put("signatures.sign." + mechanismName + ".configuration", mechanismConfiguration);
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	} else if(object == changeConfigurationButton[2]) {
	    try {
		String mechanismName = (String)accountabilityAcceptSignPreferenceList.getPreferenceList().getSelectedValue();
		Mechanism mechanism = (Mechanism)Class.forName
		    ("smi.mechanisms.accountability." + mechanismName).newInstance();
		MechanismConfiguration mechanismConfiguration = mechanism.configDetail
		    (appletFrame,
		    (MechanismConfiguration)baseConfiguration.get("signatures.accept." + mechanismName + ".configuration"));
		System.out.println(mechanismConfiguration);
		baseConfiguration.put("signatures.accept." + mechanismName + ".configuration", mechanismConfiguration);
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	} else if(object == changeConfigurationButton[3]) {
	    try {
		String mechanismName = (String)integrityPreferenceList.getPreferenceList().getSelectedValue();
		Mechanism mechanism = (Mechanism)Class.forName("smi.mechanisms.integrity." + mechanismName).newInstance();
		MechanismConfiguration mechanismConfiguration = mechanism.configDetail
		    (appletFrame,
		    (MechanismConfiguration)baseConfiguration.get("integrity." + mechanismName + ".configuration"));
		System.out.println(mechanismConfiguration);
		baseConfiguration.put("integrity." + mechanismName + ".configuration", mechanismConfiguration);
	    } catch(Exception e) {
		e.printStackTrace();
	    }
	}
	else if(object == helpButton[0]) {
	    //Confidentiality
	    String str = (String)confidentialityPreferenceList.getList().getSelectedValue(); //rechte Liste
	    if(str == null)
		str = (String)confidentialityPreferenceList.getPreferenceList().getSelectedValue();//linke Liste
	    viewHelp("CONFIDENTIALITY", str);
	}
	else if(object == helpButton[1]) {
	    //Accountability Sign
	    String str = (String)accountabilitySignPreferenceList.getList().getSelectedValue(); //rechte Liste
	    if(str == null)
		str = (String)accountabilitySignPreferenceList.getPreferenceList().getSelectedValue();//linke Liste
	    viewHelp("ACCOUNTABILITY", str);
	}
	else if(object == helpButton[2]) {
	    //Accountability Accept
	    String str = (String)accountabilityAcceptSignPreferenceList.getList().getSelectedValue(); //rechte Liste
	    if(str == null)
		str = (String)accountabilityAcceptSignPreferenceList.getPreferenceList().getSelectedValue();//linke Liste
	    viewHelp("ACCOUNTABILITY", str);
	}
	else if(object == helpButton[3]) {
	    // Integrity
	    String str = (String)integrityPreferenceList.getList().getSelectedValue(); //rechte Liste
	    if(str == null)
		str = (String)integrityPreferenceList.getPreferenceList().getSelectedValue();//linke Liste
	    viewHelp("INTEGRITY", str);
	}
	else if(object == helpButton[4]) {
	    //Anonymity
	    String str = (String)anonymityPreferenceList.getList().getSelectedValue(); //rechte Liste
	    if(str == null)
		str = (String)anonymityPreferenceList.getPreferenceList().getSelectedValue();//linke Liste
	    viewHelp("ANONYMITY", str);
	}

	else if(object == checkBoxMenuItem[0]) {
	    resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.GERMAN);
	    changeResource(resource);
	}
	else if(object == checkBoxMenuItem[1]) {
	    resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.ENGLISH);
	    changeResource(resource);
	} else {
	    if(event.getActionCommand().equals(HELP)) {
		viewHelp("GRUNDKONFIGURATION","");
	    }
	    if(event.getActionCommand().equals(INFO))
		new AboutBaseConfiguration(appletFrame);
	    else if(event.getActionCommand().equals(PERFORMANCE_TEST))
		(new PerformanceTest()).start();
	    else if(event.getActionCommand().equals(RATING))
		new rating.Rating();
	    else if(event.getActionCommand().equals(PROVIDER)) {
		GatewayWindow aGateway = new GatewayWindow();
		aGateway.showGatewayFrame();
	    } else if(event.getActionCommand().equals(TEST_CERTIFICATES)) {
		try {
		    System.out.println("reading trustcenter certificate:");
		    X509Cert trustCenterCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/TrustCenterCertificate.der");
		    System.out.println(trustCenterCertificate.toString());
		    PublicKey trustCenterPublicKey = trustCenterCertificate.getPublicKey();

		    System.out.println("testing trustcenter certificate.");
		    if (!KeyExchange.testCertificate(trustCenterCertificate, trustCenterPublicKey))
		    {
			System.out.println("TrustCenter Certificate not valid!");
			throw new InvalidCertificatException();
		    }
		    if (!IdentityCheck.getAnswer("TrustCenter certificate", trustCenterCertificate, appletFrame))
		    {
			System.out.println("TrustCenter Certificate not valid!");
			throw new InvalidCertificatException();
		    }

		    System.out.println("reading testCertificate:");
		    X509Cert testCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeDSACertificate.der");
		    System.out.println(testCertificate.toString());
		    PublicKey ownPublicKey = testCertificate.getPublicKey();

		    System.out.println("testing testCertificate.");
		    if (!KeyExchange.testCertificate(testCertificate, trustCenterPublicKey))
		    {
			System.out.println("Certificate for TestKey not valid!");
			throw new InvalidCertificatException();
		    }
		    if (!IdentityCheck.getAnswer("Certificate for sign key", testCertificate, appletFrame))
		    {
			System.out.println("Certificate for TestKey not valid!");
			throw new InvalidCertificatException();
		    }

		    System.out.println("reading cipherCertificate:");
		    X509Cert cipherCertificate = KeyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeRSACertificate.der");
		    System.out.println(cipherCertificate.toString());

		    System.out.println("testing cipherCertificate.");
		    if (!KeyExchange.testCertificate(cipherCertificate, ownPublicKey))
		    {
			System.out.println("Certificate for CipherKey not valid!");
			throw new InvalidCertificatException();
		    }
		    if (!IdentityCheck.getAnswer("Certificate for chipher key", cipherCertificate, appletFrame))
		    {
			System.out.println("Certificate for CipherKey not valid!");
			throw new InvalidCertificatException();
		    }
		    JOptionPane.showMessageDialog(this, resource.getString("VALID KEY MESSAGE"), resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
		} catch (InvalidCertificatException ice) {
		    JOptionPane.showMessageDialog(this, resource.getString("INVALID KEY MESSAGE"), resource.getString("WARNING"), JOptionPane.ERROR_MESSAGE);
		}
	    } else if(event.getActionCommand().equals(NEW_CERTIFICATE)) {
		createNewCertificate();
	    } else if(event.getActionCommand().equals(NEW_CIPHERKEY)) {
		createKeyPair();
	    }
	}
    }

    private void createKeyPair() {
	try {
	    KeyPairGenerator keyPairGenerator = KeyPairGenerator.getInstance("RSA");
	    System.out.println("initialize KeyPairGenerator for 1024 Bit KeyPair");
	    keyPairGenerator.initialize(1024);
	    System.out.println("generate KeyPair.");
	    KeyPair ownKeyPair = keyPairGenerator.generateKeyPair();

	    System.out.println("public key:\n" + ownKeyPair.getPublic().toString());
	    System.out.println("\nprivate key:\n" + ownKeyPair.getPrivate().toString());

	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeRSAPublicKey.der");
	    System.out.println("writing public key to keys/KeyExchangeRSAPublicKey.der");
	    fileOutputStream.write(ownKeyPair.getPublic().getEncoded());
	    fileOutputStream.close();

	    fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeRSAPrivateKey.der");
	    System.out.println("writing private key to keys/KeyExchangeRSAPrivateKey.der");
	    fileOutputStream.write(ownKeyPair.getPrivate().getEncoded());
	    fileOutputStream.close();
	    JOptionPane.showMessageDialog(this, resource.getString("CREATE KEY MESSAGE"), resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }

    protected void createNewCertificate() {
	try {
	    System.out.println("reading sign key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeDSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey privateKey = (PrivateKey)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();

	    System.out.println("reading sign certificate");
	    try {
		fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeDSACertificate.der");
	    } catch (IOException ie) {
		System.out.println ("KeyPair not found.");
		JOptionPane.showMessageDialog (this, resource.getString("WARNING"), resource.getString("NO KEY PAIR"), JOptionPane.ERROR_MESSAGE);
		throw ie;
	    }
	    X509Cert issuerCertificate = new X509Cert();
	    issuerCertificate.decode(fileInputStream);
	    fileInputStream.close();

	    X500Signer signer = issuerCertificate.getSigner(new AlgorithmId(AlgorithmId.DSA_oid), privateKey);
	    X500Name subjectName = issuerCertificate.getSubjectName();

	    GregorianCalendar gregorianCalendar = (GregorianCalendar)Calendar.getInstance();
	    Date notBefore = gregorianCalendar.getTime();

	    String durString = NewCertificateDialog.getAnswer(appletFrame);
	    if (durString == null) throw new CancelException();
	    // int duration = new Integer(NewCertificateDialog.getAnswer(appletFrame)).intValue();
	    int duration = Integer.parseInt(durString);
	    gregorianCalendar.add(Calendar.DAY_OF_YEAR, duration);
	    Date notAfter = gregorianCalendar.getTime();

	    System.out.println("reading public key");
	    fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeRSAPublicKey.der");
	    RawRSAPublicKey rawRSAPublicKey = new RawRSAPublicKey(fileInputStream);
	    byte[] rawKeyData = rawRSAPublicKey.getEncoded();
	    X509Key publicKey = createX509Key("MD5/RSA", rawKeyData);

	    X509Cert certificate = new X509Cert(subjectName, publicKey, notBefore, notAfter);

	    System.out.println("sign certificate");
	    SecureRandom randomGenerator = new SecureRandom();
	    byte[] randomBuffer = new byte[16];
	    randomGenerator.nextBytes(randomBuffer);
	    X509Cert signedCertificate = new X509Cert(certificate.encodeAndSign(new BigInt(randomBuffer), signer));
	    System.out.println("Certifikat:\n" + signedCertificate.toString());

	    System.out.println("writing Certificate");
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")+"keys/KeyExchangeRSACertificate.der");
	    certificate.encode(fileOutputStream);
	    fileOutputStream.close();
	    JOptionPane.showMessageDialog(this, resource.getString("CREATE CERTIFICATE MESSAGE"), resource.getString("MESSAGE"), JOptionPane.INFORMATION_MESSAGE);
	} catch(CancelException ce) {
	} catch(Exception ex) {
	    ex.printStackTrace();
	}
    }

    protected X509Key createX509Key(String algorithmID, byte[] rawKeyData) {
	try {
	    // insert leading zero
	    byte[] rawKeyData2 = new byte[rawKeyData.length + 1];
	    rawKeyData2[0] = 0;
	    for (int i = 0; i < rawKeyData.length; i++) rawKeyData2[i + 1] = rawKeyData[i];

	    byte[] algIDArray = smi.util.Emit.emit(algorithmID);

	    DerValue algDerValue = new DerValue(algIDArray);
	    DerValue keyDerValue = new DerValue(DerValue.tag_BitString, rawKeyData2);

	    DerValue[] both = {algDerValue, keyDerValue};
	    DerOutputStream derOutputStream2 = new DerOutputStream();
	    derOutputStream2.putSequence(both);
	    byte[] bothArray = derOutputStream2.toByteArray();
	    DerValue bothDerValue = new DerValue(bothArray);

	    return (X509Key)X509Key.parse(bothDerValue);
	} catch(Exception e) {
	    e.printStackTrace();
	}
	return null;
    }

    public static void viewHelp(String str1, String str2) {
	if (hp != null) {
	    hp.vanishWindow();
	}
	hp = new Help("file:///"+System.getProperty("ssonet_help")+"gk/HelpSet.hs");
	hp.viewPage (str1, str2);
    }

    public static ResourceBundle getLangResources() {
	if (resource == null)  {
	    resource = ResourceBundle.getBundle("smi.baseConf.locale.Resources", Locale.GERMAN);
	}
	return resource;
    }
}
