/*
 * smi/mechanisms/DetailConfigurationDialog.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms;

import smi.awt.Spinner;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.Toolkit;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.plaf.basic.BasicArrowButton;
import javax.swing.JOptionPane;

import smi.awt.JPreferenceList;
import smi.awt.JSpinner;
import smi.baseConf.BaseConfiguration;
import help.Help;
import gateway.GatewayWindow;

import ssonet.SSONETConstants;

public class DetailConfigurationDialog extends JDialog implements ActionListener {
    private JTabbedPane tabbedPane;
    private JPreferenceList modePreferenceList, providerPreferenceList;
    private JSpinner jSpinner;
    private JButton okButton, helpButton;
    private boolean returnCode = false;
    private String mechanismName;
    private int type = 99;
    private Mechanism mechanism = null;
    private String WARNINGMESSAGEMODE = "Es wurde kein Betriebsmodus eingestellt.";
    private String WARNINGINFOMODE    = "Kein Betriebsmodus";
    private String WARNINGMESSAGEPROVIDER = "Es wurde kein Provider eingestellt.";
    private String WARNINGINFOPROVIDER    = "Kein Provider";
    private JTextField roundsInput = null;
    private ResourceBundle resource = null;
    private GatewayWindow aGateway = null;

    public DetailConfigurationDialog(JFrame parentFrame, String mechanismName) {
        super(parentFrame,"", true);
        aGateway = new GatewayWindow();
        resource = BaseConfiguration.getLangResources();
        System.out.println("Lang.: "+resource);
        this.mechanismName = mechanismName;
        setTitle(resource.getString("DETAIL CONFIGURATION") +" "+ mechanismName.substring(mechanismName.lastIndexOf(".")+1, mechanismName.length()));
        tabbedPane = new JTabbedPane();
        Container contentPane = getContentPane();
        contentPane.add("Center", tabbedPane);

        JPanel panel = new JPanel();
        contentPane.add("North", panel);
        panel = new JPanel();
        contentPane.add("East", panel);
        panel = new JPanel();
        contentPane.add("West", panel);

        // lower buttons...
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        panel = new JPanel();
        panel.setLayout(gridBagLayout);
        gridBagConstraints.insets = new Insets(10,10,10,10);
        gridBagConstraints.anchor = GridBagConstraints.NORTH;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.fill = GridBagConstraints.NONE;
        gridBagConstraints.ipadx = 0;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.weightx = 0.333;
        gridBagConstraints.weighty = 0;

        okButton = new JButton(resource.getString("OK"));  //"     Ok     "
        gridBagLayout.setConstraints(okButton, gridBagConstraints);
        panel.add(okButton);
        okButton.addActionListener(this);

        gridBagConstraints.gridx = 1;
        JButton cancelButton = new JButton(resource.getString("CANCEL"));  //" Cancel "
        gridBagLayout.setConstraints(cancelButton, gridBagConstraints);
        panel.add(cancelButton);
        cancelButton.addActionListener(this);

        gridBagConstraints.gridx = 2;
        helpButton = new JButton(resource.getString("HELP"));  //" Help "
        gridBagLayout.setConstraints(helpButton, gridBagConstraints);
        panel.add(helpButton);
        helpButton.addActionListener(this);

        contentPane.add("South", panel);
    }

    public boolean getReturnCode() {
        return returnCode;
    }

    public void addTabProvider(String[] actualProviders) {
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        JPanel panel = new JPanel();
        panel.setLayout(gridBagLayout);
        try {
            if(!mechanismName.equals("RSA")&&!mechanismName.equals("DSA")
		&&!mechanismName.equals("ElGamal"))
                mechanism = (Mechanism)Class.forName(mechanismName).newInstance();
        } catch (Exception e) {
            e.printStackTrace();
        }
        providerPreferenceList = new JPreferenceList(7,
	    /*resource.getString("PROVIDER LIST")*/"Provider-Liste",
	    /*resource.getString("AVAILABLE PROVIDERS")*/"Verfgbare Provider",
	    actualProviders,
	    aGateway.getGWsupportMech(mechanismName.substring(mechanismName.lastIndexOf(".")+1, mechanismName.length())));

        gridBagConstraints.insets = new Insets(10, 10, 10, 10);
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 0;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(providerPreferenceList, gridBagConstraints);
        panel.add(providerPreferenceList);
        tabbedPane.addTab("Provider"/*resource.getString("PROVIDER")*/, panel);
    }

    public void addTabMode(String[] actualModes, int type) {
        this.type = type;
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        JPanel panel = new JPanel();
        panel.setLayout(gridBagLayout);

        if ((type == SSONETConstants.CONFIDENTIALITY) ||
            (type == SSONETConstants.INTEGRITY)) {
            try {
                mechanism = (Mechanism)Class.forName(mechanismName).newInstance();
            } catch (Exception e) {
                e.printStackTrace();
            }
            modePreferenceList = new JPreferenceList(7,
		resource.getString("PREFERENCE LIST"),
		resource.getString("AVAILABLE MODES"),
		actualModes,
		mechanism.getModes());
        }

        gridBagConstraints.insets = new Insets(10, 10, 10, 10);
        gridBagConstraints.anchor = GridBagConstraints.CENTER;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.fill = GridBagConstraints.BOTH;
        gridBagConstraints.ipadx = 0;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(modePreferenceList, gridBagConstraints);
        panel.add(modePreferenceList);
        tabbedPane.addTab(resource.getString("MODE"), panel);
    }

    public void addTabKey(int minimalValue, int maximalValue, int stepSize, int actualValue) {
        JPanel keyPanel = new JPanel();
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();
        keyPanel.setLayout(gridBagLayout);

        JLabel label = new JLabel(resource.getString("MINIMAL KEY LENGTH"));
        gridBagConstraints.insets.top = 15;
        gridBagConstraints.insets.left = 20;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.weightx = 0;
        gridBagConstraints.weighty = 1;
        gridBagLayout.setConstraints(label, gridBagConstraints);
        keyPanel.add(label);

        gridBagConstraints.insets.top = 17;
        gridBagConstraints.insets.left = 0;
        gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.gridwidth = 1;
        gridBagConstraints.gridheight = 1;
        gridBagConstraints.weightx = 1;
        gridBagConstraints.weighty = 0;
        jSpinner = new JSpinner(actualValue, minimalValue, maximalValue, stepSize);
        gridBagLayout.setConstraints(jSpinner, gridBagConstraints);
        keyPanel.add(jSpinner);

        tabbedPane.addTab(resource.getString("KEY"), keyPanel);
    }

    public void addTab(String tabName, JPanel panel) {
        tabbedPane.addTab(tabName, panel);
    }

    public String[] getModePreferenceList() {
        return modePreferenceList.getPreferenceListItems();
    }

    public String[] getProviderPreferenceList() {
        return providerPreferenceList.getPreferenceListItems();
    }

    public int getKeyLength() {
        return jSpinner.getValue();
    }

    public void actionPerformed(ActionEvent event) {
        Object object = event.getSource();
        if(object == okButton) {
            JPreferenceList list = null;
            for(int i=0; i<tabbedPane.getTabCount(); i++) {
                JPanel p = (JPanel)tabbedPane.getComponentAt(i);
                for(int j=0; j<p.getComponentCount(); j++) {
                    System.out.println(p.getComponent(j).getClass().getName());
                    if(p.getComponent(j).getClass().getName().equals("smi.awt.JPreferenceList"))
			list = (JPreferenceList)p.getComponent(j);
                }
            }

            if((modePreferenceList != null && getModePreferenceList().length == 0) ||
		(list != null && list.getPreferenceListItems().length == 0))
                JOptionPane.showMessageDialog(this, WARNINGMESSAGEMODE, WARNINGINFOMODE, JOptionPane.WARNING_MESSAGE);
            else if(providerPreferenceList != null && getProviderPreferenceList().length == 0)
                JOptionPane.showMessageDialog(this, WARNINGMESSAGEPROVIDER, WARNINGINFOPROVIDER, JOptionPane.WARNING_MESSAGE);
            else {
                returnCode = true;
                dispose();
            }
        }
        else if(object == helpButton) {
            String text = "gk.html";

            for(int i=0; i<tabbedPane.getTabCount(); i++) {
                String str = null;
                if(tabbedPane.getComponentAt(i).isShowing()) {
                    str = tabbedPane.getTitleAt(i);
                    if(str.equals((String)resource.getString("ROUNDS")))
                        text = "RUNDENZAHL";
                    else if(str.equals((String)resource.getString("MODE")))
                        text = "BETRIEBSARTEN";
                    else if(str.equals((String)resource.getString("KEY")))
                        text = "SCHLSSEL";
                }
            }
            System.out.println("HelpID:"+text);
            BaseConfiguration.viewHelp(text,"");
        } else dispose();
    }

    public JPanel roundsPanel(int actualRounds, String toolTip) {
	JPanel panel = new JPanel();
	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	panel.setLayout(gridBagLayout);

	JLabel label = new JLabel(resource.getString("NUMBER OF MINIMUM ROUNDS"));
	gridBagConstraints.insets.top = 20;
	gridBagConstraints.insets.left = 20;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.gridwidth = 1;
	gridBagConstraints.gridheight = 1;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.weighty = 1;
	gridBagLayout.setConstraints(label, gridBagConstraints);
	panel.add(label);

	roundsInput = new JTextField(Integer.toString(actualRounds));
	roundsInput.setColumns(4);
	roundsInput.setToolTipText(toolTip);
	gridBagConstraints.insets.left = 0;
	gridBagConstraints.insets.top = 22;
	gridBagConstraints.gridx = 1;
	gridBagConstraints.gridy = 0;
	gridBagConstraints.weightx = 1;
	gridBagLayout.setConstraints(roundsInput, gridBagConstraints);
	panel.add(roundsInput);

	return panel;
    }
    public int getRounds() {

	return Integer.parseInt(roundsInput.getText());
    }
}
