/*
 * smi/mechanisms/accountability/DSA.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.accountability;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.StringTokenizer;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.Signature;

import smi.io.IOStream;
import smi.mechanisms.AbstractAccountabilityMechanism;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.ParameterNotCompatibleException;

import ssonet.KeyExchange;

import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

public class DSA extends AbstractAccountabilityMechanism {
    public DSA() {
	mechanismName = "DSA";
	mechanismCertificateFileName = new String(System.getProperty("ssonet_cfg","")+"keys/accountability/DSACertificate.der");
	System.out.println("DSA Sign instance created");
    }

    protected X509Key readMechanismPublicKey() {
	X509Key publicKey = null;
	try {
	    System.out.println("reading public key");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPublicKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    publicKey = (X509Key)objectInputStream.readObject();
	    objectInputStream.close();
	    fileInputStream.close();
	} catch(Exception e) {
	    System.out.println(e);
	}
	return publicKey;
    }

    protected void writeMechanismPublicKey(PublicKey publicKey) {
	try {
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPublicKey.object");
	    System.out.println("writing public key to keys/accountability/DSAPublicKey.object");
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(fileOutputStream);
	    objectOutputStream.writeObject(publicKey);
	    fileOutputStream.close();
	    objectOutputStream.close();
	} catch(Exception e) {
	    System.out.println(e);
	}
    }

    protected void writeMechanismPrivateKey(PrivateKey privateKey) {
	try {
	    FileOutputStream fileOutputStream = new FileOutputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPrivateKey.object");
	    System.out.println("writing private key to keys/accountability/DSAPrivateKey.object");
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(fileOutputStream);
	    objectOutputStream.writeObject(privateKey);
	    fileOutputStream.close();
	    objectOutputStream.close();
	} catch(Exception e) {
	    System.out.println(e);
	}
    }

    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
	try {
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPublicKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PublicKey testKey = (PublicKey)objectInputStream.readObject();

	    initCipher(testKey, ioStream);
	}
	catch (Exception e) {
	    System.out.println(e);
	}
    }

    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration,
	MechanismConfiguration partnerMechanismConfiguration,
	KeyExchange keyExchange) throws Exception {
	try {
	    System.out.println("DSA init");
	    X509Cert mechanismCertificate = keyExchange.readCertificate(System.getProperty("ssonet_cfg","")+"keys/accountability/DSACertificate.der");
	    PublicKey testKey = keyExchange.getPartnerMechanismKey(mechanismCertificate);

	    initCipher(testKey, ioStream);
	}
	catch (Exception e) {
	    System.out.println(e);
	    throw new Exception();
	}
    }

    /** Initialize crypto mechanisms
    * @param testKey test key
    * @param signKey sign key
    */
    protected void initCipher(PublicKey testKey, IOStream ioStream) {
	try {
	    this.ioStream = ioStream;

	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_cfg","")+"keys/accountability/DSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey signKey = (PrivateKey)objectInputStream.readObject();
	    fileInputStream.close();
	    objectInputStream.close();

	    signatureIn = Signature.getInstance(mechanismName);
	    signatureIn.initVerify(testKey);
	    signatureOut = Signature.getInstance(mechanismName);
	    signatureOut.initSign(signKey);
	    System.out.println(mechanismName + " instance initialized");
	}
	catch(Exception e) {
	    System.out.println(e);
	}
    }
}
