/*
 * smi/mechanisms/confidentiality/SAFER.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.confidentiality;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import smi.awt.JPreferenceList;
import smi.mechanisms.AbstractConfidentialityMechanism;
import smi.mechanisms.DetailConfigurationDialog;
import smi.mechanisms.MechanismConfiguration;
import smi.baseConf.BaseConfiguration;

import ssonet.SSONETConstants;

public class SAFER extends AbstractConfidentialityMechanism {
    private JTextField inputTextField;
    private JPreferenceList variantPreferenceList;
    private static String variants[] = {"SK-128","SK-64","K-128","K-64"};

    public SAFER() {
    }

    public MechanismConfiguration getDefaultConfiguration() {
	MechanismConfiguration  defaultConfiguration = new MechanismConfiguration();
	String[] defaultMode = {"OFB"};
	String defaultProvider[] = {"Cryptix"};
	defaultConfiguration.put(PROVIDERLIST, defaultProvider); defaultConfiguration.put(KEY_MODE_PREFERENCELIST, defaultMode);
	defaultConfiguration.put(KEY_MODE_PREFERENCELIST, defaultMode);
	defaultConfiguration.put(KEY_ROUNDS, new Integer(10));
	String[] defaultVariant = {"SK-128"};
	defaultConfiguration.put(KEY_VARIANT, defaultVariant);
	return defaultConfiguration;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualConfiguration) {
	DetailConfigurationDialog detailConfigurationDialog = new DetailConfigurationDialog(parentFrame, getClass().getName());

	if(actualConfiguration == null) actualConfiguration = getDefaultConfiguration();

	// Provider
	detailConfigurationDialog.addTabProvider((String [])actualConfiguration.get(PROVIDERLIST));

	// Mode
	detailConfigurationDialog.addTabMode((String [])actualConfiguration.get(KEY_MODE_PREFERENCELIST),
	    SSONETConstants.CONFIDENTIALITY);

	// rounds
	Integer actualRounds = (Integer)actualConfiguration.get(KEY_ROUNDS);
	if (actualRounds == null)
	    actualRounds = new Integer(10);
	detailConfigurationDialog.addTab(BaseConfiguration.getLangResources().getString("VARIANTS"), createExtendedTab(actualRounds.intValue(),
	    (String[])actualConfiguration.get(KEY_VARIANT)));
	detailConfigurationDialog.pack();
	detailConfigurationDialog.setResizable(false);
	detailConfigurationDialog.setLocationRelativeTo(parentFrame);
	detailConfigurationDialog.setVisible(true);
	if(detailConfigurationDialog.getReturnCode()) {
	    actualConfiguration.put(PROVIDERLIST, detailConfigurationDialog.getProviderPreferenceList());
	    actualConfiguration.put(KEY_MODE_PREFERENCELIST, detailConfigurationDialog.getModePreferenceList());
	    actualConfiguration.put(KEY_ROUNDS, new Integer(Integer.parseInt(inputTextField.getText())));
	    actualConfiguration.put(KEY_VARIANT, variantPreferenceList.getPreferenceListItems());
	}
	return actualConfiguration;
    }

    private JPanel createExtendedTab(int actualRounds, String[] actualVariantPreferenceList) {
	JPanel panel = new JPanel();
	GridBagLayout gridBagLayout = new GridBagLayout();
	GridBagConstraints gridBagConstraints = new GridBagConstraints();
	panel.setLayout(gridBagLayout);

	variantPreferenceList = new JPreferenceList(5,
	    BaseConfiguration.getLangResources().getString("PREFERENCE LIST"),
	    BaseConfiguration.getLangResources().getString("AVAILABLE VARIANTS"),
	    actualVariantPreferenceList,
	    variants);
	panel.add(variantPreferenceList);

	JLabel label = new JLabel(BaseConfiguration.getLangResources().getString("NUMBER OF MINIMUM ROUNDS"));
	gridBagConstraints.insets.top = 20;
	gridBagConstraints.insets.left = 20;
	gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
	gridBagConstraints.gridx = 0;
	gridBagConstraints.gridy = 3;
	gridBagConstraints.gridwidth = 3;
	gridBagConstraints.gridheight = 1;
	gridBagConstraints.weightx = 0;
	gridBagConstraints.weighty = 1;
	gridBagLayout.setConstraints(label, gridBagConstraints);
	panel.add(label);

	inputTextField = new JTextField(Integer.toString(actualRounds));
	inputTextField.setColumns(4);
	inputTextField.setToolTipText("minimal: 1 - save: 10");
	gridBagConstraints.insets.left = 0;
	gridBagConstraints.insets.top = 22;
	gridBagConstraints.gridx = 3;
	gridBagConstraints.gridy = 3;
	gridBagConstraints.gridwidth = 2;
	gridBagConstraints.weightx = 1;
	gridBagLayout.setConstraints(inputTextField, gridBagConstraints);
	panel.add(inputTextField);

	return panel;
    }
}
