/*
 * catalog/awt/AnimatedOrderDialog.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.awt;

import catalog.customer.Customer;
import catalog.net.Order;

import java.io.IOException;
import java.net.ConnectException;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import smi.io.IOStream;
import smi.mechanisms.ConflictException;
import smi.mechanisms.NoCommonMechanismException;
import smi.mechanisms.ParameterNotCompatibleException;

import ssonet.SSONETClientConnection;
import ssonet.SSONETCloseConnectionException;
import ssonet.SSONETCertificateNotValidException;

/**
* A window with an animated icon during the order
*/
public class AnimatedOrderDialog extends Thread implements ActionListener {
    private JDialog dialog;
    private Customer customer;
    String merchantHost;
    int merchantHostPort;
    JFrame parentFrame;
    Order order;
    SSONETClientConnection ssonetClientConnection;

    /**
    * The class for the animated dialog
    */
    class AnimatedDialog
        extends Thread
    {
        JDialog dialog;
        public AnimatedDialog(JDialog dialog) {
            this.dialog = dialog;
        }

        public void run() {
            dialog.setVisible(true);
        }

        public void halt() {
            dialog.dispose();
        }
    }


    /**
    * An implementation for the cancel button of the animated dialog
    */
    public void actionPerformed (ActionEvent e)
    {
        ssonetClientConnection.interrupt();  // stops the Connection...
    }

    /**
    * Creates a window with an animated icon and assigns values to variables.
    */

    public AnimatedOrderDialog(JFrame parentFrame,
	Customer customer,
	String merchantHost,
	int merchantHostPort,
	Order order)
    {
        this.order = order;
        this.customer = customer;
        this.merchantHost = merchantHost;
        this.merchantHostPort = merchantHostPort;
	this.parentFrame = parentFrame;

        dialog = new JDialog(parentFrame,"Bestellung abschicken", false);
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints gridBagConstraints = new GridBagConstraints();

        dialog.getContentPane().setLayout(gridBagLayout);
        gridBagConstraints.insets.top = 10;
        gridBagConstraints.insets.right = 10;
        gridBagConstraints.insets.left = 10;
        gridBagConstraints.insets.bottom = 10;
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipady = 0;
        gridBagConstraints.ipadx = 0;
        gridBagConstraints.weightx = 0;
        Icon icon = new ImageIcon(getClass().getResource("images/order.gif"));
        FixedSizeJLabel fixedSizeJLabel = new FixedSizeJLabel(110,86);
        fixedSizeJLabel.setIcon(icon);
        gridBagLayout.setConstraints(fixedSizeJLabel, gridBagConstraints);
        dialog.getContentPane().add(fixedSizeJLabel);

        JLabel label = new JLabel("Bestellung abschicken.");
        label.setFont(new Font("Dialog", Font.BOLD,12));
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagLayout.setConstraints(label, gridBagConstraints);
        dialog.getContentPane().add(label);

        JButton cancel = new JButton("Abbruch");
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.gridwidth = 2;
        gridBagLayout.setConstraints(cancel, gridBagConstraints);
        cancel.setAlignmentX(Component.BOTTOM_ALIGNMENT);
        dialog.getContentPane().add(cancel);
        cancel.addActionListener(this);

        dialog.pack();
        Point parentLocation = parentFrame.getLocationOnScreen();
        Dimension parentDimension = parentFrame.getSize();
        Dimension ownDimension = dialog.getSize();
        int x = parentLocation.x + (parentDimension.width - ownDimension.width) / 2;
        int y = parentLocation.y + (parentDimension.height - ownDimension.height) / 2;
        dialog.setLocation(x, y);
        dialog.setResizable(false);
        start();
    }

    /**
    * Creates a new SSONETClientConnection to the merchant, sends the order and
    * displays the merchants answer in a window
    */
    public void run() {
        AnimatedDialog animatedDialog = new AnimatedDialog(dialog);
        animatedDialog.setPriority(2);
        animatedDialog.start();
        String errorMessage = "";
        try {
            System.out.println("sendOrderHere:");
            System.out.println("Host:"+merchantHost);
            System.out.println("Port:"+merchantHostPort);
            System.out.println("MerchantHost:"+merchantHost);
            ssonetClientConnection = new SSONETClientConnection(customer.name,
		OrderPanel.ORDER,
		null,
		merchantHost,
		merchantHostPort);
            try {
                IOStream ioStream = ioStream = ssonetClientConnection.connect();
                if(ioStream == null) {
                    errorMessage = "\nKein Verbindungsaufbau mglich.";
                    throw new Exception();
                }

                ioStream.writeObject(order);
                Object object = ioStream.readObject(); // reading answer
                dialog.dispose();
                if (object instanceof  String)
                    JOptionPane.showMessageDialog(null, object,"Information", JOptionPane.INFORMATION_MESSAGE);
                else {
                    System.out.println (object);
                    JOptionPane.showMessageDialog(null,"Fehler beim Versenden\nFalsche oder keine Antwort vom Hndler.","Fehler", JOptionPane.ERROR_MESSAGE);
                }
                ioStream.close();
                animatedDialog.halt();
            } catch (NullPointerException ne) {
                errorMessage = "\nEigene Konfiguration konnte nicht gelesen werden.";
                throw ne;
            } catch (SSONETCertificateNotValidException cnve) {
                errorMessage = "\nZertifikat wurde abgelehnt.";
                throw cnve;
            } catch (IOException ie) {
                errorMessage = "\nKonnte Auftrag nicht bertragen.";
                throw ie;
            }

        } catch (ParameterNotCompatibleException pe) {
            animatedDialog.halt();
            JOptionPane.showMessageDialog(null,"Folgender Konflikt trat auf:\n"+pe.getMessage(),"Information", JOptionPane.ERROR_MESSAGE);
            return;
        } catch (NoCommonMechanismException me) {
            animatedDialog.halt();
            JOptionPane.showMessageDialog(null,"Folgender Konflikt trat auf:\n"+me.getMessage(),"Information", JOptionPane.ERROR_MESSAGE);
            return;
        } catch (ConflictException ce) {
            animatedDialog.halt();
            JOptionPane.showMessageDialog(null,"Folgender Konflikt trat auf:\n"+ce.getMessage(),"Information", JOptionPane.ERROR_MESSAGE);
            return;
        } catch(SSONETCloseConnectionException se) {
            animatedDialog.halt();
            JOptionPane.showMessageDialog(null,"Abbruch auf Benutzerwunsch","Information", JOptionPane.ERROR_MESSAGE);
            return;
        } catch(ConnectException ce) {
            animatedDialog.halt();
            JOptionPane.showMessageDialog(null,"Hndler nicht erreichbar","Fehler", JOptionPane.ERROR_MESSAGE);
            System.out.println("Hndler nicht erreichbar.");
            return;
        } catch(Exception e) {
            animatedDialog.halt();
            JOptionPane.showMessageDialog(null,"Fehler beim Versenden"+errorMessage,"Fehler", JOptionPane.ERROR_MESSAGE);
            if (errorMessage.equals ("")) e.printStackTrace();
            return;
        }
    }
}
