/*
 * catalog/awt/ProductTable.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package catalog.awt;

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import javax.swing.table.*;

public class ProductTable extends JTable {

    ProductPanel productPanel;
    TableModel tableModel;
    int preSelected = 0;

    public ProductTable() {super(null, null, null);}
    public ProductTable(TableModel dm, ProductPanel f) {
        super(dm, null, null);
        productPanel = f;
        tableModel = dm;
    }

    /**
    * Updates the list (reload productPanel)
    */
    public void update() {
        Rectangle topLeft = getCellRect(0,0, false);
        Rectangle bottomRight = getCellRect(getRowCount(), getColumnCount(), false);
        Rectangle dirty = topLeft.union(bottomRight);
        repaint(dirty);
        productPanel.set(preSelected,
	    (String)tableModel.getValueAt(preSelected,2),
	    (String)tableModel.getValueAt(preSelected,1));
    }

    public void valueChanged(ListSelectionEvent e) {
        if ((tableModel == null) || (productPanel == null)) return;  // not initialized (first call)
        if (e.getValueIsAdjusting()) return;
        int firstIndex = e.getFirstIndex();
        int lastIndex = e.getLastIndex();
        int index;
        if (firstIndex == preSelected)
            index = lastIndex;
        else
            index = firstIndex;
        preSelected = index;
        if (firstIndex == -1 && lastIndex == -1) repaint();
        Rectangle firstRowRect = getCellRect(firstIndex, 0, false);
        Rectangle lastRowRect = getCellRect(lastIndex, getColumnCount(), false);
        Rectangle dirtyRegion = firstRowRect.union(lastRowRect);
        // This marks this entire row as dirty but the painting system will
        // intersect this with the clip rect of the viewport and redraw only
        // the visible cells.
        repaint(dirtyRegion.x, dirtyRegion.y, dirtyRegion.width, dirtyRegion.height);
        try {
            productPanel.set(index,
		(String)tableModel.getValueAt(index,2),
		(String)tableModel.getValueAt(index,1));
        } catch (Exception exc) {
            exc.printStackTrace();
        }
    }
}
