/*
 * client_server/Client.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package client_server;

import javax.swing.JFrame;

import smi.appConf.ApplicationConfiguration;
import smi.appConf.ActionConfiguration;
import smi.io.IOStream;

import ssonet.SSONETClientConnection;
import ssonet.Preload;
import ssonet.IdentityCheck;
import ssonet.SSONETConstraints;

public class Client {
    Preload p = new Preload();
    // Action-ID
    private static final short HELLO_WORLD = 1;
    private static int rounds = 0;
    private static boolean zero = true;
    private static String remoteHost;
    private long interactiveTime=0;

    public static void main(String[] argv) {
	remoteHost = argv[0];
	rounds = (new Integer(argv[1])).intValue();
	if ((new Integer(argv[2])).intValue()==0) {
	    zero = true;
	}
	else {
	    zero = false;
	}
	Client hc = new Client();
	hc.doIt();
    }

    public Client() {
    }

    public void doIt() {
	long startTime = System.currentTimeMillis();
	// configure Action
	System.out.println("configure Action.");
        if (smi.appConf.ApplicationConfiguration.getActionConfiguration("Client", HELLO_WORLD)==null) {
	    System.out.println("No Actionconfiguration found.");
	    SSONETConstraints configurationConstraint = new SSONETConstraints();
	    ApplicationConfiguration.configureAction(
		null,"Client","Hello World", HELLO_WORLD, configurationConstraint);
	    interactiveTime=ApplicationConfiguration.getInteractiveTime();
	    System.out.println(interactiveTime);
	}
	System.out.println("create Connection.");
	try {
	    // create Connection...
	    SSONETClientConnection ssonetClientConnection;
	    ssonetClientConnection = new SSONETClientConnection(
		"Client", HELLO_WORLD, new JFrame(), remoteHost, 2000);
	    long[] timeBuffer = new long[rounds+2];
	    long[] timeBuffer2 = new long[rounds+2];

	    System.out.println("Negotiation.");
	    timeBuffer[0] = System.currentTimeMillis();
	    IOStream ioStream = ssonetClientConnection.connect();
	    System.out.println("measurement running.");

	    timeBuffer[1] = System.currentTimeMillis();

	    byte[] sendBuffer;

	    if (!zero) {		// 1k byte verschicken
		sendBuffer = new byte[1024];
		try {
		    (getClass().getResourceAsStream("data/test.dat")).read(sendBuffer);
		} catch(Exception e) {
		    System.out.println("error while file loading");
		    e.printStackTrace();
		}
	    } else {		// nur 1 Byte
		sendBuffer = new byte[1];
		sendBuffer[0] = 0x55;
	    }

	    int i;
	    for (i=0; i<rounds; i++) {
		// write Object
		ioStream.writeObject(sendBuffer);
		// read Object
		Object object = ioStream.readObject();
		timeBuffer[i+2] = System.currentTimeMillis();
	    }
	    ssonetClientConnection.close();
	    ioStream.close();
	    if (zero) System.out.println("Measurement with "+rounds+" messages (1 Byte) finished");
	    else System.out.println("Measurement with "+rounds+" messages (1 kByte) finished");

	    long timeAll = (timeBuffer[rounds + 1] - startTime);
	    long average = (timeBuffer[rounds + 1] - timeBuffer[2]) / (2 * (rounds - 1));
	    interactiveTime+=IdentityCheck.getInteractiveTime();
	    // Aushandlung! - Ausfhrungszeit wird voll angerechnet
	    timeBuffer2[0] = (timeBuffer[1] - timeBuffer[0]);
	    for (i=1; i<rounds+1; i++) {
		timeBuffer2[i] = (timeBuffer[i+1] - timeBuffer[i]) / 2;
	    }
	    long max = timeBuffer2[2];
	    long min = timeBuffer2[2];
	    int maxPos = 3;
	    int minPos = 3;
	    for (i=3; i<rounds; i++) {
		if (max < timeBuffer2[i]) {
		    max = timeBuffer2[i];
		    maxPos = i+1;
		}
		if (min > timeBuffer2[i]) {
		    min = timeBuffer2[i];
		    minPos = i+1;
		}
	    }

	    System.out.println("\nNumber of messages sent: "+rounds);
	    System.out.println("\nBytes per message: "+sendBuffer.length);
	    System.out.println("Duration: "+timeAll+" ms");
	    System.out.println("Interactive: "+interactiveTime+" ms");
	    System.out.println("Connection: "+(timeBuffer[0]-startTime)+" ms");
	    System.out.println("Negotiation: "+(timeBuffer2[0]-interactiveTime)+" ms");
	    System.out.println("First message: "+timeBuffer2[1]+" ms");
	    System.out.println("Average from second message on: "+average+" ms");
	    System.out.println("Maximum from second message on: "+max+" ms (Pos:"+maxPos+")");
	    System.out.println("Minimum from second message on: "+min+" ms (Pos:"+minPos+")");
	    System.exit(0);
	} catch(Exception e) {
	    e.printStackTrace();
	}
    }
}
