/*
 * experimental source code
 * smi/mechanisms/accountability/Gateway.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package smi.mechanisms.accountability;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.security.Key;
import java.security.KeyGenerator;
import java.security.SecureRandom;

import java.security.Mode;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.SecureRandom;
import java.security.Signature;
import java.security.SignatureException;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.StringTokenizer;
import java.util.ResourceBundle;

import cryptix.provider.rsa.RawRSAPublicKey;
import cryptix.provider.rsa.RawRSAPrivateKey;

import java.util.StringTokenizer;
import java.util.Hashtable;
import java.net.Socket;
import javax.swing.JFrame;
import java.io.*;
import java.net.UnknownHostException;
import javax.swing.JFrame;

import smi.baseConf.RatedString;
import smi.io.*;
import smi.mechanisms.*;

import ssonet.KeyExchange;
import ssonet.SSONETConnection;
import ssonet.SSONETConstants;
import java.security.PublicKey;

import cryptix.provider.key.RawSecretKey;
import cryptix.provider.rsa.RawRSAPrivateKey;
import cryptix.provider.rsa.RawRSAPublicKey;

import gateway.GatewayWindow;
import sun.security.util.BigInt;
import sun.security.util.DerValue;
import sun.security.util.DerOutputStream;
import sun.security.x509.AlgorithmId;
import sun.security.x509.X500Name;
import sun.security.x509.X500Signer;
import sun.security.x509.X509Cert;
import sun.security.x509.X509Key;

public class Gateway implements IOStream, Mechanism {
    protected IOStream ioStream, GWioStream;
    protected X509Cert ownTestCertificate, ownCipherCertificate;
    protected PublicKey trustCenterTestKey;

    private X509Cert mechanismCertificate;
    private String packageName;
    private static String remoteHost;
    private static int remotePort;
    Socket socket;
    KeyExchange GWKeyExchange;
    boolean isServer = false;

    protected Signature signatureIn, signatureOut;
    protected static String KEY_KEYLENGTH = "KeyLength";
    protected static String PROVIDERLIST = "Providerlist";
    private static String KEY_VARIANT = "KeyVariante";
    protected String mechanismName;
    protected String mechanismCertificateFileName;
    String defaultProvider[] = {"Cryptix"};

    public String[] getModes() {
	return null;
    }

    public int available() {
	return GWioStream.available();
    }

    public void close() {
	ioStream.close();
	GWioStream.close();
    }


    public byte[] testEncrypt(byte[] buffer) {
	try {
	    SignatureObject signatureObject = new SignatureObject(buffer);
	    signatureOut.update(signatureObject.getObjectAsByteArray());
	    return signatureObject.getObjectAsByteArray();
	} catch (SignatureException se) {
	    System.out.println ("SignatureException in Accountability-encrypt");
	    se.printStackTrace();
	    return null;
	}
    }

    public byte[] testDecrypt(byte[] buffer) {
	try {
	    SignatureObject signatureObject = new SignatureObject(buffer);
	    signatureIn.update(signatureObject.getObjectAsByteArray());
	    return signatureObject.getObjectAsByteArray();
	} catch (SignatureException se) {
	    System.out.println ("SignatureException in Accountability-decrypt");
	    se.printStackTrace();
	    return null;
	}
    }


    // #########################################

    public Object readObject() throws Exception {
	try {
	    System.out.println("Gateway.readObject: Lese Object von Server");
	    Object object = ioStream.readObject(); //Object vom Server lesen

	    System.out.println("Serverobject angekommen");
	    //verschlsseltes Object zum Gateway senden und so
	    //modifiziert, da Gateway weis, da er umcodieren mu (testen, signieren)
	    Hashtable ht = new Hashtable();
	    ht.put("OBJECT", object);
	    ht.put("TODO", new String("VERIFY&SIGN"));

	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(ht);
	    objectOutputStream.flush();
	    byte[] buffer = byteArrayOutputStream.toByteArray();
	    GWioStream.writeObject(buffer);
	    objectOutputStream.close();
	    byteArrayOutputStream.close();
	    SignatureObject signatureObject = (SignatureObject)GWioStream.readObject();
	    signatureIn.update(signatureObject.getObjectAsByteArray());
	    if(signatureIn.verify(signatureObject.getSignature())) {
		System.out.println("Gatewaysignatur korrekt!");
		return signatureObject.getObject();
	    }
	    throw new Exception();
	} catch (Exception e) {
	    throw e;
	}
    }

    public void writeObject(Object object) throws Exception {
	try {
	    System.out.println("Gateway: Schreibe Object vom Client zum Gateway");
	    Hashtable ht = new Hashtable();
	    ht.put("OBJECT", object);
	    ht.put("TODO", new String("SIGN"));
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(ht);
	    objectOutputStream.flush();
	    byte[] buffer = byteArrayOutputStream.toByteArray();
	    GWioStream.writeObject(buffer);
	    objectOutputStream.close();
	    byteArrayOutputStream.close();

	    //signiertes Object vom Gateway empfangen (Hinmech.)
	    Object o1 = new Object();
	    o1 = object;
	    object = GWioStream.readObject(); //verschlssseltes Objekt empfangen

	    //Object zum Server schicken
	    System.out.println("Object zum Server mit HM signiert.");
	    System.out.println("ObjectName: "+object.getClass().getName());
	    ioStream.writeObject(object);

	    //System.out.println("Entschlsseltes Object zum Clienten zurueck gesendet.");

	} catch (Exception e) {
	    throw e;
	}
    }


    /** Initialize without key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    */
    public void init(IOStream ioStream,
	MechanismConfiguration ownMechanismConfiguration) {
    }


    /** Initialize with key exchange
    * @param ioStream mechanism IOStream
    * @param ownMechanismConfiguration own mechanism configuration
    * @param partnerMechanismConfiguration partner mechanism configuration
    * @param keyExchange prepared key exchange protocol
    * @exception Exception if initialization fails
    */
    public void init(IOStream ioStream,
	MechanismConfiguration mechs,
	MechanismConfiguration gatewayConfiguration,
	KeyExchange keyExchange) throws Exception {
	try {
	    this.ioStream = ioStream;
	    remoteHost = ((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_IP];
	    remotePort = Integer.parseInt(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_PORT]);
	    packageName = (String)gatewayConfiguration.get("PACKAGENAME");

	    MechanismConfiguration partnerMechanismConfiguration = (MechanismConfiguration)mechs.get("HIN");
	    System.out.println("AccGW: partnerMechanismConfiguration: " + partnerMechanismConfiguration);
	    MechanismConfiguration ownMechanismConfiguration = (MechanismConfiguration)mechs.get("BACK");
	    System.out.println("AccGW: ownMechanismConfiguration: " + ownMechanismConfiguration);

	    System.out.println("AccGW: RSA init");
	    String[] ownVariants = {"SHA-1"};
	    System.out.println("AccGW: ownVariants: " + ownVariants);

	    String[] partnerVariants = (String[])partnerMechanismConfiguration.get(KEY_VARIANT);
	    System.out.println("AccGW: partnerVariants: " + partnerVariants);
	    System.out.println("AccGW: partnerVariants[0]: " + partnerVariants[0]);

	    String variant = (SSONETConnection.computePreferred(RatedString.toRatedString(ownVariants),
		RatedString.toRatedString(partnerVariants)))[0];
	    System.out.println("Using variant: " + variant);

	    X509Cert mechanismCertificate = keyExchange.readCertificate(System.getProperty("ssonet_home","")+"keys/accountability/RSACertificate.der");
	    PublicKey testKeyHin = keyExchange.getPartnerMechanismKey(mechanismCertificate);
	    System.out.println("AccGW: KeyExchange Rckmechanismus beendet.");
	    try	{
		System.out.println("IP: "+remoteHost);
		System.out.println("Port: "+(remotePort));
		socket = new Socket(remoteHost, (remotePort));
	    } catch (UnknownHostException e) {
		System.err.println("Don't know about host: " + remoteHost);
	    } catch (IOException e) {
		System.err.println("Couldn't get I/O for the connection to: " + remoteHost);
	    }

	    InputStream in = socket.getInputStream();
	    OutputStream out = socket.getOutputStream();
	    System.out.println("Gateway erzeugt IOStream");
	    GWioStream = new ObjectIOStream(in, out);
	    System.out.println("AbstractAccGW: GWioStream steht.");

	    System.out.println("AbstrAccGate: send Server Public Key to Gateway");
	    GWioStream.writeObject(testKeyHin);
	    System.out.println("AbstrAccGate: send RSA Variant to Gateway");
	    GWioStream.writeObject(variant);

	    System.out.println("AbstrAccGate: write mechs");
	    GWioStream.writeObject(mechs);
	    System.out.println("AbstrAccGate: write Back_MechName");
	    GWioStream.writeObject(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_BACK_MECH]);
	    System.out.println("AbstrAccGate: write packageName");
	    GWioStream.writeObject(packageName);
	    System.out.println("AbstrAccGate: write Hin_MechName");
	    GWioStream.writeObject(((String[])gatewayConfiguration.get("GATEWAY"))[GatewayWindow.GW_HIN_MECH]);

	    System.out.println("AbstrAccGate: rufe initCipher");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_home","")+"keys/accountability/DSAPublicKey.object");
	    ObjectInputStream objectInputStream1 = new ObjectInputStream(fileInputStream);
	    PublicKey testKeyBack = (PublicKey)objectInputStream1.readObject();
	    fileInputStream.close();
	    objectInputStream1.close();

	    initCipher(testKeyBack, ioStream);
	}
	catch (Exception e) {
	    System.out.println(e);
	    throw new Exception();
	}
    }


    /** Initialize crypto mechanisms
    * @param sessionKey symmetric mechanism private key
    * @param mode mechanism mode
    */
    protected void initCipher(PublicKey testKey, IOStream ioStream) {
	mechanismName="DSA";
	try {
	    System.out.println("AbstrAccGate - InitCipher: start initcipher");
	    FileInputStream fileInputStream = new FileInputStream(System.getProperty("ssonet_home","")+"keys/accountability/DSAPrivateKey.object");
	    ObjectInputStream objectInputStream = new ObjectInputStream(fileInputStream);
	    PrivateKey signKey = (PrivateKey)objectInputStream.readObject();
	    fileInputStream.close();
	    objectInputStream.close();

	    System.out.println("AbstrAccGate - InitCipher: Erzeuge Signaturinstanz fr:" + mechanismName);
	    signatureIn = Signature.getInstance(mechanismName);
	    System.out.println("AbstrAccGate - InitCipher: initVerify");
	    signatureIn.initVerify(testKey);
	    System.out.println("AbstrAccGate - InitCipher: Erzeuge Testinstanz fr:" + mechanismName);
	    signatureOut = Signature.getInstance(mechanismName);
	    System.out.println("AbstrAccGate - InitCipher: initSign");
	    signatureOut.initSign(signKey);
	    System.out.println(mechanismName + " instance initialized");
	}
	catch(Exception e) {
	    System.out.println(e);
	}

    }

    public MechanismConfiguration getDefaultConfiguration() {
	return null;
    }

    public MechanismConfiguration configDetail(JFrame parentFrame, MechanismConfiguration actualMechanismConfiguration) {
	return null;
    }

    /** encrypt ByteArray with public key
    * @param input data to encrypt
    * @param publicKey public key
    * @return byte[] encrypted data
    */
    private byte[] encrypt(byte[] input, PublicKey publicKey) {
	return null;
    }

    public byte[] getObjectAsByteArray(Object object) {
	try {
	    ByteArrayOutputStream byteArrayOutputStream = new ByteArrayOutputStream();
	    ObjectOutputStream objectOutputStream = new ObjectOutputStream(byteArrayOutputStream);
	    objectOutputStream.writeObject(object);
	    objectOutputStream.flush();
	    byte[] b = byteArrayOutputStream.toByteArray();
	    System.out.println("getObjectAsByteArray lgt.: "+b.length);
	    return b;
	} catch (Exception e) {
	    System.out.println("Error while transforming into ByteArray.");
	    e.printStackTrace();
	    return null;
	}
    }
}
