/*
 * state/StateTreeCellRenderer.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package state;

import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JTree;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.DefaultMutableTreeNode;
import java.awt.Component;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
public class StateTreeCellRenderer extends JLabel implements TreeCellRenderer {
    static protected Font defaultFont;

    /** Color to use for the background when selected. */
    static protected final Color SelectedBackgroundColor = Color.lightGray;
    static {
        defaultFont = new Font("SansSerif", 0, 12);
    }

    /** Whether or not the item that was last configured is selected.
    * @serial selected boolean */
    protected boolean selected;
    /**
    * This is messaged from JTree whenever it needs to get the size
    * of the component or it wants to draw it.
    * This attempts to set the font based on value, which will be
    * a TreeNode.
    */
    public Component getTreeCellRendererComponent(JTree tree, Object value,
	boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {

	Font font;
	String stringValue = tree.convertValueToText(value, selected, expanded,
	    leaf, row, hasFocus);

	/* Set the text. */
	setText(stringValue);
	/* Tooltips used by the tree. */
	setToolTipText(stringValue);


	/* Set the color and the font based on the SampleData userObject. */
	TreeData userObject = (TreeData) ((DefaultMutableTreeNode) value).getUserObject();
	if (hasFocus)
	    setForeground(Color.black);
	else
	    setForeground(userObject.getColor());
	setIcon(userObject.getIcon());

	/* Update the selected flag for the next paint. */
	this.selected = selected;
	return this;
    }
    /**
    * paint is subclassed to draw the background correctly.  JLabel
    * currently does not allow backgrounds other than white, and it
    * will also fill behind the icon.  Something that isn't desirable.
    */
    public void paint(Graphics g) {
	Color bColor;
	Icon currentI = getIcon();
	if (selected)
	    bColor = SelectedBackgroundColor;
	else
	    if (getParent() != null)

	    /* Pick background color up from parent (which will come from
	    the JTree we're contained in). */
	    bColor = getParent().getBackground();
	else
	    bColor = getBackground();
	g.setColor(bColor);
	if (currentI != null && getText() != null) {
	    int offset = (currentI.getIconWidth() + getIconTextGap());
	    g.fillRect(offset, 0, getWidth() - 1 - offset, getHeight() - 1);
	} else
	    g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
	super.paint(g);
    }
}
