/*
 * ssonet/PhaseTwoConfiguration.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.io.Serializable;

import smi.baseConf.RatedString;
import smi.mechanisms.MechanismConfiguration;

import ssonet.SSONETConstants;

public class PhaseTwoConfiguration implements Serializable {
    private RatedString[] confidentialityPreferenceList, integrityPreferenceList, accountabilityPreferenceList;
    private MechanismConfiguration[] confidentialityConfiguration, integrityConfiguration, accountabilityConfiguration;
    private Settings settings;

    public PhaseTwoConfiguration(Settings settings) {
	this.settings = settings;
    }

    public void setConfidentiality(RatedString[] preferenceList,
	MechanismConfiguration[] configuration) {
	confidentialityPreferenceList = preferenceList;
	confidentialityConfiguration = configuration;
    }

    public void setAccountability(RatedString[] preferenceList,
	MechanismConfiguration[] configuration) {
	accountabilityPreferenceList = preferenceList;
	accountabilityConfiguration = configuration;
    }

    public void setIntegrity(RatedString[] preferenceList,
			     MechanismConfiguration[] configuration) {
	integrityPreferenceList = preferenceList;
	integrityConfiguration = configuration;
    }

    public MechanismConfiguration getConfiguration(int SSONETConstant, String mechanismName) {
	switch (SSONETConstant) {
	case SSONETConstants.INTEGRITY:
	    for(int i = 0; i<integrityPreferenceList.length; i++)
		if(integrityPreferenceList[i].string.equals(mechanismName))
		return integrityConfiguration[i];
	    return null;
	case  SSONETConstants.CONFIDENTIALITY:
	    for(int i = 0; i<confidentialityPreferenceList.length; i++)
		if(confidentialityPreferenceList[i].string.equals(mechanismName))
		return confidentialityConfiguration[i];
	    return null;
	case SSONETConstants.ACCOUNTABILITY_SIGN:
	    for(int i = 0; i<accountabilityPreferenceList.length; i++)
		if(accountabilityPreferenceList[i].string.equals(mechanismName))
		return accountabilityConfiguration[i];
	    return null;
	}
	return null;
    }

    public RatedString[] getPreferenceList(int SSONETConstant) {
	switch (SSONETConstant) {
	case SSONETConstants.INTEGRITY:
	    return integrityPreferenceList;
	case SSONETConstants.CONFIDENTIALITY:
	    return confidentialityPreferenceList;
	case SSONETConstants.ACCOUNTABILITY_SIGN:
	    return accountabilityPreferenceList;
	}
	return null;
    }

    public String toString() {
	String outputString = "";
	if(confidentialityPreferenceList != null)
	    outputString = "Confidentiality preference list: "+
	    confidentialityPreferenceList.toString();
	if(accountabilityPreferenceList != null)
	    outputString += "Accountability preference list: "+
	    accountabilityPreferenceList.toString();
	if(integrityPreferenceList != null)
	    outputString += "Integrity preference list: "+
	    integrityPreferenceList.toString();
	if(outputString.equals(""))
	    return "No preference lists present!";
	return outputString;
    }
}
