/*
 * ssonet/Preload.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.security.SecureRandom;
import java.security.Provider;
import java.security.Security;

/**
* Initialisiert securerandom als Thread und trgt Cryptix als Provider ein
*/
public class Preload extends Thread {
    private static boolean alreadyLoaded = false;
    private PreloadListener preloadListener;

    public Preload() {
	System.out.println("Call to Preload");
        if (!alreadyLoaded) {
	    this.preloadListener = null;
            this.setPriority(Thread.MIN_PRIORITY);
            this.start();
        }
    }

    /**
    * Initialisiert Preload und trgt einen Listener ein, der bei erfolgreichem Initialisieren aufgerufen wird
    */
    public Preload(PreloadListener pl) {
        if (!alreadyLoaded) {
	    this.preloadListener = pl;
            this.setPriority(Thread.MIN_PRIORITY);
            this.start();
        }
    }

    public void run() {
        try {
            alreadyLoaded = true;
            Security.addProvider(new cryptix.provider.Cryptix());
            Provider[] prov = Security.getProviders();  //alle registrieren Provider
            boolean installed=false;
            for(int i=0; i<prov.length; i++) {
                if(prov[i].getName().equals("Cryptix")) installed=true;
            }
            if(installed)
		System.out.println("Cryptixprovider installed");
            else {
                System.out.println("Error during installing Cryptixprovider - exit");
                System.exit(1);
            }
	    if (preloadListener != null)
		preloadListener.providerInstalled();
	    //init Random & SecureRandom ... needs a lot of time!
            if (System.getProperty ("Preload","").equals("none")) {
		System.out.println("Skip SecureRandom initialization.");
		if (preloadListener != null) preloadListener.preloadInstalled(PreloadListener.SKIPPED);
	    } else {
		int rnd = new SecureRandom().nextInt();
                System.out.println("Preload initialized.");
		if (preloadListener != null) preloadListener.preloadInstalled(PreloadListener.INSTALLED);
	    }
        }
        catch(Exception e) {
	    e.printStackTrace();
        }
    }
}
