/*
 * ssonet/SSONETNegotiate.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package ssonet;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;
import java.security.PublicKey;

import javax.swing.JFrame;

import smi.appConf.ActionConfiguration;
import smi.baseConf.BaseConfiguration;
import smi.baseConf.RatedString;
import smi.mechanisms.NoCommonMechanismException;
import smi.mechanisms.ParameterNotCompatibleException;
import smi.io.IOStream;

import ssonet.PhaseTwoConfiguration;
//import ssonet.PhaseTwoResult;
import ssonet.KeyExchange;
import ssonet.SSONETConnection;
import sun.security.x509.X509Cert;

import state.*;

final public class SSONETNegotiate {

    private SSONETNegotiate() {
    }

    private static void writeStatus(String s) {
	try {
	    String stringToDisplay = "Negotiation: " + s + "\n";
	    byte[] bytesToDisplay = new byte[stringToDisplay.length() + 1];
	    bytesToDisplay[0] = (byte)state.Pad.SSONET;
	    byte[] stringBytes = stringToDisplay.getBytes();
	    for (int i = 0; i < stringBytes.length; i++) bytesToDisplay[i + 1] = stringBytes[i];
	    DatagramPacket packet = new DatagramPacket(bytesToDisplay,
		bytesToDisplay.length,
		InetAddress.getLocalHost(),
		Pad.PORT);
	    DatagramSocket socket = new DatagramSocket();
	    socket.send(packet);
	    socket.close();
	} catch(Exception e) {
	    System.out.println("Error while sending status message!");
	}
    }
    private static void write2Tree(int ID, byte art, int wichtigkeit, String name, String string) {
	try {
	    if(name==null) name="";






	    if(string==null) string="";
	    String treeString =string;
	    String treeName   = name;
	    byte[] bytesToDisplay = new byte[treeString.length()+ name.length() + 4];
	    bytesToDisplay[0] = (byte)ID;
	    bytesToDisplay[1] = (byte)art;
	    bytesToDisplay[2] = (byte)wichtigkeit;
	    bytesToDisplay[3] = (byte)name.length();
	    byte[] stringBytesName = treeName.getBytes();
	    byte[] stringBytesString = treeString.getBytes();
	    for (int i = 0; i < treeName.length(); i++)
		bytesToDisplay[i + 4] = stringBytesName[i];
	    for (int i = treeName.length(); i < (treeName.length()+treeString.length()); i++)
		bytesToDisplay[i + 4] = stringBytesString[i-treeName.length()];

	    DatagramPacket packet = new DatagramPacket(bytesToDisplay,
		bytesToDisplay.length,
		InetAddress.getLocalHost(),
		StateTree.PORT);
	    DatagramSocket socket = new DatagramSocket();
	    socket.send(packet);
	    socket.close();
	} catch(Exception e) {
	    System.out.println("Error while sending message to StateTree!");
	    e.printStackTrace();
	}
    }
    public static void negotiatePhaseOne(ActionConfiguration ownConfig,
	ActionConfiguration partnerConfig) {
	try {
	    writeStatus("Negotiate security goal: Confidentiality");
	    negotiateSecurityGoalPhase1(SSONETConstants.CONFIDENTIALITY, ownConfig, partnerConfig);

	    writeStatus("Negotiate security goal: Anonymity");
	    negotiateSecurityGoalPhase1(SSONETConstants.ANONYMITY, ownConfig, partnerConfig);

	    writeStatus("Negotiate security goal: Accountability");
	    negotiateSecurityGoalPhase1(SSONETConstants.ACCOUNTABILITY_SIGN, ownConfig, partnerConfig);

	    writeStatus("Negotiate security goal: Integrity");
	    negotiateSecurityGoalPhase1(SSONETConstants.INTEGRITY, ownConfig, partnerConfig);
	}
	catch(Exception e) {
	    writeStatus("SSONETConnection negotiation error: " + e.getMessage());
	}
    }

    private static void negotiateSecurityGoalPhase1(int securityGoal,
	ActionConfiguration ownConfig,
	ActionConfiguration partnerConfig) {
	final boolean[][] table = {
	    {false, false, false, false, false, false, false},
	    {false, false, false, false, false, true, true},
	    {false, false, false, false, false, false, false},
	    {false, false, false, false, false, false, false},
	    {false, false, false, false, false, false, false},
	    {true, true, false, false, false, false, false},
	    {false, false, false, false, false, false, false}
	};

	if(table [ownConfig.getSecurityGoalLevel(securityGoal)] [partnerConfig.getSecurityGoalLevel(securityGoal)])
	    if (ConflictDialog.getAnswer(securityGoal, ownConfig))
	    if ((ownConfig.getSecurityGoalLevel(securityGoal)) == SSONETConstants.ON_NO_CONDITION)
	    ownConfig.raise(securityGoal);
	else
	    ownConfig.lower(securityGoal);
    }

    public static PhaseTwoResult negotiatePhaseTwo(ActionConfiguration ownConfig,
	ActionConfiguration partnerConfig,
	Settings settings) {

	PhaseTwoResult phaseTwoResult = new PhaseTwoResult();

	writeStatus("Negotiate security goal: Confidentiality");
	phaseTwoResult.set(SSONETConstants.CONFIDENTIALITY,
	    negotiateSecurityGoalPhase2(SSONETConstants.CONFIDENTIALITY, settings, ownConfig, partnerConfig));
	writeStatus("Negotiate security goal: Anonymity");
	phaseTwoResult.set(SSONETConstants.ANONYMITY,
	    negotiateSecurityGoalPhase2(SSONETConstants.ANONYMITY, settings, ownConfig, partnerConfig));
	writeStatus("Negotiate security goal: Accountability");
	phaseTwoResult.set(SSONETConstants.ACCOUNTABILITY_SIGN,
	    negotiateSecurityGoalPhase2(SSONETConstants.ACCOUNTABILITY_SIGN, settings, ownConfig, partnerConfig));
	writeStatus("Negotiate security goal: Integrity");
	phaseTwoResult.set(SSONETConstants.INTEGRITY,
	    negotiateSecurityGoalPhase2(SSONETConstants.INTEGRITY, settings, ownConfig, partnerConfig));

	return phaseTwoResult;
    }

    private static int negotiateSecurityGoalPhase2(int securityGoal,
	Settings settings,
	ActionConfiguration ownConfig,
	ActionConfiguration partnerConfig) {
	final short NO = 0;
	final short YES = 1;
	final short CONFLICT = 2;

	int[][] table = {
	    {NO, NO, NO, NO, NO, CONFLICT, CONFLICT},
	    {NO, NO, NO, NO, NO, CONFLICT, CONFLICT},
	    {NO, NO, NO, NO, YES, YES, YES},
	    {NO, NO, NO, YES, YES, YES, YES},
	    {NO, NO, YES, YES, YES, YES, YES},
	    {CONFLICT, CONFLICT, YES, YES, YES, YES, YES},
	    {CONFLICT, CONFLICT, YES, YES, YES, YES, YES}
	};

	switch (table[ownConfig.getSecurityGoalLevel(securityGoal)]
	    [partnerConfig.getSecurityGoalLevel(securityGoal)]) {
	case YES:
	    settings.securityGoal[securityGoal] = true;
	    return SSONETConstants.OK;
	case NO:
	    settings.securityGoal[securityGoal] = false;
	    return SSONETConstants.OK;
	case CONFLICT:
	    return SSONETConstants.CONFLICT;
	}
	return SSONETConstants.CONFLICT;
    }

    public static IOStream negotiatePhaseThree(PhaseTwoConfiguration ownConfiguration,
	PhaseTwoConfiguration partnerConfiguration,
	IOStream ioStream,
	JFrame parentframe,
	PublicKey trustCenterTestKey,
	X509Cert ownTestCertificate,
	X509Cert ownCipherCertificate,
	boolean isServer) throws Exception {
	writeStatus("Start negotiation phase three.");
	System.out.println("Start negotiation phase three.");

	if(ownConfiguration == null || partnerConfiguration == null) {
	    System.out.println("Configuration missing.");
	    return ioStream;
	}

	try {
	    if (ioStream == null) System.out.println("ioStream ist null");
	    if (parentframe == null) System.out.println("parentframe ist null");
	    KeyExchange keyExchange = new KeyExchange(ioStream,
		parentframe,
		ownTestCertificate,
		ownCipherCertificate,
		trustCenterTestKey,
		isServer);
	    if (ownConfiguration == null) System.out.println("ownConfiguration ist null");
	    if (partnerConfiguration == null) System.out.println("partnerConfiguration ist null");
	    if (ioStream == null) System.out.println("0. ioStream ist null");
	    if (keyExchange == null) System.out.println("keyExchange ist null");
	    ioStream = negotiateMechanism(ownConfiguration,
		partnerConfiguration,
		ioStream,
		SSONETConstants.CONFIDENTIALITY,
		keyExchange);
	    if (ioStream == null) System.out.println("1. ioStream ist null");
	    ioStream = negotiateMechanism(ownConfiguration,
		partnerConfiguration,
		ioStream,
		SSONETConstants.ACCOUNTABILITY_SIGN,
		keyExchange);
	    if (ioStream == null) System.out.println("2. ioStream ist null");
	    ioStream = negotiateMechanism(ownConfiguration,
		partnerConfiguration,
		ioStream,
		SSONETConstants.INTEGRITY,
		keyExchange);
	    if (ioStream == null) System.out.println("3. ioStream ist null");
	} catch (SSONETCertificateNotValidException cnve) {
	    throw cnve;
	} catch (NoCommonMechanismException e) {
	    throw e;
	} catch (Exception e) {
	    if (ioStream == null) System.out.println("ioStream ist null");
	    if (ownTestCertificate == null) System.out.println("ownTestCertificate ist null");
	    if (ownCipherCertificate == null) System.out.println("ownCipherCertificate ist null");
	    if (trustCenterTestKey == null) System.out.println("trustCenterTestKey ist null");
	    System.out.println("SSONETNegotiate.java:");
	    e.printStackTrace();
	    throw e;
	}
	return ioStream;
    }

    private static IOStream negotiateMechanism(PhaseTwoConfiguration ownConfiguration,
	PhaseTwoConfiguration partnerConfiguration,
	IOStream ioStream,
	int ssonetConstant,
	KeyExchange keyExchange) throws NoCommonMechanismException
    {
	RatedString[] ownPreferenceList = ownConfiguration.getPreferenceList(ssonetConstant);
	RatedString[] partnerPreferenceList = partnerConfiguration.getPreferenceList(ssonetConstant);
	if (ownPreferenceList!=null) {
	    writeStatus("Negotiate mechanism.");
	    write2Tree(1, StateTreeConstants.INFORMATION, StateTreeConstants.LOWIMPORTANT, null,"Negotiate mechanism.");
	    System.out.println("Negotiate mechanism.");
	    String[] rankedMechanisms = SSONETConnection.computePreferred(ownPreferenceList,
		partnerPreferenceList);
	    if (rankedMechanisms == null) {
		String[] s = {"Confidentiality","Accountability","Accountability Accept","Integrity","Anonymity"};
		System.out.println("keine gemeinsamen Mechanismen fr Schutzziel "+s[ssonetConstant]);
		write2Tree(1, StateTreeConstants.CONFLICT, StateTreeConstants.VERYIMPORTANT, null,"No common mechanisms: "+s[ssonetConstant]);
		throw new NoCommonMechanismException();
	    }
	    boolean found = false;
	    for(int k = 0; k < rankedMechanisms.length; k++) {
		try {
		    System.out.println("Try mechanism: " + rankedMechanisms[k]);
		    ioStream = BaseConfiguration.getMechanismInstance (ssonetConstant,
			rankedMechanisms[k],
			ioStream,
			partnerConfiguration.getConfiguration(ssonetConstant, rankedMechanisms[k]),
			keyExchange);
		    writeStatus("Have chosen: " + rankedMechanisms[k]);
		    write2Tree(ssonetConstant, StateTreeConstants.USER, StateTreeConstants.IMPORTANT, rankedMechanisms[k], rankedMechanisms[k]);
		    System.out.println("Have chosen: " + rankedMechanisms[k]);
		    found = true;
		    break;
		} catch (ParameterNotCompatibleException pe) {
		    pe.printStackTrace();
		    writeStatus("failed");
		    write2Tree(ssonetConstant, StateTreeConstants.CONFLICT, StateTreeConstants.LOWIMPORTANT, null,"Parameter not compatible: "+rankedMechanisms[k]+", skip mechanism");
		    System.out.println("failed");
		} catch (Exception e) {
		    e.printStackTrace();
		    writeStatus("failed");
		    write2Tree(ssonetConstant, StateTreeConstants.CONFLICT, StateTreeConstants.IMPORTANT,"Unknown error using: "+rankedMechanisms[k], rankedMechanisms[k]);
		    System.out.println("failed");
		}
	    }
	    if(!found) throw new NoCommonMechanismException();
	}
	return ioStream;
    }
}
