/*
 * state/StateTree.java
 *
 * SSONET Version 1.1
 * 24. April 2000
 * <mailto:ssonet@inf.tu-dresden.de>
 *
 */
package state;

import javax.swing.JTree;
import javax.swing.UIManager;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.util.Calendar;
import java.util.GregorianCalendar;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import javax.swing.JRadioButton;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.DatagramPacket;
import java.net.DatagramSocket;
import smi.appConf.ActionConfiguration;
import ssonet.SSONETConstants;
import smi.baseConf.RatedString;
import smi.baseConf.BaseConfiguration;
import ssonet.PhaseTwoConfiguration;
import smi.mechanisms.MechanismConfiguration;
import smi.mechanisms.Mechanism;
public class StateTree extends JFrame implements Serializable {
    final public static int PORT = 2010;
    /**@serial treeModel DefaultTreeModel*/
    protected DefaultTreeModel treeModel;
    /**@serial frame JFrame*/
    protected JFrame frame;
    /**@serial tree JTree*/
    protected JTree tree;
    /**@serial importance int*/
    private int importance = StateTreeConstants.LOWIMPORTANT;
    /**@serial CBOXTEXT1 String */
    private final String CBOXTEXT1 = "Alle Informationen anzeigen";
    /**@serial CBOXTEXT2 String */
    private final String CBOXTEXT2 = "Keine Debuginformationen";
    /**@serial CBOXTEXT3 String */
    private final String CBOXTEXT3 = "Keine Debug- und Protokollinformationen";
    /**@serial OK String */
    private final String OK = "OK";
    /**@serial CANCEL String */
    private final String CANCEL = "CANCEL";
    /**@serial FRAMENAME String */
    private final String FRAMENAME = "Eigenschaften Anzeigebaum";
    /**@serial TEXTLABEL String */
    private final String TEXTLABEL = "Welche Informationen sollen nicht angezeigt werden?";
    /**@serial info TreeData */
    TreeData info = new TreeData(StateTreeConstants.GRAYICON,
	StateTreeConstants.IMPORTANT, Color.black, "Protokollnachrichten");
    /**@serial debug TreeData */
    TreeData debug = new TreeData(StateTreeConstants.GRAYICON,
	StateTreeConstants.LOWIMPORTANT, Color.black, "Debuginformationen");
    /**@serial konflict TreeData */
    TreeData konflict = new TreeData(StateTreeConstants.GRAYICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Konflikte");
    /**@serial user TreeData */
    TreeData user = new TreeData(StateTreeConstants.GRAYICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Eigene Einstellungen");
    /**@serial mechanism TreeData */
    TreeData mechanism = new TreeData(StateTreeConstants.GRAYICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Mechanismen");
    /**@serial schutzziele TreeData */
    TreeData schutzziele = new TreeData(StateTreeConstants.GRAYICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Schutzziele");
    /**@serial confidentaly TreeData */
    TreeData confidentaly = new TreeData(StateTreeConstants.INFOICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Confidentaly");
    /**@serial integrity TreeData */
    TreeData integrity = new TreeData(StateTreeConstants.INFOICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Integrity");
    /**@serial anonymity TreeData */
    TreeData anonymity = new TreeData(StateTreeConstants.INFOICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Anonymity");
    /**@serial accountability TreeData */
    TreeData accountability = new TreeData(StateTreeConstants.INFOICON,
	StateTreeConstants.VERYIMPORTANT, Color.black, "Accountability");
    /**@serial mechanismNode DefaultMutableTreeNode */
    DefaultMutableTreeNode mechanismNode = new DefaultMutableTreeNode(mechanism);
    /**@serial schutzzielNode DefaultMutableTreeNode */
    DefaultMutableTreeNode schutzzielNode = new DefaultMutableTreeNode(schutzziele);
    /**@serial confidentalyNode DefaultMutableTreeNode */
    DefaultMutableTreeNode confidentalyNode = new DefaultMutableTreeNode(confidentaly);
    /**@serial integrityNode DefaultMutableTreeNode */
    DefaultMutableTreeNode integrityNode = new DefaultMutableTreeNode(integrity);
    /**@serial anonymityNode DefaultMutableTreeNode */
    DefaultMutableTreeNode anonymityNode = new DefaultMutableTreeNode(anonymity);
    /**@serial accountabilityNode DefaultMutableTreeNode */
    DefaultMutableTreeNode accountabilityNode =
	new DefaultMutableTreeNode(accountability);
    /**@serial infoNode DefaultMutableTreeNode */
    DefaultMutableTreeNode infoNode = new DefaultMutableTreeNode(info);
    /**@serial debugNode DefaultMutableTreeNode */
    DefaultMutableTreeNode debugNode = new DefaultMutableTreeNode(debug);
    /**@serial konflictNode DefaultMutableTreeNode */
    DefaultMutableTreeNode konflictNode = new DefaultMutableTreeNode(konflict);
    /**@serial userNode DefaultMutableTreeNode */
    DefaultMutableTreeNode userNode = new DefaultMutableTreeNode(user);
    /**@serial root DefaultMutableTreeNode */
    DefaultMutableTreeNode root = new DefaultMutableTreeNode(new TreeData(0,
	StateTreeConstants.VERYIMPORTANT, Color.black, "SSONET-Ereignismonitor")); //root
    /**@serial dayOfBegin int */
    private int dayOfBegin;
    private class Display extends JFrame implements ActionListener {
	private int imp;
	private CheckboxGroup cbg = null;
	private Checkbox cb1 = null, cb2 = null, cb3 = null;
	private JFrame frame=null;
	private Panel buttonPanel=null, checkPanel=null, panel=null;
	private Button bOK = null, bCANCEL = null;
	public Display() {
	    imp = importance;
	    cbg = new CheckboxGroup();
	    cb1 = new Checkbox(CBOXTEXT1, cbg, true);
	    cb2 = new Checkbox(CBOXTEXT2, cbg, false);
	    cb3 = new Checkbox(CBOXTEXT3, cbg, false);
	    bOK = new Button(OK);
	    bCANCEL = new Button(CANCEL);
	    frame = new JFrame(FRAMENAME);
	    frame.setSize(310, 200);
	    frame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    frame.dispose();
		}
	    });
	    frame.getContentPane().setLayout(null);
	    JLabel textLabel = new JLabel();
	    textLabel.setBounds(5, 10, 300, 20);
	    textLabel.setForeground(Color.blue);
	    textLabel.setText(TEXTLABEL);
	    checkPanel = new Panel();
	    checkPanel.setBounds(40, 50, 260, 80);
	    checkPanel.setLayout(new BoxLayout(checkPanel, BoxLayout.Y_AXIS));
	    checkPanel.add(cb1);
	    checkPanel.add(cb2);
	    checkPanel.add(cb3);
	    buttonPanel = new Panel();
	    buttonPanel.setLayout(null);
	    buttonPanel.setBounds(60, 140, 190, 40);
	    bOK.setBounds(10, 7, 75, 25);
	    bOK.addActionListener(this);
	    bCANCEL.setBounds(95, 7, 75, 25);
	    bCANCEL.addActionListener(this);
	    buttonPanel.add(bOK);
	    buttonPanel.add(bCANCEL);
	    frame.getContentPane().add(textLabel);
	    frame.getContentPane().add(checkPanel);
	    frame.getContentPane().add(buttonPanel);
	}
	public void actionPerformed(ActionEvent e) {
	    frame.show();
	    if (e.getSource() == bOK) {
		Checkbox cb = cbg.getSelectedCheckbox();
		if (cb.equals(cb1))
		    imp = StateTreeConstants.LOWIMPORTANT;
		if (cb.equals(cb2))
		    imp = StateTreeConstants.IMPORTANT;
		if (cb.equals(cb3))
		    imp = StateTreeConstants.VERYIMPORTANT;
		RevalidateTree(imp);
		frame.dispose();
	    }
	    if (e.getSource() == bCANCEL) {
		frame.dispose();
	    }
	}
    }
    public StateTree() {
	try {
	    UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
	} catch (Exception e) {
	    System.err.println("Error loading L&F: " + e);
	}
	Calendar date = new GregorianCalendar();
	dayOfBegin = date.get(Calendar.DAY_OF_MONTH);
	frame = new JFrame("SSONET-Ereignismonitor");
	JMenuBar menuBar = constructMenuBar();
	JPanel panel = new JPanel(true);
	frame.setJMenuBar(menuBar);
	frame.getContentPane().add("Center", panel);
	frame.setBackground((Color) SystemColor.desktop);
	treeModel = new DefaultTreeModel(root);
	tree = new JTree(treeModel);
	tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
	InsertNode(root, infoNode);
	InsertNode(root, debugNode);
	InsertNode(root, konflictNode);
	InsertNode(root, userNode);
	InsertNode(userNode, schutzzielNode);
	InsertNode(userNode, mechanismNode);
	tree.setCellRenderer(new StateTreeCellRenderer());
	JScrollPane sp = new JScrollPane();
	sp.setPreferredSize(new Dimension(400, 400));
	sp.getViewport().add(tree);
	panel.setLayout(new BorderLayout());
	panel.add("Center", sp);
	frame.addWindowListener(new WindowAdapter() {
	    public void windowClosing(WindowEvent e) {
		System.exit(0);
	    }
	});
	frame.pack();
	frame.show();
    }
    public void append(int art, int ID, String treeName, String string) {
	switch (art) {
	case StateTreeConstants.INFORMATION :
	    String str = getDate() + string;
	    if (infoNode.getChildCount() == 0)
		info.setIcon(StateTreeConstants.INFOICON);
	    InsertNode(infoNode, new DefaultMutableTreeNode(
		new TreeData(StateTreeConstants.INFOICON,
		StateTreeConstants.IMPORTANT, Color.black, str)));
	    tree.revalidate();
	    frame.repaint();
	    break;
	    //-----------------------------------------------------------------
	case StateTreeConstants.CONFLICT :
	    if (ID == StateTreeConstants.RESETCONFLICTS) {
		user.setIcon(StateTreeConstants.INFOICON);
		schutzziele.setIcon(StateTreeConstants.INFOICON);
		mechanism.setIcon(StateTreeConstants.INFOICON);
		if (CheckNodeName(mechanismNode, accountability.string)) {
		    accountability.setIcon(StateTreeConstants.INFOICON);
		    treeModel.removeNodeFromParent(accountabilityNode);
		    accountabilityNode = new DefaultMutableTreeNode(accountability);
		}
		if (CheckNodeName(mechanismNode, integrity.string)) {
		    integrity.setIcon(StateTreeConstants.INFOICON);
		    treeModel.removeNodeFromParent(integrityNode);
		    integrityNode = new DefaultMutableTreeNode(integrity);
		}
		if (CheckNodeName(mechanismNode, confidentaly.string)) {
		    confidentaly.setIcon(StateTreeConstants.INFOICON);
		    treeModel.removeNodeFromParent(confidentalyNode);
		    confidentalyNode = new DefaultMutableTreeNode(confidentaly);
		}
		break;
	    }
	    if (konflictNode.getChildCount() == 0)
		konflict.setIcon(StateTreeConstants.CONFLICTICON);
	    InsertNode(konflictNode, new DefaultMutableTreeNode(
		new TreeData(StateTreeConstants.CONFLICTICON,
		StateTreeConstants.VERYIMPORTANT, Color.black, string)));
	    int userCount = treeModel.getChildCount(userNode);
	    if (ID == StateTreeConstants.SZ) {
		for (int i = 0; i < userCount; i++) {
		    DefaultMutableTreeNode dm =
			(DefaultMutableTreeNode) treeModel.getChild(schutzzielNode, i);
		    TreeData td = (TreeData) dm.getUserObject();
		    if (treeName.equals(td.string)) {
			DefaultMutableTreeNode dm1 = (DefaultMutableTreeNode)
			    treeModel.getChild(dm, treeModel.getChildCount(dm) - 1);
			TreeData td1 = (TreeData) dm1.getUserObject();
			td.setIcon(StateTreeConstants.CONFLICTICON);
			td1.setIcon(StateTreeConstants.CONFLICTICON);
			schutzziele.setIcon(StateTreeConstants.CONFLICTICON);
		    }
		}
	    } else {
		switch (ID) //zum rot-faerben der Eintraege bei Konfilkt fuer Mechanismen
		{
		case SSONETConstants.CONFIDENTIALITY :
		    mechanism.setIcon(StateTreeConstants.CONFLICTICON);
		    int count = treeModel.getChildCount(confidentalyNode);
		    for (int i = 0; i < count; i++) {
			DefaultMutableTreeNode dm = (DefaultMutableTreeNode)
			    treeModel.getChild(confidentalyNode, i);
			TreeData td = (TreeData) dm.getUserObject();
			if (treeName.equals(td.string)) {
			    td.setIcon(StateTreeConstants.CONFLICTICON);
			    confidentaly.setIcon(StateTreeConstants.CONFLICTICON);
			}
		    }
		    break;
		case SSONETConstants.INTEGRITY :
		    mechanism.setIcon(StateTreeConstants.CONFLICTICON);
		    count = treeModel.getChildCount(integrityNode);
		    for (int i = 0; i < count; i++) {
			DefaultMutableTreeNode dm = (DefaultMutableTreeNode)
			    treeModel.getChild(integrityNode, i);
			TreeData td = (TreeData) dm.getUserObject();
			if (treeName.equals(td.string)) {
			    td.setIcon(StateTreeConstants.CONFLICTICON);
			    integrity.setIcon(StateTreeConstants.CONFLICTICON);
			}
		    }
		    break;
		case SSONETConstants.ACCOUNTABILITY_SIGN :
		    mechanism.setIcon(StateTreeConstants.CONFLICTICON);
		    count = treeModel.getChildCount(accountabilityNode);
		    for (int i = 0; i < count; i++) {
			DefaultMutableTreeNode dm = (DefaultMutableTreeNode)
			    treeModel.getChild(accountabilityNode, i);
			TreeData td = (TreeData) dm.getUserObject();
			if (treeName.equals(td.string)) {
			    td.setIcon(StateTreeConstants.CONFLICTICON);
			    accountability.setIcon(StateTreeConstants.CONFLICTICON);
			}
		    }
		    break;
		case SSONETConstants.ANONYMITY :
		    //not implemented in ssonet
		    break;
		default :
		    break;
		} //switch
	    }
	    TreeData td2 = (TreeData) userNode.getUserObject();
	    td2.setIcon(StateTreeConstants.CONFLICTICON);
	    tree.revalidate();
	    frame.repaint();
	    break;
	    //-----------------------------------------------------------------
	    //Benutzerinformationen wie Schutzziele, Mechanismen ....
	case StateTreeConstants.USER :
	    if (userNode.getChildCount() == 0) {
		user.setIcon(StateTreeConstants.INFOICON);
	    }
	    userCount = treeModel.getChildCount(schutzzielNode);
	    boolean flag = true, changed = false;
	    if (ID == StateTreeConstants.TCBEFORE || ID == StateTreeConstants.TCAFTER) {
		if (ID == StateTreeConstants.TCBEFORE)
		    user.setIcon(StateTreeConstants.INFOICON);
		for (int i = 0; i < userCount; i++) {
		    DefaultMutableTreeNode dm = (DefaultMutableTreeNode)
			treeModel.getChild(schutzzielNode, i);
		    TreeData td = (TreeData) dm.getUserObject();
		    if (treeName.equals(td.string))
			//Sucht ob Node schon vorh. (Vertr., Zurechenbk., Integr. ....)
		    { //diesen Knoten gibt es schon!
			flag = false;
			DefaultMutableTreeNode dm1 = (DefaultMutableTreeNode)
			    treeModel.getChild(dm, treeModel.getChildCount(dm) - 1);
			TreeData td1 = (TreeData) dm1.getUserObject();
			if (ID == StateTreeConstants.TCAFTER && !string.equals(td1.string))
			    //on no cond. ... don't care
			{ //Schutzziel geaendert
			    changed = true;
			    td.setIcon(StateTreeConstants.ATTENTIONICON);
			    td1.setIcon(StateTreeConstants.ATTENTIONICON);
			    td1.setString(string);
			    user.setIcon(StateTreeConstants.ATTENTIONICON);
			    schutzziele.setIcon(StateTreeConstants.ATTENTIONICON);
			} else { //Schutzziel geblieben oder TCBEFORE
			    td.setIcon(StateTreeConstants.INFOICON);
			    td1.setIcon(StateTreeConstants.INFOICON);
			    td1.setString(string);
			} //else
		    } //if
		} //for
		if (flag)
		    //true, wenn Node noch nicht vorhanden war!
		{
		    DefaultMutableTreeNode tmp = new DefaultMutableTreeNode(
			new TreeData(StateTreeConstants.INFOICON,
			StateTreeConstants.VERYIMPORTANT, Color.black, treeName));
		    InsertNode(schutzzielNode, tmp);
		    InsertNode(tmp, new DefaultMutableTreeNode(
			new TreeData(StateTreeConstants.INFOICON,
			StateTreeConstants.VERYIMPORTANT, Color.black, string)));
		} //if
	    } //if
	    else {
		switch (ID) {
		case SSONETConstants.CONFIDENTIALITY :
		    if (!CheckNodeName(mechanismNode, confidentaly))
			InsertNode(mechanismNode, confidentalyNode);
		    if (!CheckNodeName(confidentalyNode, treeName))
			InsertNode(confidentalyNode, new DefaultMutableTreeNode(
			new TreeData(StateTreeConstants.INFOICON,
			StateTreeConstants.VERYIMPORTANT, Color.black, treeName)));
		    break;
		case SSONETConstants.INTEGRITY :
		    if (!CheckNodeName(mechanismNode, integrity))
			InsertNode(mechanismNode, integrityNode);
		    if (!CheckNodeName(integrityNode, treeName))
			InsertNode(integrityNode, new DefaultMutableTreeNode(
			new TreeData(StateTreeConstants.INFOICON,
			StateTreeConstants.VERYIMPORTANT, Color.black, treeName)));
		    break;
		case SSONETConstants.ACCOUNTABILITY_SIGN :
		    if (!CheckNodeName(mechanismNode, accountability))
			InsertNode(mechanismNode, accountabilityNode);
		    if (!CheckNodeName(accountabilityNode, treeName))
			InsertNode(accountabilityNode, new DefaultMutableTreeNode(
			new TreeData(StateTreeConstants.INFOICON,
			StateTreeConstants.VERYIMPORTANT, Color.black, treeName)));
		    break;
		case SSONETConstants.ANONYMITY :
		    break;
		default :
		} //switch
	    } //else
	    tree.revalidate();
	    frame.repaint();
	    break;
	    //-----------------------------------------------------------------
	case StateTreeConstants.DEBUG :
	    if (debugNode.getChildCount() == 0)
		debug.setIcon(StateTreeConstants.DEBUGICON);
	    InsertNode(debugNode, new DefaultMutableTreeNode(
		new TreeData(StateTreeConstants.DEBUGICON,
		StateTreeConstants.LOWIMPORTANT, Color.black, string)));
	    tree.revalidate();
	    frame.repaint();
	default :
	}
    } //append

    /** Prft, ob ein Node mit
    * @ param name Name des Knotens(Blattes)
    * @ param der zu pruefende Knoten(Blatt)
    * @ return true -> node vorhanden, sonst false
    */
    private boolean CheckNodeName(DefaultMutableTreeNode parent, String name) {
	boolean flag = false;
	for (int i = 0; i < treeModel.getChildCount(parent); i++) {
	    DefaultMutableTreeNode tmp = (DefaultMutableTreeNode)
		treeModel.getChild(parent, i);
	    TreeData td1 = (TreeData) tmp.getUserObject();
	    if (name.equals(td1.string))
		flag = true;
	}
	return flag;
    }
    private boolean CheckNodeName(DefaultMutableTreeNode parent, TreeData td) {
	boolean flag = false;
	String str = td.string();
	for (int i = 0; i < treeModel.getChildCount(parent); i++) {
	    DefaultMutableTreeNode tmp = (DefaultMutableTreeNode)
		treeModel.getChild(parent, i);
	    TreeData td1 = (TreeData) tmp.getUserObject();
	    if (str.equals(td1.string))
		flag = true;
	}
	return flag;
    }
    /** Funktion erstellt ein Menu */
    private JMenuBar constructMenuBar() {
	JMenu menu, umenu;
	JMenuBar menuBar = new JMenuBar();
	JMenuItem menuItem;
	menu = new JMenu("Datei");
	menu.setMnemonic('D');
	menuBar.add(menu);
	menuItem = menu.add(new JMenuItem("Exit"));
	menuItem.setMnemonic('E');
	menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		System.exit(0);
	    }
	});

	/* Tree related stuff. */
	menu = new JMenu("Optionen");
	menu.setMnemonic('O');
	menuBar.add(menu);
	menuItem = menu.add(new JMenuItem("Anzeige"));
	menuItem.setMnemonic('A');
	menuItem.addActionListener(new Display());
	umenu = new JMenu("Lschen");
	umenu.setMnemonic('L');
	menuItem = umenu.add(new JMenuItem("Protokoll"));
	menuItem.setMnemonic('P');
	menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		treeModel.removeNodeFromParent(infoNode);
		info.setIcon(StateTreeConstants.GRAYICON);
		infoNode = new DefaultMutableTreeNode(info);
		treeModel.insertNodeInto(infoNode, root, 0);
	    }
	});
	menuItem = umenu.add(new JMenuItem("Konflikte"));
	menuItem.setMnemonic('k');
	menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		treeModel.removeNodeFromParent(konflictNode);
		konflict.setIcon(StateTreeConstants.GRAYICON);
		konflictNode = new DefaultMutableTreeNode(konflict);
		treeModel.insertNodeInto(konflictNode, root, 2);
	    }
	});
	menuItem = umenu.add(new JMenuItem("Debug"));
	menuItem.setMnemonic('b');
	menuItem.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e) {
		treeModel.removeNodeFromParent(debugNode);
		debug.setIcon(StateTreeConstants.GRAYICON);
		debugNode = new DefaultMutableTreeNode(debug);
		treeModel.insertNodeInto(debugNode, root, 1);
	    }
	});
	menu.add(umenu);
	umenu = new JMenu("Sprache");
	umenu.setEnabled(false); //noch nicht impementiert
	menuItem = umenu.add(new JCheckBoxMenuItem("Deutsch"));
	menuItem = umenu.add(new JCheckBoxMenuItem("Englisch"));
	menu.add(umenu);
	return menuBar;
    }
    private String getDate() {
	String str = "";
	Calendar date = new GregorianCalendar();
	if (date.get(Calendar.DAY_OF_MONTH) != dayOfBegin)
	    //Neues Datum ... jetzt anzeigen
	{
	    String tmp = String.valueOf(date.get(Calendar.DAY_OF_MONTH));
	    if (tmp.length() == 1)
		tmp = "0" + tmp;
	    str = tmp + ".";
	    tmp = String.valueOf(date.get(Calendar.MONTH));
	    if (tmp.length() == 1)
		tmp = "0" + tmp;
	    str += tmp + "." + date.get(Calendar.YEAR) + " ";
	}
	String tmp = String.valueOf(date.get(Calendar.HOUR_OF_DAY));
	if (tmp.length() == 1)
	    tmp = "0" + tmp;
	str += tmp + ":";
	tmp = String.valueOf(date.get(Calendar.MINUTE));
	if (tmp.length() == 1)
	    tmp = "0" + tmp;
	str += tmp + ":";
	tmp = String.valueOf(date.get(Calendar.SECOND));
	if (tmp.length() == 1)
	    tmp = "0" + tmp;
	str += tmp + "  ";
	return str;
    }
    private void InsertNode(DefaultMutableTreeNode parent, DefaultMutableTreeNode node) {
	int newIndex;
	if (parent == null) {
	    parent = (DefaultMutableTreeNode) treeModel.getRoot();
	}
	newIndex = treeModel.getChildCount(parent);
	TreeData td = (TreeData) node.getUserObject();
	treeModel.insertNodeInto(node, parent, newIndex);
	treeModel.reload((TreeNode) parent);
    }
    public static void main(String[] args) {
	DatagramSocket socket = null;
	try {
	    StateTree st = new StateTree();
	    socket = new DatagramSocket(st.PORT);
	    while (true) {
		DatagramPacket packet = new DatagramPacket(new byte[200], 200);
		socket.receive(packet);
		byte[] buff = packet.getData();
		int ID = (int) buff[0]; //in buff[0] ID fuer zurech, intergr., anonym., confidenc.
		int art = (int) buff[1]; //				fuer INFO, USER, DEBUG ....
		st.importance = (int) buff[2]; // Wichtigkeit der Daten zum evtl. Ausblenden
		String str1 = new String(buff, 4, (int) buff[3]); //in buff[1] Lnge des 1. Strings
		String str2 = new String(buff, (int) buff[3] + 4, (packet.getLength() -
		    (int) buff[3] - 4));
		st.append(art, ID, str1, str2);
	    }
	} catch (Exception e) {
	    System.out.println("Error: " + e.getMessage() + "\n");
	    e.printStackTrace();
	    System.exit(0);
	}
    } //main
    private void RemoveNode(DefaultMutableTreeNode dmtn) {
	dmtn.removeAllChildren();
	root.remove(dmtn);
    }
    private void RevalidateTree(int imp) {
	if (imp != importance) {
	    switch (imp) {
	    case (StateTreeConstants.LOWIMPORTANT) :
		if (!CheckNodeName(root, info.string))
		    treeModel.insertNodeInto(infoNode, root, 0);
		if (!CheckNodeName(root, debug.string))
		    treeModel.insertNodeInto(debugNode, root, 1);
		break;
	    case (StateTreeConstants.IMPORTANT) :
		if (CheckNodeName(root, debug.string))
		    treeModel.removeNodeFromParent(debugNode);
		if (!CheckNodeName(root, info.string))
		    treeModel.insertNodeInto(infoNode, root, 0);
		break;
	    case (StateTreeConstants.VERYIMPORTANT) :
		if (CheckNodeName(root, info.string))
		    treeModel.removeNodeFromParent(infoNode);
		if (CheckNodeName(root, debug.string))
		    treeModel.removeNodeFromParent(debugNode);
		break;
	    default :
		break;
	    }
	    tree.revalidate();
	    frame.repaint();
	    importance = imp;
	}
    }
}
